/*
   This file is part of the MoCA video library.
   Name: Avi2Png.cpp
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Stephan Kopf

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kopf@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt


*/

// C++ includes
#include <iostream>
#include <sstream>
using namespace std;


// MoCA includes
#include <Image.h>
#include <OImageStream.h>
#include <VideoDecoder.h>
#include <ScaleFilter.h>

void options()
{
	cerr << "Usage: Avi2Png  [options]  src(avi|mpg)" << endl;
	cerr << "Version 0.35" << endl;
	cerr << endl;
	cerr << "options:" << endl;
	cerr << "  -newdir         create new directory for 1K images" << endl;
	cerr << "  -scale          reduce the size by factor of two" << endl;
	cerr << "  -skip <number>  skip number of images, e.g. 'skip 1' stores every second frame" << endl;
	cerr << endl;
	exit(1);
	return;
}


void getOptions(int argc, char* argv[], string &fn, bool &newdir, bool &scale, int32 &skip)
{
	newdir=false;
	scale=false;
	skip=0;

	if ((argc<2) || (argc>6)) options();

	fn = argv[argc-1];
	if (argc==2) return;

	for (int32 i=1; i<argc-1; i++) {
		string s = argv[i];
		FileHandling::upper(s);
		if (s == "-NEWDIR") {
			newdir=true;
		} else if (s == "-SKIP") {
			scale=true;
			i++;
			s=argv[i];
			skip = atoi(s.c_str());
		} else if (s == "-SCALE") {
			scale=true;
		} else {
			cerr << "error: unknown option ("<< s << ")." << endl << endl;
			options();
		}
	} // if

	return;
}

int main(int argc, char* argv[])
{
	string outPath = "./";
	string aviFileName = "";
	bool newdir, scale;
	int32 skip;
	getOptions(argc, argv, aviFileName, newdir, scale, skip);
	skip++;


	try {
		VideoDecoder reader (aviFileName);
		VideoSize size = reader.getSize();

		ScaleFilter<uint8> sf;

		string path = outPath;
		cerr << "read and save images (" << 0 << "..." << size[DIM_Z]-1 << ")..." << endl << endl;
		Image<uint8> img;
		for (int i=0; i<size[DIM_Z]; i++) {
			// modify path
			if (newdir) {
				if (i%1000 == 0) {
					string s = FileHandling::getNumberedFilename("",i/1000,"");
					path = outPath + s + "/";
					FileHandling::createDirectory (path, false);
				} // if
			} // if
			if (!(i % skip)) {
				cerr << "\rframe: " << i << "   " << ends;
				img = reader.getImage(i);
				
				
				// scale images
				if (scale) sf.reduce(img,2,2);
				
				// save images
				string fn = FileHandling::getNumberedFilename (path + "img",i,".png");
				WriteImage (img, fn);
			} // if
		} // for
    } catch (MocaException e) {
        cerr << "error: could not save images." << endl;
        cerr << e.what() << endl;
		options();
        exit (-1);
    } // try
	cerr << endl << endl << endl;
	return 0;
} // main



