/*
   This file is part of the MoCA video library.
   Name: CannyFilterTest.cpp
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

   \author    Stephan Kopf

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kopf@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

*/

#include <iostream>
#include <Image.h>
#include <IImageStream.h>
#include <OImageStream.h>
#include <MocaExceptions.h>
#include <CannyFilter.h>

#include <CropFilter.h>
#include <BinarizeFilter.h>

int main(int argc, char* argv[])
{
    if (argc != 3) {
        cerr << "Usage:   CannyFilterTest src dest " << endl;
        cerr << "example: CannyFilterTest test.png result.png " << endl;
        return 1;
    } // if

    Image<uint8> img;

    try {
        // open input filestream and read image
        IImageStream<uint8> iis(argv[1]);
        iis.read(img);
    } catch (MocaException e) {
        cout << "IImageStream " << e.what() << endl;
        exit (-1);
    } // try
    
    // filter
    CannyFilter<uint8> filter;
    filter.process(img);
    
    try {
        // open output filestream and write image
        OImageStream<uint8> ois(argv[2]);
        ois.write(img);
    } catch (MocaException e) {
        cout << "IImageStream " << e.what() << endl;
        exit (-1);
    } // try
    return 0;
} // main

