/*
   This file is part of the MoCA video library.
   Name: ConvolutionTest.cpp
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

   \author    Stephan Kopf

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kopf@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt


   ConvolutionSepSym mask (old Moca-Bib)          ConvolutionFilter mask (new Moca-Bib)       
   example:               0.8, 1.2          =>         1.2, 0.8, 1.2
                          0.6, 0.8, 1.2     =>    1.2, 0.8, 0.6, 0.8, 1.2
                          0.4, 0.2, 0.1     =>    0.1, 0.2, 0.4, 0.2, 0.1

   ConvolutionSepASym mask (old Moca-Bib)         ConvolutionFilter mask (new Moca-Bib)       
   example:               0.48              =>         0.48, 0, -0.48         (offset should be set to 127 in uint8 images)
                          0.40, 0.7         =>    0.7, 0.40, 0, -0.40, -0.7    

   Offset should be set to 127 in uint8 images! (process(src,dest,offset))
   ScalingFactor should be defined in uint8 images! (ConvolutionFilter (mask,DIR_X,true, ScalingFactor))
*/

#include <iostream>
#include <Image.h>
#include <MocaSignal.h>
#include <IImageStream.h>
#include <OImageStream.h>
#include <ConvolutionFilter.h>
#include <MocaExceptions.h>

int main(int argc, char* argv[])
{
    if (argc < 4) {
        cerr << "Usage:   ConvolutionTest src dest maskdata" << endl;
        cerr << "example: ConvolutionTest test.png result.png 1 2 4 2 1" << endl;
        return 1;
    } // if

    // initialize mask
    int maskSize = argc - 3;
    Signal<ieee64> maskData;
    maskData.realloc (SignalSize (maskSize), SignalSize (0), BORDER_REPLICATE);
    int i;
    for (i=3; i<argc; i++) {
        maskData(i-3) = (ieee64) atof(argv[i]);
    } // for    

    // load image
    Image<uint8> img;
    try {
        // open input filestream and read image
        IImageStream<uint8> iis(argv[1]);
        iis.read(img);
    } catch (MocaException e) {
        cout << "IImageStream " << e.what() << endl;
        exit (-1);
    } // try

    // filter
    ConvolutionFilter<uint8> cf (maskData, DIR_Y, 1.0);
    int offset = 0;
    cf.process(img, offset);

    // save image
    try {
        // open output filestream and write image
        OImageStream<uint8> ois(argv[2]);
        ois.write(img);
    } catch (MocaException e) {
        cout << "IImageStream " << e.what() << endl;
        exit (-1);
    } // try

    return 0;
} // main


