/*
   This file is part of the MoCA video library.
   Name: Draw.cpp
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Stephan Kopf

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kopf@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt
*/

#include <iostream>
#include <Image.h>
#include <IImageStream.h>
#include <OImageStream.h>
#include <DrawDot.h>
#include <DrawLine.h>
#include <DrawCross.h>
#include <DrawIcon.h>
#include <DrawCircle.h>
#include <DrawRectangle.h>
#include <MocaExceptions.h>
#include <ImagePosition.h>


int main(int argc, char* argv[])
{
    if (argc != 3) {
        cerr << "Usage: Draw src(pnm) dest(pnm)" << endl;
        return 1;
    } // if

    Image<uint8> img;

    try {
        // open input filestream and read image
        IImageStream<uint8> iis(argv[1]);
        iis.read(img);
    } catch (MocaException e) {
        cout << "IImageStream " << e.what() << endl;
        exit (-1);
    } // try
    
    {
        DrawDot dot (4);
        ImagePosition p0(8,8);
        dot.process (p0, Color::GRAY, img);
    }
    
    {
        DrawLine line;
        ImagePosition p0(40,20);
        ImagePosition p1(60,40);
        line.process (p0, p1, Color::GRAY, img);
    }
    
    {
        Color c = Color::BLACK;
        DrawRectangle rect (c);
        ImagePosition p0 (10,25);
        ImagePosition p1(40,55);
        rect.process (p0, p1, img);
    }
    
    {
        DrawIcon icon (Color::RED);
        ImagePosition p0(30,30);
        icon.process (p0, ICON_CAR, img);        
    }

    {
        const int32 lineLength = 10;
        const Color c = Color::YELLOW;
        DrawCross cross (lineLength, c);        
        ImagePosition p0(50,50);
        cross.process (p0, img);        
    }
    {
        const Color c = Color::GREEN;
        int radius = 33;
        DrawCircle circle (c);        
        ImagePosition p0(25,25);
        circle.process (p0, radius, img);        
    }


    
    try {
        // open output filestream and write image
        OImageStream<uint8> ois(argv[2]);
        ois.write(img);
    } catch (MocaException e) {
        cout << "OImageStream " << e.what() << endl;
        exit (-1);
    } // try

    return 0;
} // main



