/*
   This file is part of the MoCA video library.
   Name: WaveAudioTest.cpp
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

 
  
   \author    Stephan Kopf
   
	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de
	
	 (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt
	 
*/

// C++ includes
#include <fstream>
#include <iostream>
using namespace std;

// MoCA includes
#include <MocaSignal.h>
#include <OWaveAudio.h>
#include <IWaveAudio.h>


int main(int argc, char* argv[])
{

// read Wave file

    if (argc != 5) {
        cerr << "read and write WAVE files " << endl << endl;
        cerr << "Usage: " << endl;
		cerr << argv[0] << " src(wav)  dest1(wav) dest2(wav) dest3(wav) " << endl;
        return 1;
    } // if
	string srcFilename  =  argv[1];
	string destFilename1 = argv[2];
	string destFilename2 = argv[3];
	string destFilename3 = argv[4];

	try {
		IWaveAudio wav (srcFilename);
		WaveHeader header;
		header = wav.getHeader ();
		SignalSize size;
		size = wav.getSize ();


		// (1) read all data to signal
		cout << "read wav file..." << endl;
		Signal<int16> sig1;
		wav.read (sig1, -1);

		// write wave file using signal structure
		cout << "save wav file..." << endl;
		OWaveAudio out1 (
			destFilename1, 
			header.getChannels(), header.getSamplesPerSec(), 
			header.getBitsPerSample(), size[DIM_X]);
		out1.write(sig1);
		cout << endl << endl;


		// (2) read 80000 samples to signal
		cout << "read wav file..." << endl;
		int32 samples = 80000;
		int32 offsetSamples = 100;
		Signal<int16> sig2;
		cout << "offset before reset  : " << wav.getOffset() << endl;
		wav.reset();
		cout << "offset before reading: " << wav.getOffset() << endl;
		wav.read (sig2, samples, offsetSamples);
		cout << "offset after reading : " << wav.getOffset() << endl;
		wav.read (sig2, samples, offsetSamples);
		cout << "offset after reading : " << wav.getOffset() << endl;

		// write wave file using signal structure
		cout << "save wav file..." << endl;
		OWaveAudio out2 (
			destFilename2, 
			header.getChannels(), header.getSamplesPerSec(), 
			header.getBitsPerSample(), sig2.getSize()[DIM_X]);
		out2.write(sig2);
		cout << endl << endl;


		// (3) read data using file handler
		cout << "read wav file..." << endl;

		// read raw data without header
		int32 offsetSamples2 = 100;
		FILE *fh = wav.getFileHandle (offsetSamples2);

		cout << "save wav file..." << endl;
		OWaveAudio out3 (
			destFilename3, 
			header.getChannels(), header.getSamplesPerSec(), 
			header.getBitsPerSample(), size[DIM_X] - offsetSamples2);
		out3.write(fh);





    } catch (MocaException e) {
        cerr << "Mpeg2AudioTest  could not save mp2 file." << endl;
        cerr << e.what() << endl;
        exit (-1);
    } // try
	cout << endl;

	return 0;
} // main

