/*
   This file is part of the MoCA video library.
   Name: ActionDetection.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

 
  
   \author    Stephan Kopf
   
	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de
	
	 (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt
	 
*/


#ifndef ACTION_DETECTION_H
#define ACTION_DETECTION_H

// C includes
#include <math.h>

// C++ includes
#include <iomanip>
#include <sstream>
#include <iostream>
using namespace std;


// MoCA includes
#include <MocaSignal.h>
#include <WriteMocaData.h>
#include <HistogramDifferenceFeature.h>
#include <VideoStatistic.h>
#include <GaussFilter.h>
#include <CannyFilter.h>
#include <BinarizeFilter.h>
#include <EcrFeature.h>
#include <FileHandling.h>
#include <MotionVector.h>


// windows includes
#ifdef COMPILER_MSVC
#include <process.h>
#endif


const static string ACTION_DETECTION_VERSION  ="Version-0.06";
const static string ACTION_DETECTION_FILENAME ="action.idx";


struct ActionData
{
	ActionData () 
	{
		version = ACTION_DETECTION_VERSION;
		this->filename = "";;
		numberOfFrames = 0;
		SignalSize size (1);
		imgAction.resize(size);
		motionAction.resize(size);
		avgImgAction=0.0;
		avgMotionAction=0.0;
	} 
    ActionData& operator=(const ActionData &other) {
		if (this != &other) {
			version         = other.version;
			filename        = other.filename;
			numberOfFrames  = other.numberOfFrames;
			imgAction       = other.imgAction;
			motionAction    = other.motionAction;
			avgImgAction    = other.avgImgAction;
			avgMotionAction = other.avgMotionAction;
		} // if
        return *this;
    } // operator=

	string version;
	string filename;
	int32  numberOfFrames;
	ieee64 avgImgAction;
	ieee64 avgMotionAction;

	Signal<ieee64> imgAction;
	Signal<ieee64> motionAction;
};

std::ostream& operator<< (ostream &, const ActionData&);
OTagStream& operator<< (OTagStream&, const ActionData&);
ITagStream& operator>> (ITagStream&, ActionData&);





/*
	Constructor: initializes variables
	
	process(): 
		if status file does not exist:
		   calculate the action in mpeg-file, save status file
		else:
		   read action data from status file
*/
class ActionDetection
{
public:

	ActionDetection(const string &fullName);
	~ActionDetection() {};

	void process (ActionData&);

private:
	
	void readValues (ActionData& scd);

	string pathFilename;
	//string filename;
};

#endif
