/*
   This file is part of the MoCA video library.
   Name: CSS_DB_Schema.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

Author:    Stephan Kopf
Lehrstuhl fuer Praktische Informatik IV
Universitaet Mannheim, Germany
email: kopf@pi4.informatik.uni-mannheim.de
*/ 


// CSS database schema
// use this class to open an existing database with CSS-based contour descriptions of objects


#ifndef CSS_DATABASE_SCHEMA_H
#define CSS_DATABASE_SCHEMA_H

struct CssDatabasePeak 
{ 
    int16 position; 
    int16 height; 
    int16 width; 
    uint8 isConcave; 
}; 

struct CssDatabaseItem 
{ 
    char *objFilename;		// file of tag-file 
    int16 objectClassId;        // id of the object class  
    ieee64 xyRatio;		// xyRatio of object 
    CssDatabasePeak peak[10];	// peaks
}; 


struct CssStoredDatabaseItem
{
    CssStoredDatabaseItem(CssDatabaseItem &item)
    {
	this->objFilename   = item.objFilename;
	this->objectClassId = item.objectClassId;
	data.xyRatio        = item.xyRatio;
	data.data.clear();
	for (int32 i=0; i<10; i++) {
	    CssMaximumItem cssItem;
	    cssItem.position   = item.peak[i].position;
	    cssItem.height     = item.peak[i].height;
	    cssItem.width      = item.peak[i].width;
	    cssItem.isConcave  = item.peak[i].isConcave;
	    cssItem.curveStart = 0;
	    cssItem.curveEnd   = 0;
	    data.data.push_back(cssItem);
	} // for
    }
    
    CssMaximumData data;
    string objFilename;         // file of tag-file 
    int16 objectClassId;        // id of the object class  
};



// automatically generated files 
// include different object databases
#include<CSS_DB_Echo.h>


enum CssDatabaseSchemaType {ECHO_DB_TYPE=0};
class CssDatabaseSchema
{
public:

    // initialize database
    CssDatabaseSchema (CssDatabaseSchemaType schema)
    {
	objectClassSize = 0;
	objectClasses   = NULL;
	dbSize          = 0;
	db              = NULL;

	switch (schema) {
	case ECHO_DB_TYPE:
	    objectClassSize = ECHO_OBJECT_CLASSES_SIZE;
	    objectClasses   = ECHO_OBJECT_CLASSES;
	    dbSize          = ECHO_DB_SIZE;
	    db              = ECHO_DB;
	    break;
	} // switch

	cssData.clear();
	objectClassName.clear();
    }

    void getCssStoredDatabaseData (vector<CssStoredDatabaseItem> &cssData, vector<string> &objectClassName)
    {
	openDatabase();

	cssData = this->cssData;
	objectClassName = this->objectClassName;
	return;
    }

private:

    void openDatabase ()
    {	    
	cssData.clear();
	objectClassName.clear();

	for (int32 i=0; i<dbSize; i++) {
	    CssDatabaseItem item = db[i];
	    
	    CssStoredDatabaseItem storeItem (item);
	    cssData.push_back(storeItem);
	} // for
	

	for (int32 i=0; i<objectClassSize; i++) {
	    string s = objectClasses[i];
	    objectClassName.push_back(s);
	} // for
	
	return;
    } // openDatabase



    int32 objectClassSize;
    char** objectClasses;
    int32 dbSize;
    CssDatabaseItem *db;

    vector<CssStoredDatabaseItem> cssData;
    vector<string> objectClassName;
};


#endif


