/*
    CameraOperation.cpp     calculate the camera operation in videos
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// MoCA includes
#include <CameraOperation.h>



std::ostream& operator<< (ostream& os, const CameraOperationItem &item)
{
	
	switch (item.cot) {
	case CAMERA_OPERATION_UNKNOWN:
		os << "camera operation: UNKNOWN" << endl;
		break;		
	case CAMERA_OPERATION_STATIC:
		os << "camera operation: static" << endl;
		break;		
	case CAMERA_OPERATION_ZOOM_IN:
		os << "camera operation: zoom in" << endl;
		break;		
	case CAMERA_OPERATION_ZOOM_OUT: 
		os << "camera operation: zoom out" << endl;
		break;		
	case CAMERA_OPERATION_PAN_LEFT: 
		os << "camera operation: pan left" << endl;
		break;		
	case CAMERA_OPERATION_PAN_RIGHT: 
		os << "camera operation: pan right" << endl;
		break;		
	case CAMERA_OPERATION_TILT_UP:
		os << "camera operation: tilt up" << endl;
		break;		
	case CAMERA_OPERATION_TILT_DOWN:
		os << "camera operation: tilt down" << endl;
		break;		
	case CAMERA_OPERATION_ROTATE_LEFT:
		os << "camera operation: rotate left" << endl;
		break;		
	case CAMERA_OPERATION_ROTATE_RIGHT:
		os << "camera operation: rotate right" << endl;
		break;		
	default : 
		os << "camera operation: UNKNOWN" << endl;
		break;		
	} // switch
	
	os << "first frame     : " << item.firstFrame << endl;
	os << "last frame      : " << item.lastFrame << endl;
	os << "length          : " << item.length<< endl;
	os << "strength        : " << item.strength<< endl;
	os << "parameters      : ";
	os << item.trans << endl;

	return os;
} // operator<<

OTagStream& operator<< (OTagStream& os, const CameraOperationItem& item)
{
	string c = "UNKNOWN";
	switch (item.cot) {
	case CAMERA_OPERATION_STATIC:
		c = "STATIC";
		break;		
	case CAMERA_OPERATION_ZOOM_IN:
		c = "ZOOM_IN";
		break;		
	case CAMERA_OPERATION_ZOOM_OUT: 
		c = "ZOOM_OUT";
		break;		
	case CAMERA_OPERATION_PAN_LEFT: 
		c = "PAN_LEFT";
		break;		
	case CAMERA_OPERATION_PAN_RIGHT: 
		c = "PAN_RIGHT";
		break;		
	case CAMERA_OPERATION_TILT_UP:
		c = "TILT_UP";
		break;		
	case CAMERA_OPERATION_TILT_DOWN:
		c = "TILT_DOWN";
		break;		
	case CAMERA_OPERATION_ROTATE_LEFT:
		c = "ROTALE_LEFT";
		break;		
	case CAMERA_OPERATION_ROTATE_RIGHT:
		c = "ROTALE_RIGHT";
		break;		
	default:
		c="UNKNOWN";
	} // switch
    TagFormat::writeNodeValue(os, "CameraOperationType", c);

    TagFormat::writeNodeValue(os, "firstFrame", item.firstFrame);
    TagFormat::writeNodeValue(os, "lastFrame", item.lastFrame);
    TagFormat::writeNodeValue(os, "length", item.length);
    TagFormat::writeNodeValue(os, "strength", item.strength);

	os << item.trans << endl;

	
    return os;
} // operator<<

ITagStream& operator>> (ITagStream& is, CameraOperationItem& item)
{
    char c;
    bool failFlag = false;
	
	if (is >> c) {
		is.putback(c);
		
		string s = "";
		TagFormat::readNodeValue(is, "CameraOperationType", s);

		item.cot = CAMERA_OPERATION_UNKNOWN;
		if (s == "STATIC") {
			item.cot = CAMERA_OPERATION_STATIC;
		} else if (s == "ZOOM_IN") {
			item.cot = CAMERA_OPERATION_ZOOM_IN;
		} else if (s == "ZOOM_OUT") {
			item.cot = CAMERA_OPERATION_ZOOM_OUT;
		} else if (s == "PAN_LEFT") {
			item.cot = CAMERA_OPERATION_PAN_LEFT;
		} else if (s == "PAN_RIGHT") {
			item.cot = CAMERA_OPERATION_PAN_RIGHT;
		} else if (s == "TILT_UP") {
			item.cot = CAMERA_OPERATION_TILT_UP;
		} else if (s == "TILT_DOWN") {
			item.cot = CAMERA_OPERATION_TILT_DOWN;
		} else if (s == "ROTATE_LEFT") {
			item.cot = CAMERA_OPERATION_ROTATE_LEFT;
		} else if (s == "ROTATE_RIGHT") {
			item.cot = CAMERA_OPERATION_ROTATE_RIGHT;
		} // if

		TagFormat::readNodeValue(is, "firstFrame", item.firstFrame);
		TagFormat::readNodeValue(is, "lastFrame", item.lastFrame);
		TagFormat::readNodeValue(is, "length", item.length);
		TagFormat::readNodeValue(is, "strength", item.strength);

		is >> item.trans;
		
	} // if
		
	if (is.rdstate() & ios::failbit) {
		failFlag = true;
	} // if
	if (failFlag) {
		throw MocaException ("CameraOperation: Illegal file format!");
	} else {
		is.clear();
	} // else
	
	return is;
} // operator>> 




std::ostream& operator<< (ostream& os, const CameraOperationData &cod)
{
	os << "version : " << cod.version << endl;
	os << "filename: " << cod.filename<< endl;

	int32 n = cod.data.size();
	os << "datasize: " << n << endl;
	
	for (int32 i=0; i<n; i++) {
		os << cod.data[i] << endl;
	} // for
	
	return os;
} // operator<<

OTagStream& operator<< (OTagStream& os, const CameraOperationData& cod)
{
	TagFormat::writeNodeValue(os, "version", cod.version);
	TagFormat::writeNodeValue(os, "filename", cod.filename);
	int32 n = cod.data.size();
	TagFormat::writeNodeValue(os, "datasize", n);
	os << endl;

	for (int32 i=0; i<n; i++) {
		os << cod.data[i] << endl;
	} // for
	
	return os;
} // operator<<

ITagStream& operator>> (ITagStream& is, CameraOperationData& cod)
{
	char c;
	bool failFlag = false;
	
	if (is >> c) {
		is.putback(c);
		
		TagFormat::readNodeValue(is, "version", cod.version);
		TagFormat::readNodeValue(is, "filename", cod.filename);
		TagFormat::readNodeValue(is, "datasize", cod.dataSize);

		if (is.rdstate() & ios::failbit) {
			failFlag = true;
		} // if
		
		cod.data.clear();

		int32 cnt = 0;
		CameraOperationItem item;
		while (is >> c) {
			is.putback(c);
			if (is >> item) {
				cod.data.push_back (item);
				cnt++;
			} else {
				failFlag = true;
			} // else
		} // while
		if (cnt != cod.dataSize) {
			failFlag=true;
			cerr << "illegal datasize value!" << endl;
		} // if
	} // if
	
	if (failFlag) {
		throw MocaException ("CameraOperation: Illegal file format!");
	} else {
		is.clear();
	} // else
	
	return is;
} // operator>> 



CameraOperation::CameraOperation(const string &pathFilename) 
{
	this->pathFilename = pathFilename;
} 




void 
CameraOperation::process(CameraOperationData& data) 
{

	// read data
	ReadMocaData<CameraOperationData> reader (pathFilename, CAMERA_OPERATION_FILENAME);
	bool dataValid = reader.process (CAMERA_OPERATION_VERSION, data); 

	if (!dataValid) {

		// create new index
		cerr << "CameraOperation: Creating index file ..." << endl;
		readValues(data);
			
		// save index file
		WriteMocaData<CameraOperationData> writer (pathFilename, CAMERA_OPERATION_FILENAME);
		writer.process(data);
		cerr << "CameraOperation: Creating index file ...done" << endl;
	} // if

	return;
} // process


void 
CameraOperation::readValues (CameraOperationData& cod) 
{
	cod.data.clear();
	cod.dataSize = 0;

	CameraModelData data;
	CameraModel cm (pathFilename);
	cm.process (data);

	detectStatic (data, cod);
	detectPan    (data, cod);
	detectTilt   (data, cod);
	detectZoom   (data, cod);

	// get path of filename, do not create new directory
	CreateMocaPath mocaPath (pathFilename, false);
	cod.filename = mocaPath.file;
	
	cod.dataSize = cod.data.size();
	cod.version = CAMERA_OPERATION_VERSION;

	
	return;
} // readValues


void 
CameraOperation::smoothValues (const Signal<ieee64> &v0, Signal<ieee64> &v1, ieee64 th, ieee64 avg)
{

	MedianFilter<ieee64> med (3);
	med.process (v0, v1);
	med.process (v1);

	int32 n = v0.getSize()[DIM_X];
	ieee64 thLow = avg-th;
	ieee64 thHigh = avg+th;
	for (int32 i=0; i<n; i++) {
		if (v1(i)>thLow && v1(i)<thHigh) v1(i) = avg;
	} // for

	GaussFilter<ieee64> gauss3 (3);
	gauss3.process (v1);

	for (int32 i=0; i<n; i++) {
		if (v1(i)>thLow && v1(i)<thHigh) v1(i) = avg;
	} // for

	return;
}

void 
CameraOperation::getAverage (const CameraModelData &cm, TransformationData& t, int32 left, int32 right)
{
	t.a00 = 0;
	t.a01 = 0;
	t.a10 = 0;
	t.a11 = 0;
	t.tx  = 0;
	t.ty  = 0;
	t.px  = 0;
	t.py  = 0;

	ieee64 cnt = 0.0;
	for (int32 j=left; j<=right; j++) {
		
		TransformationData t1;
		if (cm.data[j].isValid()) {
			t1 = cm.data[j].transData;
		}
		t.a00+=t1.a00;
		t.a01+=t1.a01;
		t.a10+=t1.a10;
		t.a11+=t1.a11;
		t.tx +=t1.tx;
		t.ty +=t1.ty;
		t.px +=t1.px;
		t.py +=t1.py;
		cnt++;
	} // for
	t.a00 /= cnt;
	t.a01 /= cnt;
	t.a10 /= cnt;
	t.a11 /= cnt;
	t.tx  /= cnt;
	t.ty  /= cnt;
	t.px  /= cnt;
	t.py  /= cnt;
				
	return;
}


void 
CameraOperation::detectPan (const CameraModelData &cm, CameraOperationData& op)
{
	int32 n = cm.data.size();

	// get pan (tx):
	SignalSize s(n);
	Signal<ieee64> p0 (s);
	for (int32 i=0; i<n; i++) {
		TransformationData trans;
		if (cm.data[i].isValid()) {
			trans = cm.data[i].transData;
		}
		p0(i) = trans.tx;
	} // for

	Signal<ieee64> p1;
	smoothValues (p0, p1, 0.2, 0.0);

	int32 thWidth = 37; // 37 frame (1.5 seconds)
	int32 thValue = 50;


	// get left pan (p1 > 0)
	int32 i=0;
	while (i<n) {
		while (i<n && p1(i)<=0) i++;
		int32 left=i;
		while (i<n && p1(i)>0) i++;
		int32 right=i-1;

		int32 width = right-left+1;
		if (width>thWidth) {

			ieee64 value=0;
			for (int32 j=left; j<=right; j++) {
				value += p1(j);
			} // for

			if (value > thValue) {

				TransformationData t;
				getAverage (cm, t, left, right);

				
				CameraOperationItem item;
				item.cot = CAMERA_OPERATION_PAN_LEFT;
				item.firstFrame = left;
				item.lastFrame  = right;
				item.length     = width;
				item.strength   = value;
				item.trans      = t;

				op.data.push_back(item);
			} // if
		} // if
	} // while

	// get right pan (p1 < 0)
	i=0;
	while (i<n) {
		while (i<n && p1(i)>=0) i++;
		int32 left=i;
		while (i<n && p1(i)<0) i++;
		int32 right=i-1;

		int32 width = right-left+1;
		if (width>thWidth) {

			ieee64 value=0;
			for (int32 j=left; j<=right; j++) {
				value += p1(j);
			} // for

			if (value < -thValue) {

				TransformationData t;
				getAverage (cm, t, left, right);

				
				CameraOperationItem item;
				item.cot = CAMERA_OPERATION_PAN_RIGHT;
				item.firstFrame = left;
				item.lastFrame  = right;
				item.length     = width;
				item.strength   = value;
				item.trans      = t;

				op.data.push_back(item);
			} // if
		} // if
	} // while
	
	return;
}



void 
CameraOperation::detectTilt (const CameraModelData &cm, CameraOperationData& op)
{
	int32 n = cm.data.size();

	// get pan (tx):
	SignalSize s(n);
	Signal<ieee64> p0 (s);
	for (int32 i=0; i<n; i++) {
		TransformationData trans;
		if (cm.data[i].isValid()) {
			trans = cm.data[i].transData;
		}
		p0(i) = trans.ty;
	} // for

	Signal<ieee64> p1;
	smoothValues (p0, p1, 0.2, 0.0);

	int32 thWidth = 37; // 37 frame (1.5 seconds)
	int32 thValue = 50;


	// get tilt up (p1 > 0)
	int32 i=0;
	while (i<n) {
		while (i<n && p1(i)<=0) i++;
		int32 left=i;
		while (i<n && p1(i)>0) i++;
		int32 right=i-1;

		int32 width = right-left+1;
		if (width>thWidth) {

			ieee64 value=0;
			for (int32 j=left; j<=right; j++) {
				value += p1(j);
			} // for

			if (value > thValue) {

				TransformationData t;
				getAverage (cm, t, left, right);

				
				CameraOperationItem item;
				item.cot = CAMERA_OPERATION_TILT_UP;
				item.firstFrame = left;
				item.lastFrame  = right;
				item.length     = width;
				item.strength   = value;
				item.trans      = t;

				op.data.push_back(item);
			} // if
		} // if
	} // while

	// get tilt down (p1 < 0)
	i=0;
	while (i<n) {
		while (i<n && p1(i)>=0) i++;
		int32 left=i;
		while (i<n && p1(i)<0) i++;
		int32 right=i-1;

		int32 width = right-left+1;
		if (width>thWidth) {

			ieee64 value=0;
			for (int32 j=left; j<=right; j++) {
				value += p1(j);
			} // for

			if (value < -thValue) {

				TransformationData t;
				getAverage (cm, t, left, right);

				
				CameraOperationItem item;
				item.cot = CAMERA_OPERATION_TILT_DOWN;
				item.firstFrame = left;
				item.lastFrame  = right;
				item.length     = width;
				item.strength   = value;
				item.trans      = t;

				op.data.push_back(item);
			} // if
		} // if
	} // while
	
	return;
}


void 
CameraOperation::detectZoom (const CameraModelData &cm, CameraOperationData& op)
{
	int32 n = cm.data.size();

	// get zoom-in (a00=a11):
	SignalSize s(n);
	Signal<ieee64> p0 (s);
	for (int32 i=0; i<n; i++) {
		TransformationData trans;
		if (cm.data[i].isValid()) {
			trans = cm.data[i].transData;
		}
		p0(i) = ((trans.a00+trans.a11) / 2.0) - 1.0;
	} // for

	Signal<ieee64> p1;
	smoothValues (p0, p1, 0.004, 1.0);

#if 0
	ofstream os ("F:/temp/data.txt", std::ios::out | std::ios::binary);
	os << p1 << endl;
	os.close();
	exit(1);
#endif

	int32 thWidth = 50; // 50 frames (2 seconds)
	ieee64 thValue = 0.9; 


	// get zoom in (p1 > 0)
	int32 i=0;
	while (i<n) {
		while (i<n && p1(i)<=0) i++;
		int32 left=i;
		while (i<n && p1(i)>0) i++;
		int32 right=i-1;

		int32 width = right-left+1;
		if (width>thWidth) {

			ieee64 value=0;
			for (int32 j=left; j<=right; j++) {
				value += p1(j);
			} // for

			if (value > thValue) {

				TransformationData t;
				getAverage (cm, t, left, right);

				
				CameraOperationItem item;
				item.cot = CAMERA_OPERATION_ZOOM_IN;
				item.firstFrame = left;
				item.lastFrame  = right;
				item.length     = width;
				item.strength   = value;
				item.trans      = t;

				op.data.push_back(item);
			} // if
		} // if
	} // while

	// get zoom out (p1 < 0)
	i=0;
	while (i<n) {
		while (i<n && p1(i)>=0) i++;
		int32 left=i;
		while (i<n && p1(i)<0) i++;
		int32 right=i-1;

		int32 width = right-left+1;
		if (width>thWidth) {

			ieee64 value=0;
			for (int32 j=left; j<=right; j++) {
				value += p1(j);
			} // for

			if (value < -thValue) {

				TransformationData t;
				getAverage (cm, t, left, right);

				
				CameraOperationItem item;
				item.cot = CAMERA_OPERATION_ZOOM_OUT;
				item.firstFrame = left;
				item.lastFrame  = right;
				item.length     = width;
				item.strength   = value;
				item.trans      = t;

				op.data.push_back(item);
			} // if
		} // if
	} // while
	
	return;
}



void 
CameraOperation::detectStatic (const CameraModelData &cm, CameraOperationData& op)
{
	int32 n = cm.data.size();
	VideoIndex idx (pathFilename, true, true);
	VideoIndexData idxData;
	idx.process(idxData);
	ImageSize imgSize = idxData.getImageSize();


	// get static camera shots: vector length is approx zero
	SignalSize s(n);
	Signal<ieee64> p0 (s);
	for (int32 i=0; i<n; i++) {
		p0(i) = 0.0;
		if (cm.data[i].isValid()) {
			TransformationData trans = cm.data[i].transData;
			
			ieee64 length;
			TransformationData::getMotionVectorLength (trans, imgSize, length);
			p0 (i) = length;
		}
	} // for

	Signal<ieee64> p1;
	smoothValues (p0, p1, 1.0, 0.0);

#if 0
	ofstream os ("F:/temp/data.txt", std::ios::out | std::ios::binary);
	os << p1 << endl;
	os.close();
	exit(1);
#endif

	int32 thWidth = 75; // 75 frames (3 seconds)

	// get static camera (p1 == 0)
	int32 i=0;
	while (i<n) {
		while (i<n && p1(i)>0) i++;
		int32 left=i;
		while (i<n && p1(i)==0) i++;
		int32 right=i-1;

		int32 width = right-left+1;
		if (width>thWidth) {

			TransformationData t;
			getAverage (cm, t, left, right);

			CameraOperationItem item;
			item.cot = CAMERA_OPERATION_STATIC;
			item.firstFrame = left;
			item.lastFrame  = right;
			item.length     = width;
			item.strength   = 0;
			item.trans      = t;
			
			op.data.push_back(item);
		} // if
	} // while
	
	return;
}



void 
CameraOperation::printStatistics (string &fn, CameraOperationData& op) 
{
	ofstream os (fn.c_str(), std::ios::out | std::ios::binary);

	SignalSize s (10);
	Signal<int32> cnt (s);
	cnt.clear(0);
	for (int32 i=0; i<op.data.size(); i++) {
		cameraOperationType t = op.data[i].cot;
		cnt(t)++;
	} // for
	os << "filename: " << op.filename << endl;
	os << "camera operations: " << op.data.size() << endl;
	os << "   unknown       : " << cnt(0) << endl;
	os << "   static        : " << cnt(1) << endl;
	os << "   zoom in       : " << cnt(2) << endl;
	os << "   zoom out      : " << cnt(3) << endl;
	os << "   pan left      : " << cnt(4) << endl;
	os << "   pan right     : " << cnt(5) << endl;
	os << "   tilt up       : " << cnt(6) << endl;
	os << "   tilt down     : " << cnt(7) << endl;
	os << "   rotate left   : " << cnt(8) << endl;
	os << "   rotate right  : " << cnt(9) << endl;
	os.close();
	return;
}


#if 0
void 
CameraOperation::normalizeVectors(Image<int8> &vec) 
{
	ImageSize size = vec.getSize();
				
	// scanning in Y direction: removing extrema
	for (int y=0; y< size[DIM_Y]; y++) {
					
		int min1 = 0;
		int max1 = 0;
		int min2 = 0;
		int max2 = 0;
		for (int x=0; x< size[DIM_X]; x++) {
			ieee64 v1 = vec(x,y,0);
			ieee64 v2 = vec(x,y,1);
			if (v1 < vec(min1,y,0)) {
				min1 = x;
			} // if
			if (v1 > vec(max1,y,0)) {
				max1 = x;
			} // if
			if (v2 < vec(min2,y,1)) {
				min2 = x;
			} // if
			if (v2 > vec(max2,y,1)) {
				max2 = x;
			} // if
		} // for
					
		// replace extreme values with values from surrounding vectors
		vec(min1,y,0) = (int8)((
			(ieee64)vec(min1-1,y-1,0) + vec(min1-1,y+1,0) + 
			vec(min1+1,y-1,0) + vec(min1+1,y+1,0) ) / 4.0);
		vec(max1,y,0) = (int8)((
			(ieee64)vec(max1-1,y-1,0) + vec(max1-1,y+1,0) + 
			vec(max1+1,y-1,0) + vec(max1+1,y+1,0) ) / 4.0);
		vec(min2,y,1) = (int8)((
			(ieee64)vec(min1-1,y-1,1) + vec(min1-1,y+1,1) + 
			vec(min1+1,y-1,1) + vec(min1+1,y+1,1) ) / 4.0);
		vec(max2,y,1) = (int8)((
			(ieee64)vec(max1-1,y-1,1) + vec(max1-1,y+1,1) + 
			vec(max1+1,y-1,1) + vec(max1+1,y+1,1) ) / 4.0);
	} // for
				
	// scanning in X direction: removing extrema
	for (int x=0; x< size[DIM_X]; x++) {
					
		// calculate one line
		int min1 = 0;
		int max1 = 0;
		int min2 = 0;
		int max2 = 0;
		for (int y=0; y< size[DIM_Y]; y++) {
			ieee64 v1 = vec(x,y,0);
			ieee64 v2 = vec(x,y,1);
			if (v1 < vec(x, min1,0)) {
				min1 = x;
			} // if
			if (v1 > vec(x, max1,0)) {
				max1 = x;
			} // if
			if (v2 < vec(x, min2,1)) {
				min2 = x;
			} // if
			if (v2 > vec(x, max2,1)) {
				max2 = x;
			} // if
		} // for
		
		// replace extreme values
		vec(x, min1, 0) = (int8)((
			(ieee64)vec(x-1, min1-1,0) + vec(x+1, min1-1,0) + 
			vec(x-1, min1+1,0) + vec(x+1, min1+1,0) ) / 4.0);
		vec(x, max1, 0) = (int8)((
			(ieee64)vec(x-1, max1-1,0) + vec(x+1, max1-1,0) + 
			vec(x-1, max1+1,0) + vec(x+1, max1+1,0) ) / 4.0);
		vec(x, min2, 0) = (int8)((
			(ieee64)vec(x-1, min2-1,0) + vec(x+1, min2-1,0) + 
			vec(x-1, min2+1,0) + vec(x+1, min2+1,0) ) / 4.0);
		vec(x, max2, 0) = (int8)((
			(ieee64)vec(x-1, max2-1,0) + vec(x+1, max2-1,0) + 
			vec(x-1, max2+1,0) + vec(x+1, max2+1,0) ) / 4.0);
	} // for
	
	return;
} // normalizeVectors

void 
CameraOperation::saveImage(Image<int8> &vec, Image<uint8> &img, int blockSize, int frame) 
{				
	// open output filestream and write image
	Image<uint8> saveImg (img);
	Image<uint8> tmpImg;
	GrayscaleFilter<uint8> filter;
	filter.process (img, tmpImg);
	ImageSize size = img.getSize();
	for (int x=0; x<size[DIM_X]; x++) {
		for (int y=0; y<size[DIM_Y]; y++) {
			int v = tmpImg(x,y,0) + 50;
			if (v > 255) {
				v = 255;
			} // if
			saveImg(x,y,0) = (uint8)v;
			saveImg(x,y,1) = (uint8)v;
			saveImg(x,y,2) = (uint8)v;
		} // for
	} // for


	size = vec.getSize();
	DrawDot dot (1);
	DrawLine line;
	for (int x=0; x<size[DIM_X]; x++) {
		for (int y=0; y<size[DIM_Y]; y++) {
			ImagePosition p1 (x*blockSize+blockSize/2,y*blockSize+blockSize/2);
			ImagePosition p2 (x*blockSize+blockSize/2 + (int)vec (x,y,0),y*blockSize+blockSize/2 + (int)vec(x,y,1));
			
			line.process(p1, p2, Color::YELLOW, saveImg);
			dot.process(p1, Color::RED, saveImg);
		} // for
	} // for
			
	stringstream s;
	s << "t:/temp/test/img" << setw(4) << setfill('0') << frame << ".jpg";
	cout << "save file: " << s.str() << endl;
	OImageStream<uint8> ois(s.str());
	ois.write(saveImg);
	ois.close();

	return;
} // saveImage

void 
CameraOperation::detectPan(Image<int8> &vec, CameraOperationData& tmp, int frame) 
{
	CameraOperationItem item;
	ImageSize size = vec.getSize();
	ieee64 sum = 0;
	for (int y=0; y< size[DIM_Y]; y++) {
		for (int x=0; x< size[DIM_X]; x++) {
			sum += vec(x,y,0);
		} // for
	} // for
	sum /= ((ieee64)size[DIM_X] * size[DIM_Y]);

	bool panDetected = false;
	if (sum > 0.1) {
		item.cot = CAMERA_OPERATION_PAN_LEFT;
		item.firstFrame = frame;
		item.lastFrame = frame;
		item.length = sum;
		tmp.data.push_back(item);	
		panDetected = true;
	} // if
	if (sum < -0.1) {
		item.cot = CAMERA_OPERATION_PAN_RIGHT;
		item.firstFrame = frame;
		item.lastFrame = frame;
		item.length = sum;
		tmp.data.push_back(item);	
		panDetected = true;
	} // if

	// modify vectors
	if (panDetected) {
		for (int y=0; y< size[DIM_Y]; y++) {
			for (int x=0; x< size[DIM_X]; x++) {
				vec(x,y,0)-=(int8)sum;
			} // for
		} // for
	} // if
	return;
} // detectPan

void 
CameraOperation::detectTilt(Image<int8> &vec, CameraOperationData& tmp, int frame) 
{
	CameraOperationItem item;
	ImageSize size = vec.getSize();
	ieee64 sum = 0;
	for (int y=0; y< size[DIM_Y]; y++) {
		for (int x=0; x< size[DIM_X]; x++) {
			sum += vec(x,y,1);
		} // for
	} // for
	sum /= ((ieee64)size[DIM_X] * size[DIM_Y]);

	bool tiltDetected=false;
	if (sum > 0.1) {
		item.cot = CAMERA_OPERATION_TILT_UP;
		item.firstFrame = frame;
		item.lastFrame = frame;
		item.length = sum;
		tmp.data.push_back(item);	
		tiltDetected=true;
	} // if
	if (sum < -0.1) {
		item.cot = CAMERA_OPERATION_TILT_DOWN;
		item.firstFrame = frame;
		item.lastFrame = frame;
		item.length = sum;
		tmp.data.push_back(item);	
		tiltDetected=true;
	} // if

	// modify vectors
	if (tiltDetected) {
		for (int y=0; y< size[DIM_Y]; y++) {
			for (int x=0; x< size[DIM_X]; x++) {
				vec(x,y,1)-=(int8)sum;
			} // for
		} // for
	} // if
	return;
} // detectTilt


void 
CameraOperation::detectZoom(Image<int8> &vec, CameraOperationData& tmp, int frame, Image<uint8> &img, int blockSize) 
{
	ImageSize size = vec.getSize();
	size[DIM_BAND] = 2;
	Image<ieee64> mask (size);
	// fill mask
	for (int y=0; y< size[DIM_Y]; y++) {
		for (int x=(size[DIM_X]+1)/2; x>=0; x--) {
			mask((size[DIM_X]+1)/2 + x,     y, 0) = x*0.1;
			mask((size[DIM_X]+1)/2 - x - 1, y, 0) = -x*0.1;
		} // for
	} // for
	for (int y=(size[DIM_Y]+1)/2; y>=0; y--) {
		for (int x=0; x< size[DIM_X]; x++) {
			mask(x, (size[DIM_Y]+1)/2 + y,     1) = y*0.1;
			mask(x, (size[DIM_Y]+1)/2 - y - 1, 1) = -y*0.1;
		} // for
	} // for

/*	cout << "vector: " << endl;
	for (int y=0; y< size[DIM_Y]; y++) {
		for (int x=0; x< size[DIM_X]; x++) {
			cout << vec(x,10,0) << "   ";
		} // for
		cout << endl << endl;
	} // for
*/
/*	
	cout << "mask: " << endl;
	for (int i=0; i< size[DIM_Y]; i++) {
		cout << mask((size[DIM_X]+1)/2, i, 1) << "   ";
	} // for
*/	

	// calculate difference to mask for all factors
	const int numberOfIterations = 5;
	ieee64 minFactor  = -20;
	ieee64 maxFactor  =  20;
	const int numberOfSteps = 20;

	ieee64 stepFactor = (maxFactor - minFactor) / numberOfSteps;

	ieee64 bestFactor     = 0;
	ieee64 bestDifference = 10e10;
	
	for (int j=0; j<numberOfIterations; j++) {

		for (ieee64 factor=minFactor; factor<maxFactor; factor+=stepFactor) {

			// calculate difference
			ieee64 difference = 0;
			//cout << "factor: " << factor << "   " << flush;
			for (int y=0; y< size[DIM_Y]; y++) {
				for (int x=0; x< size[DIM_X]; x++) {
					ieee64 m1 = mask(x,y,0) * factor;
					ieee64 m2 = mask(x,y,1) * factor;
					ieee64 v1 = vec(x,y,0);
					ieee64 v2 = vec(x,y,1);

					difference += fabs(m1 - v1) + fabs(m2 - v2);
					//difference += fabs(factor*mask(x,y,0) - vec(x,y,0)) + fabs(factor*mask(x,y,1) - vec(x,y,1));
				} // for
			} // for
			//cout << difference << endl;
			if (difference < bestDifference) {
				bestDifference = difference;
				bestFactor = factor;
			} // if
		} // for

		//cout << "best factor in iteration " << j << ":  " << bestFactor << " with value: " << bestDifference << endl;
		minFactor = bestFactor - 2 * stepFactor;
		maxFactor = bestFactor + 2 * stepFactor;
		stepFactor = (maxFactor - minFactor) / numberOfSteps;
	} // for

	bestDifference /= (0.01 * size[DIM_X] * size[DIM_Y]);
	//cout << "best factor: " << bestFactor << " with value: " << bestDifference << endl;



	// save vectors with bestFactor
/*	
	for (int y=0; y<size[DIM_Y]; y++) {
		for (int x=0; x<size[DIM_X]; x++) {
			mask(x,y,0) *= bestFactor;
			mask(x,y,1) *= bestFactor;
		} // for
	} // for
	saveImage (mask, img, blockSize, frame);
*/	

/*
	// save difference between original vectors and vectors with bestFactor
	for (int y=0; y<size[DIM_Y]; y++) {
		for (int x=0; x<size[DIM_X]; x++) {
			mask(x,y,0) *= bestFactor;
			mask(x,y,1) *= bestFactor;
			mask(x,y,0) -= vec(x,y,0);
			mask(x,y,1) -= vec(x,y,1);
		} // for
	} // for
	saveImage (mask, img, blockSize, frame);
*/

	
	CameraOperationItem item;
	if (bestFactor > 0.1) {
		//cout << "ZOOM IN : " << bestFactor << endl;
		item.cot = CAMERA_OPERATION_ZOOM_IN;
		item.firstFrame = frame;
		item.lastFrame = frame;
		item.length = bestFactor;
		tmp.data.push_back(item);	
		
	} else {
		if (bestFactor < -0.1) {
			//cout << "ZOOM OUT: " << -bestFactor << endl;
			
			item.cot = CAMERA_OPERATION_ZOOM_OUT;
			item.firstFrame = frame;
			item.lastFrame = frame;
			item.length = bestFactor;
			tmp.data.push_back(item);	
			
		} else {
			//cout << "NO ZOOM" << endl;
		} // else
	} // else		

	return;
} // detectZoom



void 
CameraOperation::aggregation(CameraOperationData& in, CameraOperationData& out, int frames) 
{
	// aggregate camera operations
	bool *operation = new bool  [frames+1];
	ieee64 *value   = new ieee64[frames+1];
	int i;

	//CAMERA_OPERATION_PAN_LEFT
	for (int cot= 0; cot<7; cot++) {

		cameraOperationType checkOperation = CAMERA_OPERATION_UNKNOWN;
		switch (cot) {
		case CAMERA_OPERATION_ZOOM_IN:
			checkOperation = CAMERA_OPERATION_ZOOM_IN;
			break;		
		case CAMERA_OPERATION_ZOOM_OUT: 
			checkOperation = CAMERA_OPERATION_ZOOM_OUT;
			break;		
		case CAMERA_OPERATION_PAN_LEFT: 
			checkOperation = CAMERA_OPERATION_PAN_LEFT;
			break;		
		case CAMERA_OPERATION_PAN_RIGHT: 
			checkOperation = CAMERA_OPERATION_PAN_RIGHT;
			break;		
		case CAMERA_OPERATION_TILT_UP:
			checkOperation = CAMERA_OPERATION_TILT_UP;
			break;		
		case CAMERA_OPERATION_TILT_DOWN:
			checkOperation = CAMERA_OPERATION_TILT_DOWN;
			break;		
		} // switch
		
		if (checkOperation != CAMERA_OPERATION_UNKNOWN) {
			
			for (i=0; i<frames+1; i++) {
				operation[i] = false;
				value[i] = 0.0;
			} // for
			list<CameraOperationItem>::const_iterator it;
			for (it = in.data.begin(); it != in.data.end(); it++) {
				if ((*it).cot == checkOperation) {
					int frame = (*it).firstFrame;
					operation[frame] = true;
					value    [frame] = (*it).length;
				} // if
			} // for

// probably remove the folowing lines
			for (i=1; i<frames-1; i++) {
				if (!operation[i]) {
					if(operation[i-1] && operation[i+1]) {
						operation[i] = true;
					} // if
				} // if
			} // for
			for (i=2; i<frames-2; i++) {
				if (!operation[i]) {
					if((operation[i-1] || operation[i-2]) && operation[i+1]) {
						operation[i] = true;
					} // if
					if(operation[i-1] && (operation[i+1] || operation[i+2])) {
						operation[i] = true;
					} // if
				} // if
			} // for
			for (i=1; i<frames-1; i++) {
				if (!operation[i]) {
					if(operation[i-1] && operation[i+1]) {
						operation[i] = true;
					} // if
				} // if
			} // for




			int first=0;
			int last =0;
			ieee64 sum;
			while (first<frames) {
				while (!operation[first] && first<frames) {
					first++;
				} // while
				last = first;
				sum=0.0;
				while (operation[last]) {
					sum += value[last];
					last++;
				} // while
				
				// insert item
				if (first!=last) {
					sum /= (last-first);
					CameraOperationItem item;
					item.cot = checkOperation;
					item.firstFrame = first;
					item.lastFrame = last;
					item.length = sum;
					out.data.push_back(item);	
				} // if
				
				first=last;
			} // while
		} // if
	} // for
	
	delete[] operation;
	delete[] value;

	return;
} // aggregation


#endif


