/*
    CameraParameterEstimation.cpp     estimate parameters for a camera model 
    Copyright (C) 2004 Markus Beier

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <CameraParameterEstimation.h>

CameraParameterEstimation::CameraParameterEstimation()
{
    iterations = 2;
    multiplier = 2;
    sigmaThreshold = 1;
}

CameraParameterEstimation::CameraParameterEstimation(uint8 it, ieee64 mp, ieee64 sT)
{
    iterations = it;
    multiplier = mp;
    sigmaThreshold = sT;
}

ieee64 CameraParameterEstimation::process( Image<ieee64>& srcVectors, Vector<ieee64>& parameters, uint8 blocksize, ParameterModel model )
{
    int16 centerX, centerY;
    uint32 rowNumber;
    ieee64 x, y, xres, yres;
    chisq = 0;
    int16 rowToFill = 0;
    ieee64 sigma = 1000;
    SvdFit svd;
    //    Image<ieee64> temp = srcVectors;

    size = srcVectors.getSize();

    if (size[DIM_BAND] != 3) {
        cout << "Input Image must have 3 bands" << endl;
        return -1;
    }

    size[DIM_BAND] = 2;
    modelVectors.resize(size);
    size[DIM_BAND] = 1;
    normDev.resize(size);

    size[DIM_BAND] = 3;

    centerX = (size[DIM_X] * blocksize)/2;
    centerY = (size[DIM_Y] * blocksize)/2;



    rowNumber = 2 * (size[DIM_X] * size[DIM_Y]);
    cout << "Anz der Zeilen am Anfang: " << rowNumber << endl;

    ieee64 mean =0;
    ieee64 stdDev = 0;
    //calculating the mean value over the vector norm and the stdDeviation
    for (int x = 0 ; x < size[DIM_X]; x++) {
        for (int y= 0; y < size[DIM_Y]; y++) {
            mean += sqrt(pow(srcVectors(x,y,0),2) + pow(srcVectors(x,y,1),2));
        }
    }
    mean /= (ieee64)(size[DIM_X]*size[DIM_Y]);

    for (int x = 0 ; x < size[DIM_X]; x++) {
         for (int y= 0; y < size[DIM_Y]; y++) {
             stdDev += pow( (sqrt(pow(srcVectors(x,y,0),2) + pow(srcVectors(x,y,1),2)) - mean),2);
         }
    }
    stdDev /= (ieee64)(size[DIM_X]*size[DIM_Y]);
    stdDev = sqrt(stdDev);

    /*
    for (int16 x = 0; x < size[DIM_X]; x++) {
        for (int16 y = 0; y < size[DIM_Y]; y++) {
            if ( (sqrt(pow(srcVectors(x,y,0),2) + pow(srcVectors(x,y,1),2))) > 2*mean)
                srcVectors(x,y,0) = srcVectors(x,y,1) = 0;

        }
    }
    */



    uint32 deleteRows = 0;
    for (int16 x = 0; x < size[DIM_X]; x++) {
        for (int16 y = 0; y < size[DIM_Y]; y++) {
            if ( srcVectors(x,y,2) < 0.0) {
                deleteRows++;
            }
        }
    }
    cout << "Blocks without data: " << deleteRows<< endl;
    rowNumber = rowNumber - (2*deleteRows);




    switch (model) {
    case SIX_PARAMETER_COMMON:
    {
        for (int16 f = 0; f < iterations && sigma > sigmaThreshold; f++) {
            rowToFill = 0;
            // reserving the space for Matrix A and the vectors parameters and b regarding the parameter model
            Matrix<ieee64> A(MatrixSize(rowNumber, 6));
            Vector<ieee64> b(rowNumber);

            int16 xco;
            for (xco = 0; xco < size[DIM_X]; xco++) {
                for (int16 yco = 0; yco < size[DIM_Y]; yco++) {
                    if (srcVectors(xco, yco,2) > -0.7) {
                        // calculating the coordinates of the blocks from the motion vector data and its position in the vector image
                        x    = xco * blocksize - centerX;
                        y    = centerY - yco * blocksize;
                        xres = x + srcVectors(xco, yco, 0);
                        yres = y + srcVectors(xco, yco, 1);

                        // filling line for x coordinate
                        A[rowToFill][0] = x;
                        A[rowToFill][1] = y;
                        A[rowToFill][2] = 1;
                        A[rowToFill][3] = 0;
                        A[rowToFill][4] = 0;
                        A[rowToFill][5] = 0;
                        b[rowToFill] = xres;
                        rowToFill++;

                        // filling line for y coordinate
                        A[rowToFill][0] = 0;
                        A[rowToFill][1] = 0;
                        A[rowToFill][2] = 0;
                        A[rowToFill][3] = x;
                        A[rowToFill][4] = y;
                        A[rowToFill][5] = 1;
                        b[rowToFill] = yres;
                        rowToFill++;
                    } // if
                } // for
            } // for
            chisq = svd.solve (A, parameters, b);

            // calculation of the motion vector with the parameter model
            for (xco = 0; xco < size[DIM_X]; xco++) {
                for (int16 yco = 0; yco < size[DIM_Y]; yco++) {
                    x    = xco * blocksize - centerX;
                    y    = centerY - yco * blocksize;
                    xres = parameters[0]*x + parameters[1]*y + parameters[2];
                    yres = parameters[3]*x + parameters[4]*y + parameters[5];

                    modelVectors (xco, yco, 0) = xres - x;
                    modelVectors (xco, yco, 1) = yres - y;

                } // for
            } // for

            sigma = stdDeviation(srcVectors, modelVectors, normDev);
            cout << "sigma : " << sigma << endl;
            cout << "error while processing : " << sqrt(chisq) << endl;

            // rowNumbers is decremented according to the deleted vectors
            //rowNumber -= killVectors(srcVectors, normDev, multiplier*sigma);
            cleanVectors (srcVectors, modelVectors, normDev, multiplier*sigma);

        } // for
    }
    break;

    case SIX_PARAMETER_PERSPECTIVE:
    {
        for (int16 f = 0; f < iterations && sigma > sigmaThreshold; f++) {

            rowToFill = 0;

            // reserving the space for Matrix A and the vectors parameters and b regarding the parameter model
            Matrix<ieee64> A(MatrixSize(rowNumber, 6));
            Vector<ieee64> b(rowNumber);
            int16 xco;
            for (xco = 0; xco < size[DIM_X]; xco++) {
                for (int16 yco = 0; yco < size[DIM_Y]; yco++) {
                    if (srcVectors(xco, yco,2) > -0.7) {
                        // calculating the coordinates of the blocks from the motion vector data and its position in the vector image
                        x    = xco * blocksize - centerX;
                        y    = centerY - yco * blocksize;
                        xres = x + srcVectors(xco, yco, 0);
                        yres = y + srcVectors(xco, yco, 1);

                        // filling line for x coordinate
                        A[rowToFill][0] = x;
                        A[rowToFill][1] = y;
                        A[rowToFill][2] = 1;
                        A[rowToFill][3] = 0;
                        A[rowToFill][4] = -x * xres;
                        A[rowToFill][5] = -y * xres;
                        b[rowToFill] = xres;
                        rowToFill++;

                        // filling line for y coordinate
                        A[rowToFill][0] = y;
                        A[rowToFill][1] = -x;
                        A[rowToFill][2] = 0;
                        A[rowToFill][3] = 1;
                        A[rowToFill][4] = -x * yres;
                        A[rowToFill][5] = -y * yres;
                        b[rowToFill] = yres;
                        rowToFill++;
                    } // if
                } // for
            } // for
            chisq = svd.solve (A, parameters, b);
            // calculation of the motion vector with the parameter model
            for (xco = 0; xco < size[DIM_X]; xco++) {
                for (int16 yco = 0; yco < size[DIM_Y]; yco++) {
                    x    = xco * blocksize - centerX;
                    y    = centerY - yco * blocksize;
                    xres = (parameters[0]*x + parameters[1]*y + parameters[2]) / (parameters[4]*x + parameters[5]*y + 1);
                    yres = (-parameters[1]*x + parameters[0]*y + parameters[3]) / (parameters[4]*x + parameters[5]*y + 1);

                    modelVectors (xco, yco, 0) = xres - x;
                    modelVectors (xco, yco, 1) = yres - y;

                } // for
            } // for

            sigma = stdDeviation(srcVectors, modelVectors, normDev);
            cout << "sigma : " << sigma << endl;
            cout << "error while processing : " << sqrt(chisq) << endl;

            // rowNumbers is decremented according to the deleted vectors
            //rowNumber -= killVectors(srcVectors, normDev, multiplier*sigma);
                        cleanVectors (srcVectors, modelVectors, normDev, multiplier*sigma);

        } // for
    }
    break;

    case EIGHT_PARAMETER_PERSPECTIVE:
    {
        for (int16 f = 0; f < iterations && sigma > sigmaThreshold; f++) {
            rowToFill = 0;
            // reserving the space for Matrix A and the vectors parameters and b regarding the parameter model
            cout << "Anzahl der Zeilen: " << rowNumber << endl;
            Matrix<ieee64> A(MatrixSize(rowNumber, 8));
            Vector<ieee64> b(rowNumber);

            int16 xco;
            for (xco = 0; xco < size[DIM_X]; xco++) {
                for (int16 yco = 0; yco < size[DIM_Y]; yco++) {
                    if (srcVectors(xco, yco,2) > -0.7) {

                        // calculating the coordinates of the blocks from the motion vector data and its position in the vector image
                        x    = xco * blocksize - centerX;
                        y    = centerY - yco * blocksize;
                        xres = x + srcVectors(xco, yco, 0);
                        yres = y - srcVectors(xco, yco, 1);

                        // filling line for x coordinate
                        A[rowToFill][0] = x;
                        A[rowToFill][1] = y;
                        A[rowToFill][2] = 1;
                        A[rowToFill][3] = 0;
                        A[rowToFill][4] = 0;
                        A[rowToFill][5] = 0;
                        A[rowToFill][6] = -x * xres;
                        A[rowToFill][7] = -y * xres;
                        b[rowToFill] = xres;

//                          // output of the Matrix
//                          cout << "[";
//                          cout << A[rowToFill][0] << ", "
//                               << A[rowToFill][1] << ", "
//                               << A[rowToFill][2] << ", "
//                               << A[rowToFill][3] << ", "
//                               << A[rowToFill][4] << ", "
//                               << A[rowToFill][5] << ", "
//                               << A[rowToFill][6] << ", "
//                               << A[rowToFill][7] << "]," << endl;

                        rowToFill++;

                        // filling line for y coordinate
                        A[rowToFill][0] = 0;
                        A[rowToFill][1] = 0;
                        A[rowToFill][2] = 0;
                        A[rowToFill][3] = x;
                        A[rowToFill][4] = y;
                        A[rowToFill][5] = 1;
                        A[rowToFill][6] = -yres * x;
                        A[rowToFill][7] = -y * yres;
                        b[rowToFill] = yres;

//                          // output of the Matrix
//                          cout << "[";
//                          cout << A[rowToFill][0] << ", "
//                               << A[rowToFill][1] << ", "
//                               << A[rowToFill][2] << ", "
//                               << A[rowToFill][3] << ", "
//                               << A[rowToFill][4] << ", "
//                               << A[rowToFill][5] << ", "
//                               << A[rowToFill][6] << ", "
//                               << A[rowToFill][7] << "]," << endl;

                        rowToFill++;

                    } // if
                } // for
            } // for

            // output of the right side
//              for (uint32 right = 0; right < rowNumber; right++) {
//                  cout << "b["<<right<< "] = "<< b[right] << ";" << endl;
//              }

            chisq = svd.solve (A, parameters, b);

            // calculation of the motion vector with the parameter model
            for (xco = 0; xco < size[DIM_X]; xco++) {
                for (int16 yco = 0; yco < size[DIM_Y]; yco++) {
                    x    = xco * blocksize - centerX;
                    y    = centerY - yco * blocksize;
                    xres = (parameters[0]*x + parameters[1]*y + parameters[2]) / (parameters[6]*x + parameters[7]*y + 1);
                    yres = (parameters[3]*x + parameters[4]*y + parameters[5]) / (parameters[6]*x + parameters[7]*y + 1);

                    modelVectors (xco, yco, 0) = xres - x;
                    modelVectors (xco, yco, 1) = y - yres;

                } // for
            } // for

            sigma = stdDeviation(srcVectors, modelVectors, normDev);
            cout << "sigma : " << sigma << " after " << f << " iterations" << endl;
            cout << "error while processing : " << sqrt(chisq) << endl;

            // rowNumbers is decremented according to the deleted vectors
            //rowNumber -= killVectors(srcVectors, normDev, multiplier*sigma);
            cleanVectors (srcVectors, modelVectors, normDev, multiplier*sigma);

            /*
            // recentering of the image. Improves the parameter estimation in case of Zoom, but leads to difficulties for further processment
            cout << "CenterX: " << centerX << " centerY: " << centerY << endl;
            // detect whether there is a Zoom and set the center to the zoom center
            if (parameters[0] > 1.01 && parameters[4] > 1.01  || parameters[0] < 0.99 && parameters[4] < 0.99) {
                cout << "recentering image..." << endl;
                uint32 tmpCenterX, tmpCenterY;
                ieee64 min;
                tmpCenterX = tmpCenterY = 0;
                min = 100.0;
                for (xco = 0; xco < size[DIM_X]; xco++) {
                    for (int16 yco = 0; yco < size[DIM_Y]; yco++) {
                        if ( (pow(modelVectors(xco, yco, 0),2) + pow(modelVectors(xco, yco, 1),2)) < min) {
                            tmpCenterX = xco * blocksize;
                            tmpCenterY = yco * blocksize;
                            min = pow(modelVectors(xco, yco, 0),2) + pow(modelVectors(xco, yco, 1),2);
                        }
                    } // for
                } // for
                centerX = tmpCenterX;
                centerY = tmpCenterY;
            } // if
            */

        } // for


    }
    break;


    } // switch

    cout << "error at end of processing : " << sqrt(chisq) << endl;

    printSlice( srcVectors, "source.F" );
    printSlice( modelVectors, "model.F" );

    /*
      // Modelling a Zoom vector field
    for (int16 xco = 0; xco < size[DIM_X]; xco++) {
        for (int16 yco = 0; yco < size[DIM_Y]; yco++) {
            x    = xco * blocksize - centerX;
            y    = centerY - yco * blocksize;
            xres = 1.2*x;
            yres = 1.2*y;

            modelVectors (xco, yco, 0) = xres - x;
            modelVectors (xco, yco, 1) = y - yres;

        } // for
    } // for
    printSlice( modelVectors, "ZoomModel.F" );
    */

    return sigma;

} // process


// This method calculates the standard deviation based on the euklid norms of the two vector images
ieee64 CameraParameterEstimation::stdDeviation( Image<ieee64>& vector1, Image<ieee64>& vector2, Image<ieee64>& normDev )
{
    ieee64 Dev = 0;
    ieee64 temp = 0;
    uint32 numberOfVectors = 0;
    for (int16 xco = 0; xco < size[DIM_X]; xco++) {
        for (int16 yco = 0; yco < size[DIM_Y]; yco++) {
            if (vector1(xco, yco, 2) > -0.7) {
                temp = pow(vector1(xco, yco, 0) - vector2(xco, yco, 0),2) + pow(vector1(xco, yco, 1) - vector2(xco, yco, 1),2);
                normDev(xco, yco, 0) = sqrt(temp);
                Dev += temp;
                numberOfVectors++;
            } // if
        } // for
    } // for

    return sqrt( (Dev / numberOfVectors) );
}

ieee64 CameraParameterEstimation::getError()
{
    return sqrt(chisq);
}


void CameraParameterEstimation::cleanVectors ( Image<ieee64>& src, Image<ieee64>& mdl, Image<ieee64>& Deviation, ieee64 maxDev)
{
    for (int16 xco = 0; xco < size[DIM_X]; xco++) {
        for (int16 yco = 0; yco < size[DIM_Y]; yco++) {
            if (Deviation(xco, yco, 0) > maxDev && src(xco, yco, 2) > -0.7 ) {
                // the value of -1.0 in the second band of Deviation indicates the deletion of the vector from further processing
                src  (xco, yco, 0) = mdl (xco, yco, 0);
                src  (xco, yco, 1) = mdl (xco, yco, 1);
            } // if
        } // for
    } // for
}


int16 CameraParameterEstimation::killVectors( Image<ieee64>& vectors, Image<ieee64>& Deviation, ieee64 maxDev)
{
    int16 rowsToDelete = 0;
    for (int16 xco = 0; xco < size[DIM_X]; xco++) {
        for (int16 yco = 0; yco < size[DIM_Y]; yco++) {
            if (Deviation(xco, yco, 0) > maxDev && vectors(xco, yco, 2) > -0.7 ) {
                // the value of -1.0 in the second band of Deviation indicates the deletion of the vector from further processing
                vectors  (xco, yco, 2) = -1.0;
                vectors  (xco, yco, 0) = 100.0;
                vectors  (xco, yco, 1) = -100.0;
                rowsToDelete++;
            } // if
        } // for
    } // for
    return 2*rowsToDelete;
}

void CameraParameterEstimation::setIterationsTo(uint8 it)
{
    iterations = it;
}

void CameraParameterEstimation::setSigmaRangeTo(ieee64 mult)
{
    multiplier = mult;
}

void CameraParameterEstimation::setSigmaThreshold(ieee64 sigThresh)
{
    sigmaThreshold = sigThresh;
}

void CameraParameterEstimation::printSlice( Image<ieee64>& srcVectors, char* filename )
{
    ofstream ofs(filename);
    ofs << size[DIM_X] << "  " << size[DIM_Y] << "  "
        << size[DIM_X] << "  " << size[DIM_Y] << " 0 0" << endl;
    for (indexType y = 0; y < size[DIM_Y]; y++) {
        for (indexType x = 0; x < size[DIM_X]; x++) {
            float norm = sqrt(srcVectors(x,y,0)*srcVectors(x,y,0) +
                              srcVectors(x,y,1)*srcVectors(x,y,1));
            if (norm > 0.1) {
                ofs << srcVectors(x,y,0) << "  " << -srcVectors(x,y,1) << endl;
            } else {
                ofs <<  "0.0 0.0" << endl;
            } // if - else
        } // for
    } // for
}

void CameraParameterEstimation::getModelVectors(Image<ieee64>& mdlImg)
{
    ImageSize mdlSize;
    mdlSize = modelVectors.getSize();
    mdlSize[DIM_BAND] = 2;
    mdlImg.resize(mdlSize);

    for (int16 xco = 0; xco < mdlSize[DIM_X]; xco++) {
        for (int16 yco = 0; yco < mdlSize[DIM_Y]; yco++) {
            mdlImg(xco,yco,0) = modelVectors(xco,yco,0);
            mdlImg(xco,yco,1) = modelVectors(xco,yco,1);
        }
    }

} // getDeviationImage
