/*
   This file is part of the MoCA video library.
   Name: CssDisplayResult.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


/* -*- C++ -*-

   file.h  -- short description

   Author:    Stephan Kopf
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: kopf@pi4.informatik.uni-mannheim.de

              $Id: CssDisplayResult.h,v 1.1 2004/02/17 11:52:58 kopf Exp $
*/



#ifndef CSS_RESULT_H
#define CSS_RESULT_H

// MoCA includes
#include <MocaTypes.h>
#include <ImagePosition.h>
#include <ImageEdgePosition.h>
#include <CssMaximaFeature.h>
#include <FileList.h>

// C++ includes
#include <math.h>
#include <string>
#include <list>
#include <set>
#include <GeometricFeature.h>


/*!
  \brief parametizes a list of contour pixels

  A list of ImageEdgePositions (input) is transformed into a list
  of ImagePositions which describe a conto7ur of an object.
 */

const int MAX_DATABASE_SIZE = 400;
const int MAX_SEQUENCE_SIZE = 500;
const int MAX_OBJECT_CLASS  =  20;


struct CssDatabaseResultItem
{    
    string fn;				         // filename of tag-file in database
    string objectType;		         // object class name
    int objectTypeNr;		         // object class name id
    CssMaximumData cmS;              // stores up to ten peaks (standard)
    CssMaximumData cmX;              // stores up to ten peaks (extended)
	GeometricFeatureData geometric;  // stores global geometric features 
};

struct CssDifference
{
	int32 distance;
	int32 dbRef;
    bool operator< (const CssDifference& other) const {
		return (distance < other.distance);
	}
};

struct CssSequence
{    
    string fn;							// filename of tag-file in database
	list<CssDifference> diff;
    CssMaximumData cmS;
    CssMaximumData cmX;
	GeometricFeatureData geometric;  // stores global geometric features 
};

struct CssResults
{    
    int topMatch;
    int top5Match;
    int top10Match;
    int top20Match;
    int f1;
    int foundAll;
    int notFoundAll;
    int found10;
    int notFound10;
    ieee64 topValuePercent;
};


class CssDisplayResult
{
public:
	//! the constructor, input: database and sequence list
    CssDisplayResult(vector<FileInfo> &databaseList, vector<FileInfo> &sequenceList, 
		             bool showStatus, bool extended, bool useGlobalDescriptors=false);

    //! destructor
    ~CssDisplayResult() {};
    
    //! calculate results
    void showAll(int32 maxItems);
    void show(string &searchString);
	void printMatrix(string &matrixFile);

    CssSequence seq            [MAX_SEQUENCE_SIZE];
    CssDatabaseResultItem data [MAX_DATABASE_SIZE];

    inline uint16 getSequenceSize ();
    inline uint16 getDatabaseSize ();
    
protected:
private:
    void readDatabaseFiles();
    void readSequenceFiles();

    vector<FileInfo> databaseList;
    vector<FileInfo> sequenceList;
    
    uint16 sequenceSize;
    int objectClassSize;
    uint16 databaseSize;

    string objectClass [MAX_OBJECT_CLASS];
	bool showStatus;
	bool extended;
	bool useGlobalDescriptors;

}; // class CssDisplayResult


inline uint16
CssDisplayResult::getSequenceSize ()
{
    return sequenceSize;
} // getSequenceSize

inline uint16
CssDisplayResult::getDatabaseSize ()
{
    return databaseSize;
} // getDatabaseSize


#endif




