/*
    CssMatching.cpp     match two CSSS images
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/




#include <CssMatching.h>


int
CssMatching::distance(CssMaximumData &cssi, CssMaximumData &cssp)
{

    //    cerr << "Entering CssMatching::distance" << endl;
    int mindiff = INT_MAX;

    // sort entries in vector according to height
    //cssi.data.sort();
    //cssi.data.reverse();
	cssi.inverseSort();
    //cssp.data.sort();
    //cssp.data.reverse();
    cssp.inverseSort();

    for (int i = 0; i <= 1; i++) {
        if (i == 1) {           // mirror image
            // shift cssi
            for (vector<CssMaximumItem>::iterator iiter = cssi.data.begin();// image iterator
                 iiter != cssi.data.end(); iiter++) {
                (*iiter).position = contourPixels - (*iiter).position;
            } // for
        } // if

        vector<int> shiftlist = calculateShiftList(cssi, cssp);

        if (shiftlist.empty()) {
            return INT_MAX;
        } // if


        //    for (vector<int>::iterator iter = shiftlist.begin();
        //         iter != shiftlist.end();
        //         iter++) {
        //      cerr << *iter << endl;
        //    } // for

        // for all entries in shiftlist
        for (vector<int>::iterator sliter = shiftlist.begin();// shift list iterator
             sliter != shiftlist.end();
             sliter++) {

            // shift cssi
			vector<CssMaximumItem>::iterator iiter;
            for (iiter = cssi.data.begin();
                 iiter != cssi.data.end(); iiter++) {
                (*iiter).position = ((*iiter).position - (*sliter) + contourPixels) % contourPixels;
            } // for

            //            cerr << "i p" << endl;

            // calculateDiff of cssishifted and cssp, considering thresholds
            int cdiffip = calculateDiff(cssi, cssp);

            //            cerr << "p i" << endl;

            // calculateDiff of cssp and cssishifted, considering thresholds
            int cdiffpi = calculateDiff(cssp, cssi);

            // cdiff is maxima of calculated differences (take realistic match)
            int cdiff = (cdiffip > cdiffpi) ? cdiffip : cdiffpi;

            // if difference < mindiff, set new found
            mindiff = (cdiff < mindiff) ? cdiff : mindiff;

            // unshift cssi
            for (iiter = cssi.data.begin();
                 iiter != cssi.data.end(); iiter++) {
                (*iiter).position = ((*iiter).position + (*sliter) + contourPixels) % contourPixels;
            } // for

        } // for

        if (i == 1) {		// unmirror image
            // shift cssi
            for (vector<CssMaximumItem>::iterator iiter = cssi.data.begin();
                 iiter != cssi.data.end(); iiter++) {
                (*iiter).position = contourPixels - (*iiter).position;
            } // for
        } // if

    } // for

    // return mindiff
    return mindiff;
} // CssMatching::distance



int
CssMatching::distanceExt (const CssMaximumData &one_, const CssMaximumData &oneExt_,
					      const CssMaximumData &two_, const CssMaximumData &twoExt_)
{

    int mindiff = INT_MAX;
	CssMaximumData s1 = one_;
	CssMaximumData x1 = oneExt_;
	CssMaximumData s2 = two_;
	CssMaximumData x2 = twoExt_;

    // sort entries in vector according to height
	s1.inverseSort();
	x1.inverseSort();
	s2.inverseSort();
	x2.inverseSort();


    for (int i = 0; i <= 1; i++) {

		// mirror image
        if (i == 1) {   
			
            // shift (mirror) one
            for (int32 i=0; i<s1.data.size(); i++) {
				s1.data[i].position = contourPixels - s1.data[i].position;
            } // for
            // shift (mirror) oneExt
            for (int32 i=0; i<x1.data.size(); i++) {
				x1.data[i].position = contourPixels - x1.data[i].position;
            } // for
        } // if

		// create backup of vectors
		CssMaximumData bs1 = s1;
		CssMaximumData bx1 = x1;

		// do not use a shift list, but calculate all shift positions
		for (int32 shift=0; shift<contourPixels; shift++) {

			// restore backup 
			s1 = bs1;
			x1 = bx1;

            // shift one
            for (int32 i=0; i<s1.data.size(); i++) {
				s1.data[i].position = (s1.data[i].position - shift + contourPixels) % contourPixels;
            } // for

            // shift oneExt
            for (int32 i=0; i<x1.data.size(); i++) {
				x1.data[i].position = (x1.data[i].position - shift + contourPixels) % contourPixels;
            } // for


            // calculate difference of one(shifted) and two, considering thresholds
            int sDiff1 = calculateDiff(s1, s2);
            int sDiff2 = calculateDiff(s2, s1);
            int sDiff  = max (sDiff1,sDiff2);

            // calculate difference of one(shifted) and two, considering thresholds
            int xDiff1 = calculateDiff(x1, x2);
            int xDiff2 = calculateDiff(x2, x1);
            int xDiff  = max (xDiff1,xDiff2);

			// summarize difference
			int32 diff = max (sDiff, xDiff);

			// avoid overflow
			int32 ddd = (int32) (sDiff + xDiff);
			if (diff < ddd) diff=ddd;

            // if difference < mindiff, set new found
            mindiff = (diff < mindiff) ? diff : mindiff;
        } // for
    } // for

    return mindiff;
} // CssMatching::distanceExt




// cssi: image css
// cssp: prototype css
//
vector<int>
CssMatching::calculateShiftList(CssMaximumData &cssi, CssMaximumData &cssp) 
{
    vector<int> temp;

    int maxheighti = (*cssi.data.begin()).height;
    maxheighti -=  int (thHeight * maxheighti);

    //    cerr << "maxheighti " << maxheighti << endl;

    int maxheightp = (*cssp.data.begin()).height;
    maxheightp -= int (thHeight * maxheightp);

    //    cerr << "maxheightp " << maxheightp << endl;

    for (vector<CssMaximumItem>::iterator iiter = cssi.data.begin();
        iiter != cssi.data.end() && (*iiter).height >= maxheighti;
        iiter++) {
        for (vector<CssMaximumItem>::iterator piter = cssp.data.begin();
             piter != cssp.data.end() && (*piter).height >= maxheightp;
             piter++) {

            int maxheight = ((*iiter).height > (*piter).height) ? (*iiter).height : (*piter).height;
            int minheight = ((*iiter).height < (*piter).height) ? (*iiter).height : (*piter).height;

//              cerr << "maxheight " << maxheight << "minheight " << minheight << endl;

            int maxwidth = ((*iiter).width > (*piter).width) ? (*iiter).width : (*piter).width;
            int minwidth = ((*iiter).width < (*piter).width) ? (*iiter).width : (*piter).width;

            //            cerr << endl << "maxwidth " << maxwidth << "minwidth " << minwidth << endl;

            if (minheight >= (maxheight - thHeight * maxheight)) {
                if (minwidth >= (maxwidth - thWidth * maxwidth)) {
					if (rotateCss) {
						temp.push_back((*iiter).position - (*piter).position);
					} else {
						int32 diff = labs((*iiter).position - (*piter).position);
						if (diff < ContourParametrization::DEFAULT_CONTOUR_SIZE/3) {
							temp.push_back((*iiter).position - (*piter).position);
						} // if
					} // else
                } // if
            } // if

        } // for
    } // for

    return temp;
} // CssMatching::calculateShiftList

// cssi: shifted css image maxima
// cssp: css prototype maxima
//
int
CssMatching::calculateDiff(CssMaximumData &cssi, CssMaximumData &cssp) {

    //    cerr << "Entering calculateDiff" << endl;

    int diff = 0;

    // set all maxima in prototype to unvisited
	vector<CssMaximumItem>::iterator piter;
    for (piter = cssp.data.begin();
         piter != cssp.data.end();
         piter++) {
        (*piter).visited = false;
    } // for

    // height of largest css maxima in image
    int maxheight = (*cssi.data.begin()).height;

    //    cerr << "maxheight: " << maxheight << endl;

    // match each maxima in image
    for (vector<CssMaximumItem>::iterator iiter = cssi.data.begin();
         iiter != cssi.data.end() && diff != INT_MAX;
         iiter++) {

        // calculate potential range for position
        int posmin = int((*iiter).position - thPosition * (*iiter).position) + contourPixels;
        posmin %= contourPixels;
        int posmax = int((*iiter).position + thPosition * (*iiter).position) + contourPixels;
        posmax %= contourPixels;

        int iterdiff = INT_MAX;	       // smallest diff of current maxima in image
        CssMaximumItem *iterposit = NULL;  // position of that maxima (the one for iterdiff)

        // find maxima in prototype which is in position range and has smallest diff
        // to current maxima in image
        for (vector<CssMaximumItem>::iterator piter = cssp.data.begin();
             piter != cssp.data.end();
             piter++) {

            if (!(*piter).visited) {

                int maxwidth = ((*iiter).width > (*piter).width) ? (*iiter).width : (*piter).width;
                int minwidth = ((*iiter).width < (*piter).width) ? (*iiter).width : (*piter).width;

                // check if width requirement is ok
                if (minwidth < (maxwidth - thWidth * maxwidth)) {
                    continue;
                } // if

                // check if both peaks are concave or convex
				if (optimizeConcave) {
					if ((*piter).isConcave != (*iiter).isConcave) {
						continue;
					} // if
				} // if

                // check if position range fits
                if (posmin <= posmax) {	// normal case
                    if ((*piter).position >= posmin && (*piter).position <= posmax) {
                        int temp = abs((*iiter).height - (*piter).height);
                        if (temp < iterdiff) {
                            iterdiff = temp;
                            iterposit = &(*piter);
                        } // if
                    } // if
                } else {		// wrap around case
                    if ((*piter).position >= posmin || (*piter).position <= posmax) {
                        int temp = abs((*iiter).height - (*piter).height);
                        if (temp < iterdiff) {
                            iterdiff = temp;
                            iterposit = &(*piter);
                        } // if
                    } // if
                } // if
            } // if
        } // for

        // if maxima handled is significant (i.e. > maxheight - 50 %), check if match is close enough (i.e. +- 20 %)
        if ((*iiter).height > maxheight * thSignificantHeight) {
            if (iterdiff != INT_MAX) {
                int heightmin = int((*iiter).height - thHeight * (*iiter).height);
                int heightmax = int((*iiter).height + thHeight * (*iiter).height);
                if (iterposit->height < heightmin || iterposit->height > heightmax) {
                    diff = INT_MAX;
                } // if
            } else {
                diff = INT_MAX;
            } // if
        } // if


        // if matching maxima is found, calculate euclidian distance
        if (iterdiff != INT_MAX) {
            iterposit->visited = true;

            int diffiter = abs(iterposit->height - (*iiter).height);

            int diffpos1 = abs(iterposit->position - (*iiter).position);
            int diffpos2 = contourPixels - diffpos1;
            int diffpos  = (diffpos1 < diffpos2) ? diffpos1 : diffpos2;

            if (diff != INT_MAX) {
                //                cerr << "Found maxima: " << iterposit->height << " " << (*iiter).height << endl;
                diff += (int) sqrt((ieee64)(diffiter * diffiter + diffpos * diffpos));
            } // if
        } else {
            if (diff != INT_MAX) {
                diff += (*iiter).height * 2; // penalty form
            } // if
        } // if

    } // for

    // no need to increase INT_MAX...
    if (diff != INT_MAX) {
        for (piter = cssp.data.begin();
             piter != cssp.data.end();
             piter++) {
            if (!(*piter).visited) {
                diff += (*piter).height * 2; // penalty form
            } // if
        } // for
    } // if

    //    cerr << "Leaving calculateDiff: " << diff << endl;

    return diff;
} // CssMatching::calculateDiff
