/*
    CssVideoObject.cpp     analyse CSS features in video objects
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// MoCA includes
#include <CssVideoObject.h>


OTagStream& operator<< (OTagStream& os, const CssVideoObjectItem &item) 
{
    TagFormat::writeNodeValue(os, "frame", item.frame);
    TagFormat::writeNodeValue(os, "shot", item.shot);
    TagFormat::writeNodeValue(os, "databaseRef", item.databaseRef);
    TagFormat::writeNodeValue(os, "databaseDist", item.databaseDist);
    TagFormat::writeNodeValue(os, "px1", item.px1);
    TagFormat::writeNodeValue(os, "px2", item.px2);
    TagFormat::writeNodeValue(os, "py1", item.py1);
    TagFormat::writeNodeValue(os, "py2", item.py2);
	os << endl;
	os << item.data << endl;
	os << endl;
	
    return os;
} // operator<<


ITagStream& operator>> (ITagStream& is, CssVideoObjectItem &item)
{
	
    char c;
    bool failFlag = true;
	
	if (is >> c) {
		failFlag = false;
		is.putback(c);

		TagFormat::readNodeValue(is, "frame", item.frame);
		TagFormat::readNodeValue(is, "shot", item.shot);
		TagFormat::readNodeValue(is, "databaseRef", item.databaseRef);
		TagFormat::readNodeValue(is, "databaseDist", item.databaseDist);
		TagFormat::readNodeValue(is, "px1", item.px1);
		TagFormat::readNodeValue(is, "px2", item.px2);
		TagFormat::readNodeValue(is, "py1", item.py1);
		TagFormat::readNodeValue(is, "py2", item.py2);

		is >> item.data;
	} // if

	if (is.rdstate() & ios::failbit) {
		throw MocaException ("CssVideoObject: Illegal index file format!");
	} else {
        is.clear();
    } // else
	
    return is;
} // operator>> 





std::ostream& operator<< (ostream &os, const CssVideoObjectData &data) 
{
	os << "version               : " << data.version << endl;
	os << "filename              : " << data.filename << endl;
	os << "number of items       : " << data.numberOfItems << endl;
	os << endl;
	os << endl;

	return os;
} // operator <<

OTagStream& operator<< (OTagStream& os, const CssVideoObjectData &data) 
{
    TagFormat::writeStartTag (os, "CSS_VIDEO_OBJECT");
	os << endl;
	os << endl;
	
    TagFormat::writeNodeValue(os, "version", data.version, 1);
    TagFormat::writeNodeValue(os, "filename", data.filename, 1);
	int32 s = data.data.size();
    TagFormat::writeNodeValue(os, "numberOfItems", s, 1);
	os << endl;
	
	for (int32 i=0; i<(int32)data.data.size(); i++) {
		os << data.data[i];
	} // for

    TagFormat::writeEndTag (os, "CSS_VIDEO_OBJECT");
	os << endl;
	
    return os;
} // operator<<


ITagStream& operator>> (ITagStream& is, CssVideoObjectData& data)
{
	
    char c;
    bool failFlag = true;
	data.data.clear();
	
	if (is >> c) {
		failFlag = false;
		is.putback(c);

		TagFormat::readStartTag (is, "CSS_VIDEO_OBJECT");
		
		TagFormat::readNodeValue(is, "version", data.version);
		TagFormat::readNodeValue(is, "filename", data.filename);
		TagFormat::readNodeValue(is, "numberOfItems", data.numberOfItems);
		if (is.rdstate() & ios::failbit) {
			throw MocaException ("CssVideoObject: Illegal index file format!");
		} // if


		// read cuts
		if (is >> c) {
			is.putback(c);

			CssVideoObjectItem item;
			for (int32 i=0; i<data.numberOfItems; i++) {
				is >> item;
				if (is.rdstate() & ios::failbit) {
					throw MocaException ("CssVideoObject: Illegal index file format!");
				} // if
				data.data.push_back(item);
			} // for
		} // if
		
		TagFormat::readEndTag (is, "CSS_VIDEO_OBJECT");
	} // if

	if (is.rdstate() & ios::failbit) {
		throw MocaException ("CssVideoObject: Illegal index file format!");
	} else {
        is.clear();
    } // else
	
    return is;
} // operator>> 





CssVideoObject::CssVideoObject(const string &pathFilename) 
{
	this->pathFilename = pathFilename;
} 


// void 
// CssVideoObject::updateValues(CssVideoObjectData& data) 
// {
	
// 	// calculate video object masks
// 	ObjectMaskData objectMasksData;
// 	{
// 		int minimumShotLength = 12;
// 		bool preciseCalculation=false;
// 		ObjectMask objMask(pathFilename, minimumShotLength, preciseCalculation);
// 		objMask.process(objectMasksData);
// 	}


// 	// calculate borders of image and find best matching object in database
// 	int32 cnt = 0;
// 	list<ObjectMaskItem>::iterator it;
// 	for (it=objectMasksData.frameData.begin(); it!=objectMasksData.frameData.end(); it++) {

// 		// copy mask pixels to ipList
// 		ObjectMaskItem tmpItem = (*it);
// 		int32 px1 = 10000;
// 		int32 px2 = -1;
// 		int32 py1 = 10000;
// 		int32 py2 = -1;
// 		for (int32 i=0; i<ContourParametrization::DEFAULT_CONTOUR_SIZE; i++) {
// 			int32 px = (*it).contour (i,0);
// 			int32 py = (*it).contour (i,1);
// 			if (px1>px) px1=px;
// 			if (px2<px) px2=px;
// 			if (py1>py) py1=py;
// 			if (py2<py) py2=py;
// 		} // for

// 		bool error = false;
// 		if (cnt > (int32)data.data.size()) {
// 			error=true;
// 		} else {
// 			if ((*it).frame != data.data[cnt].frame) error = true;
// 		} // else
// 		if (error) {
// 			cerr << "CssVideoObject::updateValues: cannot update values!" << endl;
// 		} // if
// 		data.data[cnt].px1 = px1;
// 		data.data[cnt].px2 = px2;
// 		data.data[cnt].py1 = py1;
// 		data.data[cnt].py2 = py2;

// 		cnt++;
// 		cout << "\rframe: " << cnt << "     " << ends;
// 	} // for



// 	// initialize database
// 	vector<CssStoredDatabaseItem> database;
// 	vector<string> objectClassName;
// 	{
// 	    // see CSS_DB_Schema.h for different schemas
// 	    CssDatabaseSchemaType schema = ECHO_DB_TYPE;
// 	    CssDatabase cssDB(schema);
// 	    cssDB.getDatabase (database, objectClassName);
// 	}

// 	// initialize matching
// 	int32 tPosition       = 20;
// 	int32 tHeight         = 20;
// 	int32 tWidth          = 40;
// 	int32 tCriticalHeight = 50;
// 	bool tCheckConcave    = true;
// 	bool XYratio          = true;
// 	bool rotateCss        = true;
// 	CssMatching cssm(
// 		ContourParametrization::DEFAULT_CONTOUR_SIZE, 
// 		tPosition, tHeight, tWidth, tCriticalHeight, tCheckConcave, XYratio, rotateCss); 


// 	// find best matching CSS value
// 	for (int32 i=0; i<(int32)data.data.size(); i++) {
// 		ieee64 minDist = 10e10;
// 		int32 minRef = -1;
// 		for (int32 j=0; j<(int32)database.size(); j++) {
// 			ieee64 dist = cssm.process(data.data[i].data, database[j].data);
// 			if (dist<minDist) {
// 				minDist = dist;
// 				minRef = j;
// 			} // if
// 		} // for
// 		data.data[i].databaseRef  = minRef;
// 		data.data[i].databaseDist = (int32)minDist;
// 		if (minDist > 3000) {
// 			data.data[i].databaseRef = -1;
// 		} // if

// 		cout << "\rframe: " << i << "     " << ends;
// 	} // for
	
// 	return;
// } // updateValues

void 
CssVideoObject::process(CssVideoObjectData& data/*, bool update*/) 
{
	// read data
	ReadMocaData<CssVideoObjectData> reader (pathFilename, CSS_VIDEO_OBJECT_FILENAME);
	bool dataValid = reader.process (CSS_VIDEO_OBJECT_VERSION, data); 


// 	if (update) {
// 		if (dataValid) {
// 			cerr << "\rCssVideoObject: Update index file ...          " << endl;
// 			updateValues(data);
			
// 			// save index file
// 			WriteMocaData<CssVideoObjectData> writer (pathFilename, CSS_VIDEO_OBJECT_FILENAME);
// 			writer.process(data);
// 			cerr << "\rCssVideoObject: Update index file ...done          " << endl;
// 		} // if
// 	} // if
	
	if (!dataValid) {
		
		// create new index
		cerr << "\rCssVideoObject: Creating index file ...          " << endl;
		readValues(data);
		
		// save index file
		WriteMocaData<CssVideoObjectData> writer (pathFilename, CSS_VIDEO_OBJECT_FILENAME);
		writer.process(data);
		cerr << "\rCssVideoObject: Creating index file ... done          " << endl;
	} // if

} // process


void 
CssVideoObject::readValues (CssVideoObjectData& data) 
{
	// set variables
	data.version = CSS_VIDEO_OBJECT_VERSION;
	data.numberOfItems = 0;
	data.data.clear();

	// get path of filename, do not create new directory
	CreateMocaPath mocaPath (pathFilename, false);
	data.filename = mocaPath.file;


	// calculate video object masks
	ObjectMaskData objectMasksData;
	{
		int minimumShotLength = 12;
		bool preciseCalculation=false;
		ObjectMask objMask(pathFilename, minimumShotLength, preciseCalculation);
		objMask.process(objectMasksData);
	}


	// calculate CSS images 
	list<ObjectMaskItem>::iterator it;
	for (it=objectMasksData.frameData.begin(); it!=objectMasksData.frameData.end(); it++) {

		// copy mask pixels to ipList
		ObjectMaskItem tmpItem = (*it);
        vector<ImagePosition> ipList;
		int32 px1 = 10000;
		int32 px2 = -1;
		int32 py1 = 10000;
		int32 py2 = -1;
		for (int32 i=0; i<ContourParametrization::DEFAULT_CONTOUR_SIZE; i++) {
			ImagePosition ip ((*it).contour (i,0), (*it).contour (i,1));
			ipList.push_back(ip);
			if (px1>ip[DIM_X]) px1=ip[DIM_X];
			if (px2<ip[DIM_X]) px2=ip[DIM_X];
			if (py1>ip[DIM_Y]) py1=ip[DIM_Y];
			if (py2<ip[DIM_Y]) py2=ip[DIM_Y];
		} // for


        // calculate css-image
	    Image <uint8> cssImg;
		{
			CssImage css;
			css.process(ipList, cssImg);
		}
		cerr << "\rframe: " << (*it).frame << "   " << ends;


		// debug: store css image
		//string tmpCssFilename = "V:/test_css.jpg";
        //OImageStream<uint8> ois(tmpCssFilename);
        //ois.write(cssImg);


		// get css maxima
        CssMaximumData cmList;
		{
			CssMaximaFeature cmf;
			cmf.process (cssImg, cmList, ipList);
			//cmList.data.sort();
		    //cmList.data.reverse();
			cmList.inverseSort();

			if ((int32)cmList.data.size()>DEFAULT_CSS_PEAK_NUMBER) {
				CssMaximumData tmpList;
				vector<CssMaximumItem>::iterator it;
				for (it=cmList.data.begin(); it!=cmList.data.end(); it++) {
					if ((int32)tmpList.data.size()>=DEFAULT_CSS_PEAK_NUMBER) {
						break;
					} // if
					tmpList.data.push_back(*it);
				} // for
				cmList = tmpList;
			} // if
			CssMaximumItem item;
			while ((int32)cmList.data.size()<DEFAULT_CSS_PEAK_NUMBER) {
				cmList.data.push_back(item);
			} // if
		}

		
		// store css data
		{
			CssVideoObjectItem item (cmList, (*it).frame, (*it).shot, px1, px2, py1, py2);
			data.data.push_back (item);
		}
	} // for




	// initialize database
	vector<CssStoredDatabaseItem> database;
	vector<string> objectClassName;
	{
	    // see CSS_DB_Schema.h for different schemas
	    CssDatabaseSchemaType schema = ECHO_DB_TYPE;
	    CssDatabase cssDB(schema);
	    cssDB.getDatabase (database, objectClassName);
	}

	// initialize matching
	int32 tPosition       = 20;
	int32 tHeight         = 20;
	int32 tWidth          = 40;
	int32 tCriticalHeight = 50;
	bool tCheckConcave    = true;
	bool XYratio          = true;
	bool rotateCss        = true;
	CssMatching cssm(
		ContourParametrization::DEFAULT_CONTOUR_SIZE, 
		tPosition, tHeight, tWidth, tCriticalHeight, tCheckConcave, XYratio, rotateCss); 


	// find best matching CSS value
	for (int32 i=0; i<(int32)data.data.size(); i++) {
		ieee64 minDist = 10e10;
		int32 minRef = -1;
		for (int32 j=0; j<(int32)database.size(); j++) {
			ieee64 dist = cssm.process(data.data[i].data, database[j].data);
			if (dist<minDist) {
				minDist = dist;
				minRef = j;
			} // if
		} // for
		data.data[i].databaseRef  = minRef;
		data.data[i].databaseDist = (int32)minDist;
		if (minDist > 3000) {
			data.data[i].databaseRef = -1;
		} // if
	} // for



	data.numberOfItems = data.data.size();
	cerr << endl;


	return;	
} // readValues
