/*
    EdgebasedDistance.cpp     calculate a distance measure of two frames based on edges
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <EdgebasedDistance.h>


EdgebasedDistance::EdgebasedDistance()
{}

void
EdgebasedDistance::process(Image<uint8>& src, Image<uint8>& dest, Image<int16>& distanceMap)
{
    size = src.getSize();
    queue<position> q;
    position pixel;
    size[DIM_BAND] = 1;
    distanceMap.resize(size);
    Image<bool> visited;
    visited.resize(size);

    for (int16 x = 0; x < size[DIM_X]; x++) {
        for (int16 y = 0; y < size[DIM_Y]; y++) {
            if ( src(x, y, 0) == 255 ) {
                pixel.dist = 0;
                pixel.x = x;
                pixel.y = y;
                q.push(pixel);

                while ( !q.empty() ) {
                    pixel = q.front();
                    q.pop();
                    if ( dest(pixel.x, pixel.y, 0) == 255 ) {
                        distanceMap(x, y, 0) = pixel.dist;
                        while( !q.empty() ) {
                            q.pop();
                        } // while
                    } else {
                        visited (pixel.x, pixel.y, 0) = true;
                        pixel.dist++;
                        // push the 4 neighbours if in bounds
                        if ( pixel.x  > 0 && visited(pixel.x - 1, pixel.y, 0) == false ) {
                            pixel.x--;
                            q.push(pixel);
                            pixel.x++;
                        }
                        if ( pixel.x < size[DIM_X] - 1 && visited(pixel.x + 1, pixel.y, 0) == false ) {
                            pixel.x++;
                            q.push(pixel);
                            pixel.x--;
                        }
                        if ( pixel.y > 0 && visited(pixel.x, pixel.y -1 , 0) == false ) {
                            pixel.y--;
                            q.push(pixel);
                            pixel.y++;
                        }
                        if ( pixel.y < size[DIM_Y] - 1 && visited(pixel.x, pixel.y + 1, 0) == false ) {
                            pixel.y++;
                            q.push(pixel);
                        }
                    } // else
                } // while

                //erase vistited image
                for (int16 i = 0; i < size[DIM_X]; i++) {
                    for (int16 j = 0; j < size[DIM_Y]; j++) {
                        visited(i,j,0) = false;
                    }
                }
            } else {
                distanceMap(x, y, 0) = -100;
            } // else

        } // for
    } // for
} // process
