/*
    GeometricFeature.cpp     calculate geometric features of an object in a binary image
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



#ifndef GEOMETRIC_FEATURES_H
#define GEOMETRIC_FEATURES_H

// C++ includes
#include <iostream>
using namespace std;

// MoCA includes
#include <MocaTypes.h>
#include <OTagStream.h>
#include <ITagStream.h>
#include <Image.h>
#include <BinarizeFilter.h>
#include <RegionFeature.h>
#include <ContourEdgeList.h>


struct GeometricFeatureData
{
	GeometricFeatureData () 
	{
		compactness = 0.0;
		eccentricity  = 0.0;
		numberObjectPixel = 0;
		contourLength = 0;
		avgX = 0;
		avgY = 0;
		m20 = 0;
		m02 = 0;
		m11 = 0;
	} 
	ieee64 compactness;  // similarity to a circle
	ieee64 eccentricity; // relation of length of major axis
	int32 numberObjectPixel;
	ieee64 contourLength;
	ieee64 avgX;
	ieee64 avgY;
	ieee64 m20;
	ieee64 m02;
	ieee64 m11;
};
std::ostream& operator<< (ostream&, const GeometricFeatureData&);
OTagStream& operator<< (OTagStream&, const GeometricFeatureData&);
ITagStream& operator>> (ITagStream&, GeometricFeatureData&);



/*
	Constructor: initializes variables
	
	process(img, data): 
		calculate the geometric features
*/
class GeometricFeature
{
public:

	GeometricFeature() {};
	~GeometricFeature() {};

	// calculate the geometric features
	void process (const Image<uint8>&, GeometricFeatureData&);

private:

};

#endif
