/*
    HistogramDifferenceFeature.cpp     calculate teh difference of histograms
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// MoCA includes
#include <HistogramDifferenceFeature.h>



std::ostream& operator<< (ostream& os, const HistogramDifferenceItem &data)
{
	os << "msBits              : " << (int)data.msBits       << endl;
	os << "skipedFrames        : " << (int)data.skipedFrames << endl;
	if (data.grayscale) {
		os << "grayscale           : yes"  << endl;
	} else {
		os << "grayscale           : no"  << endl;
	} // else
	if (data.optimize) {
		os << "optimize            : yes"  << endl;
	} else {
		os << "optimize            : no"  << endl;
	} // else
	if (data.normalize) {
		os << "normalize           : yes"  << endl;
	} else {
		os << "normalize           : no"  << endl;
	} // else
	os << "areas               : " << data.areas             << endl;
	os << "number of frames    : " << data.frames            << endl;
	os << "minimum value       : " << data.minHistValue      << endl;
	os << "maximum value       : " << data.maxHistValue      << endl;
	os << "average value       : " << data.avgHistValue      << endl;
	os << "histogram file      : " << data.histFilename      << endl;

	if (data.metric==HDM_L1) os << "metric              : L1-norm" << endl;
	else if (data.metric==HDM_L2) os << "metric              : L2-norm" << endl;
	else if (data.metric==HDM_EMD) os << "metric              : earth movers distance" << endl;
	else os << "metric              : unknown" << endl;
	os << endl;

	return os;
} // operator<<

OTagStream& operator<< (OTagStream& os, const HistogramDifferenceItem& data)
{
    TagFormat::writeNodeValue(os, "msBits",       data.msBits);
    TagFormat::writeNodeValue(os, "skipedFrames", data.skipedFrames);
    TagFormat::writeNodeValue(os, "grayscale",    data.grayscale);
    TagFormat::writeNodeValue(os, "optimize",     data.optimize);
    TagFormat::writeNodeValue(os, "normalize",    data.normalize);
    TagFormat::writeNodeValue(os, "areas",        data.areas);
    TagFormat::writeNodeValue(os, "frames",       data.frames);
    TagFormat::writeNodeValue(os, "minHistValue", data.minHistValue);
    TagFormat::writeNodeValue(os, "maxHistValue", data.maxHistValue);
    TagFormat::writeNodeValue(os, "avgHistValue", data.avgHistValue);
    TagFormat::writeNodeValue(os, "histFilename", data.histFilename);

	int32 v=0;
	switch (data.metric) {
	case HDM_L1:
		v=1;
		break;
	case HDM_L2:
		v=2;
		break;
	case HDM_EMD:
		v=3;
		break;
	default:
		v=4;
		break;
	} // switch
    TagFormat::writeNodeValue(os, "metric",v);
	os << endl;

	return os;
} // operator<<

ITagStream& operator>> (ITagStream& is, HistogramDifferenceItem& data)
{
    char c;
    bool failFlag = false;

	if (is >> c) {
		is.putback(c);

		data.msBits             = 3;
		data.skipedFrames       = 0;
		data.grayscale          = false;
		data.optimize           = false;
		data.frames             = 0;
		data.minHistValue       = 1e10;
		data.maxHistValue       = -1 ;
		data.avgHistValue       = 0;
		data.histFilename       = "";
		data.metric             = HDM_L1;
		data.values.clear();

		TagFormat::readNodeValue(is, "msBits", data.msBits);
		TagFormat::readNodeValue(is, "skipedFrames", data.skipedFrames);
		TagFormat::readNodeValue(is, "grayscale", data.grayscale);
		TagFormat::readNodeValue(is, "optimize",  data.optimize);
		TagFormat::readNodeValue(is, "normalize", data.normalize);
		TagFormat::readNodeValue(is, "areas", data.areas);

		uint32 tmp = 0;
		TagFormat::readNodeValue(is, "frames", tmp);
		data.frames = (vector<ieee64>::size_type) tmp;
		TagFormat::readNodeValue(is, "minHistValue", data.minHistValue);
		TagFormat::readNodeValue(is, "maxHistValue", data.maxHistValue);
		TagFormat::readNodeValue(is, "avgHistValue", data.avgHistValue);
		TagFormat::readNodeValue(is, "histFilename", data.histFilename);

		int32 v=0;
		TagFormat::readNodeValue(is, "metric",v);
		switch (v) {
		case 1:
			data.metric=HDM_L1;
			break;
		case 2:
			data.metric=HDM_L2;
			break;
		case 3:
			data.metric=HDM_EMD;
			break;
		default:
			data.metric=HDM_L1;
			failFlag=true;
			cerr << "unknown value (data.metric)" << endl;
			break;
		} // case
		
		if (is.rdstate() & ios::failbit) {
			failFlag = true;
		} // if
	} // if

	data.values.clear();
    if (failFlag) {
		throw MocaException ("HistogramDifferenceFeature: Illegal histogram file format!");
	} // if
    is.clear();

    return is;
} // operator>> 






std::ostream& operator<< (ostream& os, const HistogramDifferenceData &data)
{
	os << "version          : " << data.version << endl;
	os << "filename         : " << data.filename<< endl;
	os << "numberOfElements : " << data.numberOfElements << endl;
	os << endl << endl;

	vector<HistogramDifferenceItem>::const_iterator it;
	for (it=data.hdi.begin(); it!=data.hdi.end(); it++) {
		os << (*it);
	} // for

	return os;
} // operator<<

OTagStream& operator<< (OTagStream& os, const HistogramDifferenceData& data)
{
    TagFormat::writeNodeValue(os, "version", data.version);
    TagFormat::writeNodeValue(os, "filename", data.filename);
    TagFormat::writeNodeValue(os, "numberOfElements", data.numberOfElements);
	os << endl;

	vector<HistogramDifferenceItem>::const_iterator it;
	for (it=data.hdi.begin(); it!=data.hdi.end(); it++) {
		os << (*it);
	} // for

	return os;
} // operator<<

ITagStream& operator>> (ITagStream& is, HistogramDifferenceData& data)
{
    char c;
    bool failFlag = false;

	if (is >> c) {
		is.putback(c);

		TagFormat::readNodeValue(is, "version", data.version);
		TagFormat::readNodeValue(is, "filename", data.filename);
		TagFormat::readNodeValue(is, "numberOfElements", data.numberOfElements);
		if (is.rdstate() & ios::failbit) {
			failFlag = true;
		} // if

		// read histogram differences
		data.hdi.clear();
		HistogramDifferenceItem tmp;

		bool endTagFound = false;
		while ((!endTagFound) && (!failFlag) && (is >> tmp))  {
			if (is.rdstate() & ios::failbit) {
				failFlag = true;
			} else {
				data.hdi.push_back(tmp);	
			} // if

			if (is >> c) {
				is.putback(c);
			} else {
				endTagFound = true;
			} // if
		} // while 
	} // if

    if (failFlag) {
		data.hdi.clear();
		throw MocaException ("HistogramDifferenceFeature: Illegal histogram file format!");
	} else {
        is.clear();
    } 

    return is;
} // operator>> 






const string HistogramDifferenceFeature::DEFAULT_HISTOGRAM_DIFFERENCE_FILE_NAME = "histogramDifference.data";
const int HistogramDifferenceFeature::MAX_SKIP_FRAME_SIZE = 65;



HistogramDifferenceFeature::
HistogramDifferenceFeature (string &pathFilename, uint8 msBits, 
							uint8 skip, bool gray, bool optimize, bool normalizeHistograms, int16 areas,
							HistogramDifferenceMetric histogramDifferenceMetric) 
{
	this->pathFilename = pathFilename;

	bits = msBits;
	skipedFrames = skip;
	grayscale = gray;
	this->optimize = optimize;
	this->normalizeHistograms = normalizeHistograms;
	this->areas = areas;
	if (!grayscale) {
		this->optimize = false;
		this->normalizeHistograms=false;
	} // if
	this->histogramDifferenceMetric = histogramDifferenceMetric;

	if (skipedFrames > MAX_SKIP_FRAME_SIZE) {
		cerr << "HistogramDifferenceFeature:  skipFrames too large!" << endl;
		cerr << "                             setting: skipFrames = " << MAX_SKIP_FRAME_SIZE << endl;	
		skipedFrames = MAX_SKIP_FRAME_SIZE;
	} // if	

	// get video size
	numberOfPixelFactor=1.0;
	{
		VideoDecoder tmpReader (pathFilename);
		ImageSize size = tmpReader.getImageSize();
		numberOfPixel=size[DIM_X]*size[DIM_Y];
		numberOfPixelFactor=1.0/numberOfPixel;

		{
			VideoStatisticData data;
			VideoStatistic vidStat (pathFilename);
			vidStat.process(data);
			gray = (data.color==TYPE_GRAYSCALE);
		}

		//if (tmpReader.isGrayscale()) gray=true;
	}

	// open codec
	VideoCodec codec = CODEC_YUV444;
	if (gray) codec = CODEC_Y;

	reader=NULL;
	reader = new VideoDecoder (pathFilename, 3, -1, codec);
	numberOfFrames=reader->getSize()[DIM_Z];
	nextImage = -1;
	nextPosition = -1;


	// how much memory is required, to keep the histograms in memory?
	ieee64 requiredMemory=1.0;
	if (gray) {
		// required memory for one frame
		requiredMemory=pow(2.0,(ieee64)msBits);
	} else {
		// required memory for one plane in one frame
		requiredMemory=pow(2.0,(ieee64)msBits);
		requiredMemory=pow(requiredMemory,3.0);
	} // else
	requiredMemory*=4.0; // int32


	// set buffer size
	bufferSize = skipedFrames+2;
	if (requiredMemory*(numberOfFrames+2) < HISTOGRAM_DIFFERENCE_MAX_MEMORY) {
		bufferSize = numberOfFrames + 2;
	} else {
		bufferSize = (int32)(HISTOGRAM_DIFFERENCE_MAX_MEMORY / requiredMemory);
		if (bufferSize<skipedFrames+2) {
			bufferSize=skipedFrames+2;
			requiredMemory*=(bufferSize/(1024.0*1024.0));
			cerr << "Warning in HistogramDifferenceFeature: " << (int32)requiredMemory << " MByte memory required." << endl << endl;
		} // if 
	} // else

	// allocate memory for histograms
	hist=NULL;
	hist = new Signal<uint32>[bufferSize]; 
	histFrame = NULL;
	histFrame = new int32[bufferSize]; 
	for (int32 i=0; i<bufferSize; i++) histFrame[i]=-1;
	histCalc = NULL;
	histCalc = new HistogramFeature (bits, grayscale, optimize, areas);
	int32 tmpFramePosition=-1;
	getHistogram (0, tmpFramePosition);
//for (int32 i=0; i<hist[tmpFramePosition].getSize()[DIM_X]; i++)
//	cerr << i << "  " << hist[tmpFramePosition](i) << endl;

	histSize = hist[tmpFramePosition].getSize()[DIM_X];
};




void
HistogramDifferenceFeature::process (HistogramDifferenceItem& item) 
{
	HistogramDifferenceData data;

	// read data
	ReadMocaData<HistogramDifferenceData> reader (pathFilename, HISTOGRAM_DIFFERENCE_FILENAME);
	bool dataValid = reader.process (HISTOGRAM_DIFFERENCE_VERSION, data); 

	bool itemExists = false;
	if (!dataValid) {
		data.version = HISTOGRAM_DIFFERENCE_VERSION;
		data.numberOfElements = 0;
		data.hdi.clear();

		// get path of filename, do not create new directory
		CreateMocaPath mocaPath (pathFilename, false);
		data.filename = mocaPath.file;
	} else {

		// check if item exists
		vector<HistogramDifferenceItem>::iterator it;
		for (it=data.hdi.begin(); it!=data.hdi.end(); it++) {
			
			// two items are equal, if bits, skipedFrames and grayscale is equal
			item = (*it);
			if ( (item.msBits              == bits)					&& 
				 (item.skipedFrames        == skipedFrames)			&&
				 (item.grayscale           == grayscale)			&&
				 (item.optimize            == optimize)				&& 
				 (item.normalize           == normalizeHistograms)  &&
				 (item.areas               == areas)				&&
				 (item.metric              == histogramDifferenceMetric)) {
				itemExists = true;
				break;
			} // if
			
		} //for
	} // else


	if (itemExists) {

		// read hist file
		CreateMocaPath mocaPath (pathFilename);
		try {
			ITagStream its (mocaPath.destinationPath + item.histFilename);
		    TagFormat::readNodeValue(its, item.values);
			its.close();
		} catch (MocaException ex) {
			cout << ex.what() << endl;
			itemExists = false;
		} // try
		if (item.values.size() <= 0) {
			itemExists = false;
		} // if
	    if (!itemExists) {
			cerr << "HistogramDifferenceFeature: Illegal histogram file format (" 
				 << mocaPath.destinationPath << item.histFilename 
				 << ")!" << endl;
		} // if
    } // else


	if ((!dataValid) || (!itemExists)) {

		// create new index
		cerr << "\rHistogramDifference: Creating index file ...        " << endl;
		readValues(data);
			
		// save index file
		WriteMocaData<HistogramDifferenceData> writer (pathFilename, HISTOGRAM_DIFFERENCE_FILENAME);
		writer.process(data);

		// get item
		vector<HistogramDifferenceItem>::iterator it;
		for (it=data.hdi.begin(); it!=data.hdi.end(); it++) {
			
			// two items are equal, if bits, skipedFrames and grayscale is equal
			item = (*it);
			if ( (item.msBits == bits)						&& 
				 (item.skipedFrames == skipedFrames)		&&
				 (item.grayscale    == grayscale)			&&
				 (item.optimize     == optimize)			&& 
				 (item.normalize    == normalizeHistograms) &&
				 (item.areas        == areas)				&&
				 (item.metric       == histogramDifferenceMetric)) {
				itemExists = true;
				break;
			} // if
		} // for

		cerr << "\rHistogramDifference: Creating index file ... done        " << endl;
	} // if

	return;
}


void
HistogramDifferenceFeature::getHistogram (int32 frame, int32 &framePosition) 
{
	// get position of histogram
	framePosition=-1;
	for (int32 i=0; i<bufferSize; i++) {
		if (histFrame[i] == frame) {
			framePosition=i;
			return;
		} // if
	} // for

	// histogram not calculated?
	if (framePosition==-1) {

		// image already processed?
		if (nextImage > frame) {
			if (reader!=NULL) delete reader;
			reader = NULL;
			VideoCodec codec = CODEC_YUV444; 
			if (grayscale) codec = CODEC_Y; 
			reader = new VideoDecoder (pathFilename, 3, -1, codec);
			nextImage = -1;
		} // if


		// read next image
		while (nextImage <frame) {
			nextImage++;
			nextPosition=(nextPosition+1)%bufferSize;
			img = reader->getImage(nextImage);
			histCalc->process (img, hist[nextPosition]);
			histFrame[nextPosition]=nextImage;
		} // while

		// get position of histogram
		framePosition=-1;
		for (int32 i=0; i<bufferSize; i++) {
			if (histFrame[i] == frame) {
				framePosition=i;
				return;
			} // if
		} // for
	} // if
	if (framePosition==-1) {
		cout << endl;
	} // if

	return;
} // getHistogram 

void
HistogramDifferenceFeature::readValues (HistogramDifferenceData& data) 
{
	//Signal<int32> histogram;
	//HistogramFeature<uint8> histCalc (bits, grayscale, optimize, areas);
	//histCalc.process (filename, histogram);
	//SignalSize size =histogram.getSize();

	for (int32 skip=skipedFrames; skip>=0; skip--) {

		// does item exist?
		// two items are equal, if bits, skipedFrames, grayscale, and optimize is equal
		bool itemExist=false;
		vector<HistogramDifferenceItem>::iterator it;
		for (it=data.hdi.begin(); it!=data.hdi.end(); it++) {			
			if ( ((*it).msBits       == bits)				 && 
				 ((*it).skipedFrames == skip)				 &&
				 ((*it).grayscale    == grayscale)			 &&
				 ((*it).optimize     == optimize)			 && 
				 ((*it).normalize    == normalizeHistograms) &&
				 ((*it).areas        == areas)               && 
				 ((*it).metric       == histogramDifferenceMetric)) {
				itemExist = true;
				break;
			} // if
		} // for

		if (!itemExist) {
			// initialize values
			HistogramDifferenceItem item;
			item.msBits       = bits;
			item.skipedFrames = skip;
			item.grayscale    = grayscale;
			item.optimize     = optimize;
			item.normalize    = normalizeHistograms;
			item.areas        = areas;
			item.frames       = numberOfFrames;
			item.minHistValue = 0.0;
			item.maxHistValue = 0.0;
			item.avgHistValue = 0.0;
			item.histFilename = "";
			item.metric       = histogramDifferenceMetric;
			item.values.resize(item.frames);

			// calculate the HISTOGRAM-DIFFERENCE (HD) for all frames in the video
			calculateHistogramDifference(item, skip);
			

			// define histogramm filename
			CreateMocaPath mocaPath (pathFilename);
			{
			  stringstream oss;
			  oss << "hist" << setw(4) << setfill('0') << (data.hdi.size()) << ".idx";
			  item.histFilename = oss.str();
			}			
			data.hdi.push_back(item);
			data.numberOfElements = data.hdi.size();

			// write hist file
			OTagStream ots (mocaPath.destinationPath + item.histFilename);
			TagFormat::writeNodeValue(ots, item.values);
			ots.close();
		} // if
	} // for
	
	return;
} // process




void
HistogramDifferenceFeature::normalize (Signal<uint32> &hist1, Signal<uint32> &hist2) 
{
	//int32 a1=hist1(0);
	//int32 a2=hist1(1);
	//int32 a3=hist2(0);
	//int32 a4=hist2(1);


	// get average histogram value of frame i and p
	int32 avg1=0;
	int32 avg2=0;
	for (int32 j= 0; j<histSize; j++) {
		avg1 += j*hist1(j);
		avg2 += j*hist2(j);
	} // for
	int32 diff=0;
	if (avg2>avg1) {
		diff=(int32)(0.5+(avg2-avg1) / numberOfPixel);
	} else {
		diff=(int32)(-0.5+(avg2-avg1) / numberOfPixel);
	} // else
	
	// modify values of hist1 if required
	if (diff>=1) {
		SignalSize tmpSize(histSize);
		Signal<int32> tmp (tmpSize);
		int counter=0;
		while (diff>=1) {
			counter++;
			if (counter>10) break;
			
			// initialize tmp
			for (int32 j= 0; j<histSize; j++) {
				tmp(j)=0;
			} // for
			
			// copy shifted values to tmp
			for (int32 j= 0; j<histSize; j++) {
				int32 destP=j+diff;
				if (destP<0) destP=0;
				if (destP>=histSize) destP=histSize-1;
				tmp(destP)+=hist1(j);
			} // for
			
			// copy values back
			for (int32 j= 0; j<histSize; j++) {
				hist1(j)=tmp(j);
			} // for
			
			// remove half of peak at first position in hist1 and values in hist2
			if (hist1(0)>0) {
				int32 reduce=hist1(0)/2;
				hist1(0)-=reduce;
				int32 posP=0;
				while (reduce>0) {
					if ((int32)hist2(posP)>reduce) {
						hist2(posP)-=reduce;
						reduce=0;
					} else {
						reduce-=hist2(posP);
						hist2(posP)=0;
					} // else
					posP++;
				} // while
			} // if
			
			// remove half of peak at last position in hist1 and values in hist2
			if (hist1(histSize-1)>0) {
				int32 reduce=hist1(histSize-1)/2;
				hist1(histSize-1)-=reduce;
				int32 posP=histSize-1;
				while (reduce>0) {
					if ((int32)hist2(posP)>reduce) {
						hist2(posP) -=reduce;
						reduce=0;
					} else {
						reduce-=hist2(posP);
						hist2(posP)=0;
					} // else
					posP--;
				} // while
			} // if
			
			// count number of pixel
			int32 num=0;
			for (int32 j= 0; j<histSize; j++) {
				num+=hist1(j);
			} // for
			if (num<=1) num=1;
			
			// calculate average and diff again
			avg1=0;
			avg2=0;
			for (int32 j= 0; j<histSize; j++) {
				avg1 += j*hist1(j);
				avg2 += j*hist2(j);
			} // for
			diff=0;
			if (avg2>avg1) {
				diff=(int32)(0.5+(avg2-avg1) / numberOfPixel);
			} else {
				diff=(int32)(-0.5+(avg2-avg1) / numberOfPixel);
			} // else
		} // while
	} // if
	
	return;
} // HistogramDifferenceFeature::normalizeHistograms 
	
	
	
void
HistogramDifferenceFeature::calculateHistogramDifference (HistogramDifferenceItem &item, int32 skip) 
{
	
	// calculate the HISTOGRAM-DIFFERENCE (HD) for all frames in the video
	ieee64 vMin = 10e10;
	ieee64 vMax=  -1.0;
	ieee64 vAvg = 0;
	int32 n=0;
	ieee64 divFactor = 1.0 / numberOfPixel;
	
		
	Signal<uint32> hist1;
	Signal<uint32> hist2;
	if (normalizeHistograms) {
		SignalSize size(histSize);
		hist1.resize(size);
		hist2.resize(size);
	} // if


	for (uint32 i= 0; i<item.frames; i++) {
		uint32 p = i+skip+1;
		if (p>=item.frames) {
			item.values[i] = 0.0;
		} else {
			
			int32 framePos1=-1;
			int32 framePos2=-1;
			getHistogram (i, framePos1);
			getHistogram (p, framePos2);
			
			
			int32 sum = 0;
			switch (histogramDifferenceMetric) {
			case HDM_L1: // L1 norm
				for (int16 a=0; a<areas; a++) {
					
					
					// calculate the HISTOGRAM-DIFFERENCE (HD) following the definition 
					// given in the paper of Rainer Lienhart
					if (normalizeHistograms) {
						// copy hist values
						for (int32 j= 0; j<histSize; j++) {
							hist1(j) = hist[framePos1](j, a);
							hist2(j) = hist[framePos2](j, a);
						} // for
						normalize (hist1, hist2);
						for (int32 j= 0; j<histSize; j++) {
							sum += labs (hist1(j) -hist2(j));
						} // for
						
					} else {
						for (int32 j= 0; j<histSize; j++) {
							sum += labs (hist[framePos1](j, a) -hist[framePos2](j, a));
						} // for
					} // else
				} // for
				break;
			case HDM_L2: // L2 norm
				for (int16 a=0; a<areas; a++) {
					
					
					// calculate the HISTOGRAM-DIFFERENCE (HD) following the definition 
					// given in the paper of Rainer Lienhart
					if (normalizeHistograms) {
						// copy hist values
						for (int32 j= 0; j<histSize; j++) {
							hist1(j) = hist[framePos1](j, a);
							hist2(j) = hist[framePos2](j, a);
						} // for
						normalize (hist1, hist2);
						for (int32 j= 0; j<histSize; j++) {
							int32 d=(hist1(j) -hist2(j));
							sum +=  d*d;
						} // for
						
					} else {
						for (int32 j= 0; j<histSize; j++) {
							int32 d=(hist[framePos1](j, a) -hist[framePos2](j, a));
							sum += d*d;
						} // for
					} // else
				} // for
				break;
			case HDM_EMD: // Earth movers distance
				for (int16 a=0; a<areas; a++) {
					
					
					// calculate the HISTOGRAM-DIFFERENCE (HD) following the definition 
					// given in the paper of Rainer Lienhart
					if (normalizeHistograms) {
						// copy hist values
						for (int32 j= 0; j<histSize; j++) {
							hist1(j) = hist[framePos1](j, a);
							hist2(j) = hist[framePos2](j, a);
						} // for
						normalize (hist1, hist2);
						int32 sum0=0;
						int32 sum1=0;
						for (int32 j= 0; j<histSize; j++) {
							sum0 += hist1(j);
							sum1 += hist2(j);
							sum += labs (sum0 - sum1);
						} // for
						
					} else {
						int32 sum0=0;
						int32 sum1=0;
						for (int32 j= 0; j<histSize; j++) {
							sum0 += hist[framePos1](j, a);
							sum1 += hist[framePos2](j, a);
							sum += labs (sum0- sum1);
						} // for
					} // else
				} // for
				break;
			} // switch

			
			item.values[i] = divFactor*sum;				
			if (vMin > item.values[i]) {
				vMin = item.values[i];
			} // if
			if (vMax < item.values[i]) {
				vMax = item.values[i];
			} // if
			vAvg += item.values[i];
			n++;
			
		} // else
		cerr << "\rframe: " << i << "    "<< ends;
	} // for



	if (vMin >= 0.9*10e10) vMin=0.0;
	if (vMax <= 0.0) vMax=0.0;
	if (n==0) n=1;
	
	item.minHistValue = vMin;
	item.maxHistValue = vMax;
	item.avgHistValue = vAvg / ((ieee64)n);
	return;
} // HistogramDifferenceFeature::calculateHistogramDifference () 


