/*! \file
 * 
   \author    Manuel Knaus

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

*/

// MoCA includes
#include <HistoricVideoOptimization.h>

HistoricVideoOptimization::HistoricVideoOptimization() 
{
  /* Global static variables and constants*/
  yMaxPos = 0;  //needed for line detetion
  TESTRUN = false; //true, if its a test
  horizontalLineDetected = false; //only for shell output needed
  maxCorrFactorOfActualCutDirectionX = 0;
  maxCorrFactorOfActualCutDirectionY = 0;
  minCorrFactorOfActualCutDirectionX = 0;
  minCorrFactorOfActualCutDirectionY = 0;
  THRESBIGMOVEMENT = 32.0;  //15 //20
  doLineDetection = true; //possibility to camcel a special process
  doShakingDetection = true;
  doLumContrastCorrection = true;  
}
/* Main Method of Historic Video Optimization Process
 * input: bools, wheather single corrections have to be done
 * output: none
 * */
void HistoricVideoOptimization::process(string &inputFileName, 
                                        bool doLineDetection1, 
                                        bool doShakingDetection1,
                                        bool doLumContrastCorrection1)
{
  /* Overgiven methode values are given to global varaibles */
  doLineDetection = doLineDetection1;
  doShakingDetection = doShakingDetection1;
  doLumContrastCorrection = doLumContrastCorrection1;
    
  cout << "\nHistoric video quality improving in progress...\n"  ;
  /* Video Decoder settings*/
  //string inputFileName = "C:/programme/cygwin/home/mali/libmoca/examples/quality6.mpg"; 
  cout << "\nOpen file: " << inputFileName << "\n";
  const int32 CACHE = 100; //number of frames in cache  

  VideoDecoder reader (inputFileName); //reader for first rum
  VideoDecoder reader2 (inputFileName); //reader for second run



  //const int32 LASTFRAME = 250; //Last frame to decode
  
    //if you only want to have one raeder, you must make CACHE=LASTFRAME and 
    //raplace "reader2" with reader in the code below
  
  VideoSize size = reader.getSize();
  ImageSize imgSize = reader.getImageSize();
  //yCounter.resize(size[DIM_Y]); //size of global varaible is set

  /* Video Encoder settings */
  cout << "open encoder\n";
  VideoEncoderOptions options;
  //setting up output filename; extenson is replaced by "_new.avi"     
  string outputFileName = inputFileName.substr(0,inputFileName.rfind(".")) 
                          + "_new.avi"; 
  options.pathFilename = outputFileName;
  //change size for double out vid  
  ImageSize imgSizeDbl = imgSize; 
  imgSizeDbl[DIM_X] = 2 * imgSize[DIM_X];   
  options.size = imgSize; //settings for NOT double vid  (setting 1 of 2)
  //options.size = imgSizeDbl; //settings for double out vid (setting 1 of 2)

  // define video coding settings
  options.quality = QUALITY_HIGH;
  options.bitrate = 6000;

  // open encoder
  VideoEncoder writer(options);
        
  /* Initialising image classes */ 
  Image<uint8> img; 
  Image<uint8> simg; //source image
  Image<uint8> dblimg (imgSizeDbl); //Image containig two images, the source and the changed one
  
  /* Video size */
  int32 vidsize = 0;
  vidsize = size[DIM_Z]; //if going through the complete video
  //vidsize = LASTFRAME;  //only interested in first LASTFRAME frames
  
  /* Statistic data */
  ImageStatistic stat;  
  ImageStatisticData statdat;  
  
  /* Cut detection*/   
  CutDetectionData cutdata;
  {
    CutDetection detect(inputFileName);
    detect.process(cutdata);
  }
  //cout << cutdata;
  //cout << "\nSIZE=" << (int) size[DIM_X]; //=352
  /* Go through all frames (READ ONLY) and get some lum data */ 
  cout << "\nFirst run through the video...\n";
  cerr << "\n==========================================================================\n";
  int *lumMaxArray = new int[vidsize+1]; //Stores the maxLum values of each sequence of vid
  int *lumMinArray = new int[vidsize+1];  
  int actualCut = 0; //the game just begins, there is no cut yet
  int lumMax = 0;  //start value of maximum luminance 
  int lumMin = 255; //start value of min luminance   
  do  /* Go through one sequence (until next hard cut) and find some data */
  {
     int nextCut = findNextCut(actualCut, cutdata, vidsize);   
     for (int32 i=actualCut;i<nextCut;i++) //alle frames until next hard cut 
     {
        cerr << ".";                
        img = reader.getImage(i);     // get image
        stat.process(img, statdat); //gets the statistics for current image         
        lumMax = findMaxLum(statdat, lumMax);
        lumMin = findMinLum(statdat, lumMin);
        if (doLineDetection)
        {
          presearchHorizontalLine(img, size);
        }
       if (i == 0) //only for nice shell output
       {
         cerr << "==========================================================================\n";
         cerr << "\nProcessing frames";
       }         
     }    
     //cerr << "lumMax=" << (int) lumMax; //TEST     
     lumMaxArray[nextCut] = (int) lumMax; //lumMax of current sequence ist computed and stored
     lumMinArray[nextCut] = (int) lumMin;
     actualCut = nextCut;                        
  }
  while (actualCut != vidsize); //end of video has been reached

  /* Preparations for line detection */
  yMaxPos = (int) yMax();

  
  /* TEST outputs*/
  cout << "\nFirst run through the video complete!\n";
  cout << "\nStarting video manipulation process...\n";  
  //cout << "\nlumMax: >" << (int) lumMax << "< \n";    
  //cout << "lumMin: >" << (int) lumMin << "< \n";        
  //cout << "d: >" << (int) (abs ( lumMax - lumMin)) << "< \n";   
  
  /* Go through all frames (MANIPULATING FRAMES)*******************************/ 
  actualCut = 0;
  double sumX = 0; //TEST
  double sumY = 0; //TEST
  do 
  {
     /* Shell outputs */
     int nextCut = findNextCut(actualCut, cutdata, vidsize);
     cerr << "\n----------------------------------------"
          << "---------------------------------------";
     cerr << "\n----------------------work between cut=" << actualCut 
          << " and cut=" << nextCut << " -------------------------\n";
  

     /* Anti Shaking preparations and presearch */       
     CameraModelData cmData;
     //bool isShakingX = false;
     //bool isShakingY = false;
     int middleShakingPosX = -1;
     int middleShakingPosY = -1;
     if (doShakingDetection)
     {  
       int32 cropleft  =4; // remove border
       int32 cropright =4;
       int32 croptop   =4;
       int32 cropbottom=4;
       int32 defaultMaxMotionVectorLength = 60;
       bool calcTranslationOnly = true;       
       CameraModel cmReader (inputFileName, cropleft, cropright, croptop, 
                             cropbottom, defaultMaxMotionVectorLength, 
                             calcTranslationOnly);
       cmReader.process (cmData);  
       //tests, if current cut is shaking in x or y direction, yes if != -1    
       middleShakingPosX = preprocessAntiShaking(actualCut, nextCut, true, cmData);
       middleShakingPosY = preprocessAntiShaking(actualCut, nextCut, false, cmData);        
       if (middleShakingPosX != -1) //shaking in x direction
       {
         cerr << "\nShaking until cut=" << nextCut <<
                 " in x direction will be corrected.\n" ;         
       }  
       if (middleShakingPosY != -1) //shaking in y direction
       {
         cerr << "\nShaking until cut=" << nextCut <<
                 " in y direction will be corrected.\n" ;
       } 
     }
     /* Lum and contrast checks and outputs */
     bool changeLuminance = checkLumImbalance(lumMaxArray[nextCut], 
                                              lumMinArray[nextCut]);       
     bool overbright = checkOverbright(lumMaxArray[nextCut], lumMinArray[nextCut]);
     bool overdark = checkOverdark(lumMaxArray[nextCut], lumMinArray[nextCut]);
     if (overbright)
     {
       cerr << "\nOverbright correction in current cut.\n" ;
     }
     else if (overdark)
     {
       cerr << "\nOverdark correction in current cut.\n" ;
     }           
     cerr << "\n===========================================================================\n";
     
     /* Go through all frames until next cut */
     for (int32 i=actualCut;i<nextCut;i++) 
     {               
       /* get image */
       img = reader2.getImage(i); 
       simg = img; //source image needed for the double output img 
       
       /* Finding horizontal line in the image and correct it */
       if (doLineDetection)
       {
         img = findHorizontalLine(img, size);                
       }
       
       /* Change of lum if necessary and wished by the user */
       stat.process(img, statdat);             
       int actlum = (int) statdat.avg[1]; //also needed later in program!
       
       if (changeLuminance && doLumContrastCorrection) 
       {
           int lumChangeRate = computeLumChangeRate(actlum);   
           LuminanceFilter lum(lumChangeRate); //value for lum change
           lum.process(img);
           //After the change of luminance there is a need to adapt the
           //contrast also, because of sharpness
           int contrastChangeRate = 
                         computeContrastChangeRate(0, 0, actlum, false, false, 
                                                   true, lumChangeRate);          
           ContrastFilter contr(contrastChangeRate);
           contr.process(img);           
           //cerr << " Lum=" <<  changeRate << " ";
       }

       /* Change of contrast if necessary and wished by the user */       
       if ( (overbright || overdark) && doLumContrastCorrection) 
       {
           int contrastChangeRate = 
                     computeContrastChangeRate(lumMaxArray[nextCut], 
                                               lumMinArray[nextCut], 
                                               actlum, overbright, overdark, 
                                               false, 0);   
           ContrastFilter contr(contrastChangeRate);
           contr.process(img);
           //cerr << " contr=" << changeRate;
       }
        
       /* Anti-shaking corrections */                          
       if ( (middleShakingPosX != -1) && doShakingDetection) // x direction
       { 
         //cerr << "\nMotion vektor x frame " << i << "=" << cmData.data[i].transData.tx ;
         img = processAntiShaking(i, true, cmData, img, middleShakingPosX, 
                                  size); 
       }
       if ( (middleShakingPosY != -1) && doShakingDetection) //y direction
       {
         //cerr << "\nMotion vektor y frame " << i << "=" << cmData.data[i].transData.ty ;
         img = processAntiShaking(i, false, cmData, img, middleShakingPosY, 
                                  size);        
       }
       
       /* Copy two images to a double out vid */ 
       dblimg.copy(simg, SampleBlockPosition(0,0,0,0));
       dblimg.copy(img, SampleBlockPosition(size[DIM_X],0,0,0));
    
       /* store as video */
       //writer.write(dblimg); //for double img video (setting 2 of 2)
       writer.write(img); //for single img video (setting 2 of 2)
       
       /* store individual images */      
       //string fn = "C:/programme/cygwin/home/mali/libmoca/examples/img/img";
       //string imgFn = FileHandling::getNumberedFilename(fn,i,".jpg");
       //WriteImage (dblimg,imgFn); //write only new image 
       if (i == 0) //only for nice shell output
       {
         cerr << "===========================================================================\n";
         cerr << "\nCorrecting frames:";
       }
       cerr << " " << i << ends;      
     } //end for     
     actualCut = nextCut;
     cerr << "CUT\n";
  }  
  while (actualCut != vidsize); //end of sequence has been reached   
  
  /***************************************************************************/
  /* final shell outputs */
  cerr << "\nAll corrections complete.\n";
  if (horizontalLineDetected && doLineDetection)
  {    
    cerr << "\nThere was a horizontal white scrambling line. It has been corrected.\n";
  }
  else
  {
    cerr << "\nThere was NO horizontal white scrambling line.";
  }
  cout << "\nVideo improvement complete.";
  cerr << "\nVideo stored as " << outputFileName << "\n";
  //cerr << "\nIf necessary please terminate the program manually!";
  cerr << endl; 
  
  delete[] lumMaxArray;
  delete[] lumMinArray;
  return;
}


/* Check for luminace imbalance 
 * input: Max and Min luminace of video
 * output: boolean if lum has to be changed
 * */
bool HistoricVideoOptimization::checkLumImbalance(int lumMax, int lumMin)
{
  const int32 DTHRESIMBALANCE = 20;
  int32 d = abs (lumMax - lumMin);  
  bool changeLuminance = false; //boolean, true if lum has to be changed
  if (d > DTHRESIMBALANCE) //there is an imbalance
  {
    changeLuminance = true;
  }  
  cerr << "\nLuminance change=" << changeLuminance;      
  return(changeLuminance);
}

/* Finds the maximum luminance in a sequence (one cut)
 * input: image, current maxLum
 * output: new maxLum
 * */
int HistoricVideoOptimization::findMaxLum(ImageStatisticData &statdat, int lumMax)
{
  int actlum = (int) statdat.avg[1]; //luminace of current image
  if (actlum >= lumMax)
  {
      lumMax = actlum;
  } 
  return (lumMax); 
}

/* Finds the minimum luminance in a sequence (one cut)
 * input: image, current minLum
 * output: new minLum
 * */
int HistoricVideoOptimization::findMinLum(ImageStatisticData &statdat, int lumMin)
{  
  //ImageStatistic stat;  
  //ImageStatisticData statdat;
  //stat.process(img, statdat); //gets the statistics for current image
  int actlum = (int) statdat.avg[1]; //luminace of current image
  if (actlum <= lumMin)
  {
      lumMin = actlum;    
  }  
  return (lumMin); 
}

/* Goes through the cutdata and search for the next hard cut
 * input: last hard cut, cutDetection data, vidsize
 * output: position of next hard cut
 * */
 
int HistoricVideoOptimization::findNextCut(int actualCut, CutDetectionData &cutdata, int vidsize)  
{
  for (int a=actualCut+1; a<vidsize; a++)
  {
    if (cutdata.cuts(a) == 2) //there is a hard cut
    {
      return(a);  
    }
  }  
  return(vidsize); //no more cuts in sequence 
}
  
/* Check for overbrightness 
 * input: Max and Min luminace of video
 * output: boolean if lum has to be changed
 * */  
bool HistoricVideoOptimization::checkOverbright(int lumMax, int lumMin)
{  
  const int THRESOVERBRIGHT = 110; //naja
  const int THRESNOOVERCORECTION = 40; //ok
  int32 d = abs ( lumMax - lumMin);
  bool overbright = false; //default
  if ((lumMax >= THRESOVERBRIGHT) && (d <= THRESNOOVERCORECTION))
  {
    overbright = true;
  }
  //cerr << "\nCheck overbright, values: d=" << (int) d << " lumMax=" << (int) lumMax << " lumMin=" << (int) lumMin; 
  cerr << "\noverbright=" << overbright;
  return (overbright);
}

/* Check for overdarkness 
 * input: Max and Min luminace of video
 * output: boolean if lum has to be changed
 * */  
bool HistoricVideoOptimization::checkOverdark(int lumMax, int lumMin)
{  
  const int THRESOVERDARK = 70; //muss noch austariert werden
  const int THRESNOOVERCORECTION = 40;
  int32 d = abs ( lumMax - lumMin);
  bool overdark = false; //default
  if ((lumMax <= THRESOVERDARK) && (d <= THRESNOOVERCORECTION))
  {
    overdark = true;
  }
  //cout << "\nd=" << (int) d << " lumMax=" << (int) lumMax; 
  cerr << "\noverdark=" << overdark;
  return (overdark);
}

/* Computes the lum change rate for a single frame
 * input: luminace of current frame
 * output: lum change rate for curent frame
 */
int HistoricVideoOptimization::computeLumChangeRate(int actLum)
{
  const int32 UPPERBOUND = 110; //Upper bound for lum imbalance correction 
  const int32 LOWERBOUND = 90; 
  int32 lumChangeRate = 0; //default, no change between UPPER and LOWERBOUND   
  if (actLum < LOWERBOUND) //dark image
  {
    lumChangeRate =  (int) 0.8 * (LOWERBOUND - actLum); //lum change rate   
  }
  else if (actLum > UPPERBOUND) //bright image
  {
    lumChangeRate = UPPERBOUND - actLum; //lum change rate, negativ number   
  }   
  return(lumChangeRate);       
}

/* Computes the contrast change rate for a single frame
 * input: lumMax and lumMin of current cut
 *        actual lum
 *        bools which case of change is in progress
 *        prevousely computed lumChageRate
 * output: lum change rate for curent frame
 */
int HistoricVideoOptimization::computeContrastChangeRate(int lumMax, int lumMin, int actLum, bool overbright, 
                              bool overdark, bool changeLum, int lumChangeRate)
{
  int32 d = abs ( lumMax - lumMin);  
  int contrastChangeRate = 0;
  if (overbright)
  {
    contrastChangeRate = 201 - 5 * d;
  }
  if (overdark)
  {
    contrastChangeRate = -(41 - d);
  }
  if (changeLum)
  {
    if (lumChangeRate > 0) //dark image
    {
      contrastChangeRate = (int) - (0.5 * lumChangeRate); 
    }
    else if (lumChangeRate < 0) //bright image
    {
      contrastChangeRate = (int) - (0.5 * lumChangeRate);
    }   
  }
  return(contrastChangeRate);       
}

/* Computes a value between a and b. If the factor is 0.5, the new value
 * is the average of a and b. Otherwise the higher the factor, the nearer the 
 * new value is the a value.
 * input: two pixel values a and b, factor
 * */
int HistoricVideoOptimization::averageValue(int a, int b, double factor)
{
  int newValue = 0;
  newValue = (int) floor( (a-b) * factor + b );
  return newValue;  
}

/* Checks the pixel of an image if its possibly part of a horizonal white line
 * input: some pixel values of some positions of the frame
 * output: true or false
 * */
bool HistoricVideoOptimization::checkPixel(int32 posAct, int32 posLeft, int32 posRight, int32 pos2Left, int32 pos2Right, int32 pos2Above, int32 pos2Below)
{
   const int32 WHITETHRES = 230; //former value 240 
   const int32 VERTICALDIFFERENCETHRES = 20; //former value 10  
   bool white = false;     
   //pixel value is nearly white and horizontal neighbour pixels are nearly white
   if ( (posAct > WHITETHRES) || (posLeft > WHITETHRES) || (posRight > WHITETHRES) 
         || ( (pos2Left > WHITETHRES) && (pos2Right > WHITETHRES) ) ) 

   {
      white = true;
   }      
   /* must be a horizontal line */
   bool notvertical = false; 
   if ( (abs(posAct-pos2Above) > VERTICALDIFFERENCETHRES) 
         && (abs(posAct-pos2Below) > VERTICALDIFFERENCETHRES) )
         /*Explanation: If Pixel above/ below (the actual pixel) has nearly 
          * the SAME color, it is NOT a horizontal line but a area or a 
          * vertical line. 
          * */  
      {
        notvertical = true;
      } 
    //end if
   if (white && notvertical)
   {
      return (true);
   }
   else
   {
      return (false);
   }   
}

/* Checks for nearly black pixel (of horizontal white line)
 * input: some pixel values of some positions of the frame
 * output: true or false
 * */
bool HistoricVideoOptimization::check4BlackPixel(int32 posAct, int32 pos3Left, int32 pos3Right, int32 pos2Above, int32 pos2Below)
{
   const int32 BLACKTHRES = 100; 
   const int32 VERTICALDIFFERENCETHRES = 20; 
   const int32 HORIZONTALDIFFERENCETHRES = 40;    
   bool black = false;     
   //pixel value is nearly white and horizontal neighbour pixels are nearly white
   if ( (posAct < BLACKTHRES) ) 
   {
      black = true;
   }     
   else 
   {
     return false; //no further tests necessary
   } 
   /*  */
   bool singleBlackPixel = false;
   if ( (abs(posAct-pos3Left) > HORIZONTALDIFFERENCETHRES) && 
        (abs(posAct-pos3Right) > HORIZONTALDIFFERENCETHRES) && 
        (abs(posAct-pos2Above) > VERTICALDIFFERENCETHRES) && 
        (abs(posAct-pos2Below) > VERTICALDIFFERENCETHRES)     )
   {
      singleBlackPixel = true; 
   }   
    //end if
   if (black && singleBlackPixel)
   {
      return (true);
   }
   else
   {
      return (false);
   }   
}

/** First Step in the horizontal Line search
 * (No image corrections by now)
 * 
 * input: image, size
 * output: directly to static array yCounter
 */
void HistoricVideoOptimization::presearchHorizontalLine(Image<uint8> &img, VideoSize &size)
{
   //int yCounter[size[DIM_Y]]; //array counts the possibly lines    
   for (int32 y=0; y<size[DIM_Y]; y++) //go through all lines 
   {  
      for (int32 x=0; x<size[DIM_X]; x++) 
      {          
         bool possiblyPixel = 
             checkPixel(img(x,y,0), img(x-1,y,0), img(x+1,y,0), img(x-2,y,0), 
                        img(x+2,y,0) ,img(x,y-2,0), img(x,y+2,0));                     
         if (possiblyPixel) //current pixel is possibly a pixel of a line            
         {    
            yCounter[y]++; //counts 1 to the y value --> increasing chance for line
            break; //go to next y value, break the x for loop
         }
      }
   }    
   return; 
}


/* Search an image for a horizontal white line
 * and do some corrections
 * input: source image, video size
 * output: corrected image
 * */
Image<uint8> HistoricVideoOptimization::findHorizontalLine(Image<uint8> &img, VideoSize &size)
{
   int32 DIFFFIRSTLASTTHRES = 50;//(int) (size[DIM_X] / 4);  //line must be at least one quarter (f: half) of the screen
   const int8 LINECOUNTERTHRES = 30;  //15     
   int YDIFFTHRES = 10; //allowed zone for line
   //const int BLACKTHRES = 30;
   int32 lineCounter = 0;         //counts the white pixels in a line
   int32 firstOccurrence = size[DIM_X];  //first/ last detection of possible line pixel
   int32 lastOccurrence = 0;   //
   //bool lineArray[size[DIM_X]]; //bool array storing if x is white
             //for (int32 y=0; y<size[DIM_Y]; y++) //go through all lines   
   /* Only go through the lines near most possible pixel line */
   for (int y=yMaxPos-YDIFFTHRES; y<yMaxPos+YDIFFTHRES; y++) 
   {
       lineCounter = 0; //next y, reset lineCounter
       int8 blackPixelCounter = 0;
       firstOccurrence = 0; //reset
       for (int32 x=0; x<size[DIM_X]; x++) 
       {          
         bool possiblyPixel = 
             checkPixel(img(x,y,0), img(x-1,y,0), img(x+1,y,0), img(x-2,y,0), 
                        img(x+2,y,0) ,img(x,y-2,0), img(x,y+2,0));    
         bool blackPixel = 
              check4BlackPixel(img(x,y,0), img(x-3,y,0), img(x+3,y,0), 
                               img(x,y-2,0), img(x,y+2,0));          
         if (blackPixel)
         {             
           lineCounter++;                    
         }                                                                
         if (possiblyPixel) //current pixel is possibly a pixel of a line            
         {
           lineCounter++;             
           if (x < firstOccurrence) 
           {
              firstOccurrence = x;  
           } 
           lastOccurrence = x;  //current pixel is always last pixel in line
              //TEST
              //img(x,y,0) = 250; //R  //yellow
              //img(x,y,1) = 250;  //G
              //img(x,y,2) = 40;  //B             
         }        
       } //end for x
       int8 diffFirstLast = lastOccurrence - firstOccurrence;
       
       /* Check if white line, if yes correct the pixels */
       if ( ((lineCounter > LINECOUNTERTHRES) ) 
         && (lastOccurrence > DIFFFIRSTLASTTHRES) )
       {
         horizontalLineDetected = true; //only needed for shell output
          
         /* Do pixel based corrections */
         const double VERT1 = 0.3; //percent value of original
         for (int x=0; x<size[DIM_X]; x++)
         {           
           /* beginning of the line */
           if (x<firstOccurrence && x> firstOccurrence-80)
           {
             /* The farer away the pixel is from the firstOccurrence, the less
              * the value is changed */
             int factor = 1 - ( (firstOccurrence-x) / 80); //percent of new value relative to original value 
             img(x,y,0) = averageValue( averageValue(img(x,y-2,0), img(x,y+2,0), 0.5), img(x,y,0), factor);
             img(x,y,1) = averageValue( averageValue(img(x,y-2,1), img(x,y+2,1), 0.5), img(x,y,1), factor);
             img(x,y,2) = averageValue( averageValue(img(x,y-2,2), img(x,y+2,2), 0.5), img(x,y,2), factor);                                         
              /* new = ( [ ((a-b)/2 + b) - c ] * factor) + c */                                
             img(x,y-1,0) = averageValue(img(x,y-1,0), img(x,y,0), VERT1);
             img(x,y-1,1) = averageValue(img(x,y-1,1), img(x,y,1), VERT1);
             img(x,y-1,2) = averageValue(img(x,y-1,2), img(x,y,2), VERT1);
             img(x,y+1,0) = averageValue(img(x,y+1,0), img(x,y,0), VERT1);
             img(x,y+1,1) = averageValue(img(x,y+1,1), img(x,y,1), VERT1);
             img(x,y+1,2) = averageValue(img(x,y+1,2), img(x,y,2), VERT1);                   
           }            
           /* pixels in the middle of line */
           else if (x >= firstOccurrence && x <= lastOccurrence) //white line 
           {
             /* New values are computed and set 
              * new = (a-b)/2 + b --> average value of a and b */
             img(x,y,0) = averageValue(img(x,y-2,0), img(x,y+2,0), 0.5);
             img(x,y,1) = averageValue(img(x,y-2,1), img(x,y+2,1), 0.5);
             img(x,y,2) = averageValue(img(x,y-2,2), img(x,y+2,2), 0.5);
             /* Not only the actual line, also the line above/ below are
              * changed, but only by VERT1 % */
             img(x,y-1,0) = averageValue(img(x,y-1,0), img(x,y,0), VERT1);
             img(x,y-1,1) = averageValue(img(x,y-1,1), img(x,y,1), VERT1);
             img(x,y-1,2) = averageValue(img(x,y-1,2), img(x,y,2), VERT1);
             img(x,y+1,0) = averageValue(img(x,y+1,0), img(x,y,0), VERT1);
             img(x,y+1,1) = averageValue(img(x,y+1,1), img(x,y,1), VERT1);
             img(x,y+1,2) = averageValue(img(x,y+1,2), img(x,y,2), VERT1);             
           }
           /* pixels at the end of the line */
           else if (x > lastOccurrence && x<lastOccurrence+20) //end of white line
           {
             /* The farer away the pixel is from the firstOccurrence, the less
              * the value is changed */
             int factor =  ( 1 - (x-lastOccurrence) / 20); //percent of new value relative to original value 
             img(x,y,0) = averageValue( averageValue(img(x,y-2,0), img(x,y+2,0), 0.5), img(x,y,0), factor);
             img(x,y,1) = averageValue( averageValue(img(x,y-2,1), img(x,y+2,1), 0.5), img(x,y,1), factor);
             img(x,y,2) = averageValue( averageValue(img(x,y-2,2), img(x,y+2,2), 0.5), img(x,y,2), factor);                                         
              /* new = ( [ ((a+b)/2 + b) - c ] * factor) + d */    
             img(x,y-1,0) = averageValue(img(x,y-1,0), img(x,y,0), VERT1);
             img(x,y-1,1) = averageValue(img(x,y-1,1), img(x,y,1), VERT1);
             img(x,y-1,2) = averageValue(img(x,y-1,2), img(x,y,2), VERT1);
             img(x,y+1,0) = averageValue(img(x,y+1,0), img(x,y,0), VERT1);
             img(x,y+1,1) = averageValue(img(x,y+1,1), img(x,y,1), VERT1);
             img(x,y+1,2) = averageValue(img(x,y+1,2), img(x,y,2), VERT1);              
           }         
          } // x for loop 
        } //end if      
   } //y for loop           
   return (img);
}

/* Serach for the max value in yCounter and returns the position of this value
 * input: global static yCounter
 * output: position
 * */
int HistoricVideoOptimization::yMax()
{
  int pos = 0;  
  int value =0;
  for (int i=0; i<1000; i++)
  {
    if (yCounter[i] > value)
    {
      value = yCounter[i];
      pos = i;  
      //cout << yCounter[i];
    } 
  }  
  if (pos < 20)
  {
    pos = 20; // 
  }
  //if (pos >  //hier max size eingeben!
  return pos;
}

/* Finds the max corrFactor for actual cut. They are needed for the dynamcal
 * black  boarders at the anti shaking process.
 * input: Positions in the cut, cmdata
 * output: writes to the global variables maxCorrFactorOfActualCutDirectionX,
           maxCorrFactorOfActualCutDirectionY, 
           minCorrFactorOfActualCutDirectionX,
           minCorrFactorOfActualCutDirectionY
 */
void HistoricVideoOptimization::findMaxCorrFactors(int actualCut, int nextCut, int middleShakingPos,
                        CameraModelData &cmData)
{
  double maxCorrFactorX = 0;
  double maxCorrFactorY = 0;  
  double minCorrFactorX = 0;
  double minCorrFactorY = 0; 
  for (int32 i=actualCut+1;i<nextCut;i++) //alle frames until next hard cut 
  {   
    //corrFactors of current frame 
    double corrFactorX = 0; //reset
    double corrFactorY = 0;      
    if (i < middleShakingPos) //act frame is BEFORE middle position
    {
      for (int a=middleShakingPos-1; a > i-1; a--) //heading backwards
      { 
          if (cmData.data[a].transData.tx < THRESBIGMOVEMENT) 
          {      
            corrFactorX += cmData.data[a].transData.tx;            
          }
          if (cmData.data[a].transData.ty < THRESBIGMOVEMENT)
          {      
            corrFactorY += cmData.data[a].transData.ty; 
            //cerr << corrFactorY << ";";
          }
      } //end for
    } //end if
    else if (i > middleShakingPos) //act frame is AFTER middle position
    {
      for (int a=middleShakingPos; a < i; a++) //heading foreward
      {
          if (cmData.data[a].transData.tx < THRESBIGMOVEMENT) 
          {      
            corrFactorX += cmData.data[a].transData.tx;
          }
           if (cmData.data[a].transData.ty < THRESBIGMOVEMENT)
          {      
            corrFactorY += cmData.data[a].transData.ty; 
          }
        //}
      } //end for
    } //end if    
    if (corrFactorX > maxCorrFactorX)
    {  
      maxCorrFactorX = corrFactorX;
    }
    if (corrFactorX < minCorrFactorX)
    {
      minCorrFactorX = corrFactorX;    
    }
    //cerr << "\n corrFactorY=" << corrFactorY << " maxCorrFactorY=" << maxCorrFactorY << "\n";    
    if (corrFactorY > maxCorrFactorY)
    {        
      maxCorrFactorY = corrFactorY;
      //cerr << "\nNew max corr FactorY=" << maxCorrFactorY;      
    } 
    if (corrFactorY < minCorrFactorY)
    {
      minCorrFactorY = corrFactorY;    
    }           
  } //end for

  //give values to global variables for later use
  maxCorrFactorOfActualCutDirectionX = (int) floor (maxCorrFactorX);
  maxCorrFactorOfActualCutDirectionY = (int) floor (maxCorrFactorY);
  minCorrFactorOfActualCutDirectionX = (int) floor (minCorrFactorX);
  minCorrFactorOfActualCutDirectionY = (int) floor (minCorrFactorY);
  //cerr << "\nTEST=" << maxCorrFactorY;  
  //cerr << "\nBoarder left:" << maxCorrFactorOfActualCutDirectionX;
  //cerr << "\nBoarder top:" << maxCorrFactorOfActualCutDirectionY;
  //cerr << "\nBoarder right:" << abs(minCorrFactorOfActualCutDirectionX);
  //cerr << "\nBoarder bottom:" << abs(minCorrFactorOfActualCutDirectionY);
  //cerr << "\nBoarders only when necessary." <<"\n";
  return;         
}


/* Checks, wheather a given cut of the video is shakig in a specified 
 * direction (x or y)
 * input: int number of current and nextCut, search direction, CameraModelData
 * output: "-1000" if there is no shaking, otherwiese the average deviation of cut
 * */
int HistoricVideoOptimization::preprocessAntiShaking(int actualCut, int nextCut, bool directionX, 
                          CameraModelData &cmData)
{
  cerr << "\n\nstarting shaking detection, diretionX=" << directionX << "...";
  const double THRESNORMALMOVING = 1.1; //4 //2 //1.5 //1.8 //1.1
  const double THRESNOMOVEMENT = 0.65; //1
  double sum = 0;
  double noShakingSum= 0;
  int frameCounter = 0; //counts from 1 to 8  
             
  //try not to take the motion vector between 2 cuts (actualCut+1)
  for (int32 i=actualCut+1;i<nextCut;i++) //alle frames until next hard cut 
  {    
    frameCounter++;
    if (directionX) //x direction has been selected
    {
      //No very big values, they would falsify the results
      if (cmData.data[i].transData.tx < THRESBIGMOVEMENT) 
      {
        sum += cmData.data[i].transData.tx; //adds x deviation of current frame
        noShakingSum += abs (cmData.data[i].transData.tx);
        //important: here are the absolute values needed
      }
      else //value is ignored
      {
        noShakingSum += THRESNOMOVEMENT / 8; //compensation value
      }
      //cerr << "\nx-value for sum=" << cmData.data[i].transData.tx;
    }
    else           //y direction
    {                
      if (cmData.data[i].transData.ty < THRESBIGMOVEMENT)
      {
        sum += cmData.data[i].transData.ty;
        noShakingSum += abs (cmData.data[i].transData.ty);
      }
      else 
      {
        noShakingSum += THRESNOMOVEMENT / 8; //compensation THRESBIGMOVEMENT
      }            
    }
    //if in 8 connected frames there is few movement
    if ( (noShakingSum <= THRESNOMOVEMENT) && (frameCounter == 8) ) 
    {
      cerr << "break because no movement"; //; noShakingSum=" << noShakingSum;
      return (-1); //no shaking in cut, also no/ few movement   
    }
    if (frameCounter >= 8)
    {
      frameCounter = 0; //reset 
      noShakingSum = 0; //reset       
    }              
  } //end for 
  sum = abs (sum / (nextCut - actualCut-1) ) ; //average deviation of cut
  //middle pos of the cut
  int middleShakingPos = actualCut + (int) floor ((ieee64)((nextCut - actualCut) / 2));
   
  if (sum > THRESNORMALMOVING) //its a normal movement in cut
  {
    cerr << "break because normal movement"; //;  sum=" << sum <<" > THRESNORMALMOVING=" << THRESNORMALMOVING;
    return (-1); //no shaking in cut, but a normal movement, break
  }
  else
  {
    findMaxCorrFactors(actualCut, nextCut, middleShakingPos, cmData);
    cerr << "Shaking detected: \nsum=" << sum 
         << "; middleShakingPos=" << middleShakingPos  << " \n";
    return (middleShakingPos);  //yes!! Shaking in cut!
  }
}

/* Computes the correction faktor for anti shaking and
 * do the anti shaking operations
 * input: Number of current frame i, direction of correction, cmdata,
 *        the frame to correct, the refference postion for the correction,
 *        videsize, ev. refference frames 
 * output: The corrected frame
 * */
Image<uint8> HistoricVideoOptimization::processAntiShaking(int i, bool directionX, CameraModelData &cmData, 
                                Image<uint8> &img, int middleShakingPos, 
                                VideoSize &size)      //, Image<uint8> prevImg,
                                                      //Image<uint8> nextImg )
{ 
  const int THRESCORRECTIONIMPOSSIBLE = 90; //40 //35 //100
  //double corrFactor = 0;  
  double corrFactorX = 0;
  double corrFactorY = 0;
  /* Process the correction factor for current frame
   * adjustment takes place with reference to the middleShakingPos frame 
   * of the cut */
  if (i < middleShakingPos) //act frame is BEFORE middle position
  {
    for (int a=middleShakingPos-1; a > i-1; a--) //heading backwards
    { 
      //if (directionX) //x direction has been selected
      //{
        //No very big values, they would falsify the results
        if (cmData.data[a].transData.tx < THRESBIGMOVEMENT) 
        {      
          corrFactorX += cmData.data[a].transData.tx;
          //cerr << corrFactorX << ", " ;          
        }
      //}
      //else  
      //{
        if (cmData.data[a].transData.ty < THRESBIGMOVEMENT)
        {      
          corrFactorY += cmData.data[a].transData.ty; 
          //cerr << corrFactorY << ", " ;
        }
      //}
    } //end for
  } //end if
  else if (i > middleShakingPos) //act frame is AFTER middle position
  {
    for (int a=middleShakingPos; a < i; a++) //heading foreward
    {
      //if (directionX) //x direction has been selected
      //{
        //No very big values, they would falsify the results
        if (cmData.data[a].transData.tx < THRESBIGMOVEMENT) 
        {      
          corrFactorX += cmData.data[a].transData.tx;
          //cerr << corrFactor << ", " ;
        }
      //}
      //else  
      //{
        if (cmData.data[a].transData.ty < THRESBIGMOVEMENT)
        {      
          corrFactorY += cmData.data[a].transData.ty; 
          //cerr << corrFactorY << ", " ;
        }
      //}
    } //end for
  } //end if  

  //preperation of correction Factors and there correct sign
  int corrFactorIntX = 0;
  int corrFactorIntY = 0; 
  if (i < middleShakingPos && directionX)
  {
    corrFactorIntX = (int) - (corrFactorX); //round the factor down and negate it
    corrFactorIntY = (int) - (corrFactorY);  
  }
  else if (i > middleShakingPos && directionX)
  {
    corrFactorIntX = (int)  (corrFactorX); //round the factor down and negate it
    corrFactorIntY = (int)  (corrFactorY);     
  }  
  else if (i < middleShakingPos && !directionX)
  {
    corrFactorIntX = (int)  (corrFactorX); //round the factor down and negate it
    corrFactorIntY = (int)  (corrFactorY);  
  }
  else if (i > middleShakingPos && !directionX)
  {
    corrFactorIntX = (int) - (corrFactorX); //round the factor down and negate it
    corrFactorIntY = (int) - (corrFactorY);     
  }
  
  //set the factors to max possible if necessary
  if (corrFactorIntX > THRESCORRECTIONIMPOSSIBLE) //positive max
  {
    corrFactorIntX = THRESCORRECTIONIMPOSSIBLE; //set the factor to max
  }
  else if (corrFactorIntX < -THRESCORRECTIONIMPOSSIBLE) //negative max
  {
    corrFactorIntX = -THRESCORRECTIONIMPOSSIBLE;
  }
  if (corrFactorIntY > THRESCORRECTIONIMPOSSIBLE) //positive max
  {
    corrFactorIntY = THRESCORRECTIONIMPOSSIBLE; //set the factor to max
  }
  else if (corrFactorIntY < -THRESCORRECTIONIMPOSSIBLE) //negative max
  {
    corrFactorIntY = -THRESCORRECTIONIMPOSSIBLE;
  }
  
  if (maxCorrFactorOfActualCutDirectionX > THRESCORRECTIONIMPOSSIBLE)
  {
    maxCorrFactorOfActualCutDirectionX = THRESCORRECTIONIMPOSSIBLE;
  }
  if (maxCorrFactorOfActualCutDirectionY > THRESCORRECTIONIMPOSSIBLE)
  {
    maxCorrFactorOfActualCutDirectionY = THRESCORRECTIONIMPOSSIBLE;
  } 
  if (minCorrFactorOfActualCutDirectionX < -THRESCORRECTIONIMPOSSIBLE)
  {
    minCorrFactorOfActualCutDirectionX = THRESCORRECTIONIMPOSSIBLE;
  }
  if (minCorrFactorOfActualCutDirectionY < -THRESCORRECTIONIMPOSSIBLE)
  {
    minCorrFactorOfActualCutDirectionY = THRESCORRECTIONIMPOSSIBLE;
  }  
  
  /* Do anti shaking corrections */   
          
  if (directionX)   //direction x 
  {
    //cerr << "\nCorrFactorIntX(frame " << i <<")=" << corrFactorIntX<< "\n";
    if (corrFactorIntX < 0)  //pan shot to the left 
     //The motion vector between frame i and i+1 is negativ, the unwanted motion 
     //is from right to left. The corr Factor is positive, the correction has to
     //be from left to right.  
    {
      corrFactorIntX = abs (corrFactorIntX); //make positive
      for (int y=0; y < size[DIM_Y]; y++)
      //for (int y=THRESCORRECTIONIMPOSSIBLE; y < size[DIM_Y]-THRESCORRECTIONIMPOSSIBLE; y++)
      {
        //for (int x=corrFactorIntX; x <= (size[DIM_X]-corrFactorIntX); x++)
        for (int x=0; x <= (size[DIM_X]-corrFactorIntX); x++) // new
        //for (int x=THRESCORRECTIONIMPOSSIBLE; x <= (size[DIM_X]-THRESCORRECTIONIMPOSSIBLE); x++)
        {
          img(x,y,0) = img(x+corrFactorIntX,y,0);  
          img(x,y,1) = img(x+corrFactorIntX,y,1); 
          img(x,y,2) = img(x+corrFactorIntX,y,2);           
        }  
      } //end y             
    }
    
    else //corrFactorIntX > 0 
         //pan shot to the right, corrections to the right, "shaking to the left" 
    {
      for (int y=0; y < size[DIM_Y]; y++)
      //for (int y=THRESCORRECTIONIMPOSSIBLE; y < size[DIM_Y]-THRESCORRECTIONIMPOSSIBLE; y++)      
      {
        //go backwars from right to left through the rows        
        for (int x=size[DIM_X]-corrFactorIntX-1; x >= corrFactorIntX ; x--) 
        //for (int x=size[DIM_X]-THRESCORRECTIONIMPOSSIBLE-1; x >= THRESCORRECTIONIMPOSSIBLE ; x--) 
        {
          img(x,y,0) = img(x-corrFactorIntX,y,0); 
          img(x,y,1) = img(x-corrFactorIntX,y,1);
          img(x,y,2) = img(x-corrFactorIntX,y,2);        
        }
      } //end y              
    } //end corrections right  
     
  } //end corrections direction x
  
  else             //direction y
  {        
    // Camera Movement (pan shot) is top-down. Objects seems to move bottom up.
    // This is corrected .
    if (corrFactorIntY < 0)    
    {
      corrFactorIntY = abs (corrFactorIntY); //make positive      
      //cerr << "\nCorrFactorIntY=" << corrFactorIntY;
      for (int y=size[DIM_Y]-1; y >= corrFactorIntY; y--)
      //for (int y=size[DIM_Y]-THRESCORRECTIONIMPOSSIBLE-1; y >= THRESCORRECTIONIMPOSSIBLE; y--)
      {
        for (int x=0; x < size[DIM_X]; x++)
        //for (int x=THRESCORRECTIONIMPOSSIBLE; x < size[DIM_X]-THRESCORRECTIONIMPOSSIBLE; x++)
        {
          img(x,y,0) = img(x,y-corrFactorIntY,0); //TEST TEMP
          img(x,y,1) = img(x,y-corrFactorIntY,1);
          img(x,y,2) = img(x,y-corrFactorIntY,2);  
        }
      }        
    }
    else     //pan shot bottom up             
    {
      for (int y=0; y<=size[DIM_Y]-corrFactorIntY; y++)
      //for (int y=THRESCORRECTIONIMPOSSIBLE; y<=size[DIM_Y]-THRESCORRECTIONIMPOSSIBLE; y++)
      {
        for (int x=0; x < size[DIM_X]; x++)
        //for (int x=THRESCORRECTIONIMPOSSIBLE; x < size[DIM_X]-THRESCORRECTIONIMPOSSIBLE; x++)        
        {
          img(x,y,0) = img(x,y+corrFactorIntY,0); 
          img(x,y,1) = img(x,y+corrFactorIntY,1);
          img(x,y,2) = img(x,y+corrFactorIntY,2); 
        }  
      }    
    } 
     
  } //end corrections direction y  
    

  /* Make black boarder
   * The thickness of boarder is relative to the precomputed max corrFactor
   * at the moment the left/right and top/down boarder have the same values
   *  */
  if (directionX)   //direction x
  {  
    for (int y=0; y < size[DIM_Y]; y++)
    { 
      //for (int x=size[DIM_X]-THRESCORRECTIONIMPOSSIBLE; x < size[DIM_X]; x++) 
      for (int x=size[DIM_X]-maxCorrFactorOfActualCutDirectionX; x < size[DIM_X]; x++) //boarder left      
      //for (int x=size[DIM_X]-abs(minCorrFactorOfActualCutDirectionX); x < size[DIM_X]; x++)
      {
        img(x,y,0) = 0; //black
        img(x,y,1) = 0;
        img(x,y,2) = 0; 
      } 
      //for (int x=0; x < THRESCORRECTIONIMPOSSIBLE; x++) 
      for (int x=0; x < abs(minCorrFactorOfActualCutDirectionX); x++) //boarder left
      {
        img(x,y,0) = 0; //black
        img(x,y,1) = 0;
        img(x,y,2) = 0;           
      }  
    } 
  }   
  else  //direction y
  {
    //for (int y=0; y<THRESCORRECTIONIMPOSSIBLE;y++) //boarder top
    for (int y=0; y<maxCorrFactorOfActualCutDirectionY;y++) //boarder top
    //for (int y=0; y<abs(minCorrFactorOfActualCutDirectionY);y++) //boarder top
    {
       for (int x=0; x < size[DIM_X]; x++)
       {
         img(x,y,0) = 0; //black
         img(x,y,1) = 0;
         img(x,y,2) = 0;  
       }        
    }
    //for (int y=size[DIM_Y]-THRESCORRECTIONIMPOSSIBLE; y < size[DIM_Y]; y++) //boarder down
    for (int y=size[DIM_Y]-abs(minCorrFactorOfActualCutDirectionY); y < size[DIM_Y]; y++) //boarder down
    {
      for (int x=0; x < size[DIM_X]; x++)
      {
        img(x,y,0) = 0; //black
        img(x,y,1) = 0;
        img(x,y,2) = 0; 
      }  
    } 
  }


    //cerr << i << "-frame is corrected in directionX=" << directionX << 
      //      " by " << floor(corrFactor) << " \n";     
  return (img);  
}
