/*
    LogoDetection.cpp     detect logos in videos
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// MoCA includes
#include <LogoDetection.h>


std::ostream& operator<< (ostream &os, const LogoDetectionData &data) 
{
	os << "version  : " << data.version << endl;
	os << "filename : " << data.filename << endl;
	os << "minX[0]  : " << data.box[0].minX << endl;
	os << "maxX[0]  : " << data.box[0].maxX << endl;
	os << "minY[0]  : " << data.box[0].minY << endl;
	os << "minY[0]  : " << data.box[0].maxY << endl;
	os << endl;

	return os;
} // operator <<

OTagStream& operator<< (OTagStream& os, const LogoDetectionData &data) 
{
    TagFormat::writeStartTag (os, "LOGO");
	os << endl;
	
    TagFormat::writeNodeValue(os, "version", data.version, 1);
    TagFormat::writeNodeValue(os, "filename", data.filename, 1);
    TagFormat::writeNodeValue(os, "maxEdgeValue", data.maxEdgeValue, 1);
	os << endl;

    TagFormat::writeNodeValue(os, "minX[0]", data.box[0].minX, 1);
    TagFormat::writeNodeValue(os, "maxX[0]", data.box[0].maxX, 1);
    TagFormat::writeNodeValue(os, "minY[0]", data.box[0].minY, 1);
    TagFormat::writeNodeValue(os, "maxY[0]", data.box[0].maxY, 1);
    TagFormat::writeNodeValue(os, "rel[0]",  data.box[0].rel, 1);
	os << endl;

    TagFormat::writeNodeValue(os, "minX[1]", data.box[1].minX, 1);
    TagFormat::writeNodeValue(os, "maxX[1]", data.box[1].maxX, 1);
    TagFormat::writeNodeValue(os, "minY[1]", data.box[1].minY, 1);
    TagFormat::writeNodeValue(os, "maxY[1]", data.box[1].maxY, 1);
    TagFormat::writeNodeValue(os, "rel[1]",  data.box[1].rel, 1);
	os << endl;

    TagFormat::writeNodeValue(os, "minX[2]", data.box[2].minX, 1);
    TagFormat::writeNodeValue(os, "maxX[2]", data.box[2].maxX, 1);
    TagFormat::writeNodeValue(os, "minY[2]", data.box[2].minY, 1);
    TagFormat::writeNodeValue(os, "maxY[2]", data.box[2].maxY, 1);
    TagFormat::writeNodeValue(os, "rel[2]",  data.box[2].rel, 1);
	os << endl;

    TagFormat::writeNodeValue(os, "minX[3]", data.box[3].minX, 1);
    TagFormat::writeNodeValue(os, "maxX[3]", data.box[3].maxX, 1);
    TagFormat::writeNodeValue(os, "minY[3]", data.box[3].minY, 1);
    TagFormat::writeNodeValue(os, "maxY[3]", data.box[3].maxY, 1);
    TagFormat::writeNodeValue(os, "rel[3]",  data.box[3].rel, 1);
	os << endl;
	
    TagFormat::writeEndTag (os, "LOGO");
	os << endl;
	
    return os;
} // operator<<


ITagStream& operator>> (ITagStream& is, LogoDetectionData& data)
{
	
    char c;
    bool failFlag = true;
	
	if (is >> c) {
		failFlag = false;
		is.putback(c);

		TagFormat::readStartTag (is, "LOGO");
		
		TagFormat::readNodeValue(is, "version", data.version);
		TagFormat::readNodeValue(is, "filename", data.filename);
		TagFormat::readNodeValue(is, "maxEdgeValue", data.maxEdgeValue);

		TagFormat::readNodeValue(is, "minX[0]", data.box[0].minX);
		TagFormat::readNodeValue(is, "maxX[0]", data.box[0].maxX);
		TagFormat::readNodeValue(is, "minY[0]", data.box[0].minY);
		TagFormat::readNodeValue(is, "maxY[0]", data.box[0].maxY);
		TagFormat::readNodeValue(is, "rel[0]",  data.box[0].rel);

		TagFormat::readNodeValue(is, "minX[1]", data.box[1].minX);
		TagFormat::readNodeValue(is, "maxX[1]", data.box[1].maxX);
		TagFormat::readNodeValue(is, "minY[1]", data.box[1].minY);
		TagFormat::readNodeValue(is, "maxY[1]", data.box[1].maxY);
		TagFormat::readNodeValue(is, "rel[1]",  data.box[1].rel);

		TagFormat::readNodeValue(is, "minX[2]", data.box[2].minX);
		TagFormat::readNodeValue(is, "maxX[2]", data.box[2].maxX);
		TagFormat::readNodeValue(is, "minY[2]", data.box[2].minY);
		TagFormat::readNodeValue(is, "maxY[2]", data.box[2].maxY);
		TagFormat::readNodeValue(is, "rel[2]",  data.box[2].rel);

		TagFormat::readNodeValue(is, "minX[3]", data.box[3].minX);
		TagFormat::readNodeValue(is, "maxX[3]", data.box[3].maxX);
		TagFormat::readNodeValue(is, "minY[3]", data.box[3].minY);
		TagFormat::readNodeValue(is, "maxY[3]", data.box[3].maxY);
		TagFormat::readNodeValue(is, "rel[3]",  data.box[3].rel);

		TagFormat::readEndTag (is, "LOGO");
	} // if

	if (is.rdstate() & ios::failbit) {
		throw MocaException ("LogoDetection: Illegal index file format!");
	} else {
        is.clear();
    } // else
	
    return is;
} // operator>> 






void 
LogoDetection::process(LogoDetectionData& data) 
{
	// read data
	ReadMocaData<LogoDetectionData> reader (pathFilename, LOGO_DETECTION_FILENAME);
	bool dataValid = reader.process (LOGO_DETECTION_VERSION, data); 
	
	if (!dataValid) {
		
		// create new index
		cerr << "\rLogoDetection: Creating index file ...          " << endl;
		readValues(data);
		
		// save index file
		WriteMocaData<LogoDetectionData> writer (pathFilename, LOGO_DETECTION_FILENAME);
		writer.process(data);
		cerr << "\rLogoDetection: Creating index file ... done          " << endl;
	} // if

} // process


void 
LogoDetection::summarizeLines(Image<uint8> &tmp, 
							  Signal<int32> &sigX, Signal<int32> &sigSumX, Signal<int32> &sigY, Signal<int32> &sigSumY,
							  int32 &lineMaxX, int32 &lineMaxY)
{
	int32 sizeX = tmp.getSize()[DIM_X];
	int32 sizeY = tmp.getSize()[DIM_Y];

	sigX.resize (SignalSize(sizeX));
	sigY.resize (SignalSize(sizeY));
	sigSumX.resize (SignalSize(sizeX));
	sigSumY.resize (SignalSize(sizeY));

	sigX.clear(0);
	sigY.clear(0);
	sigSumX.clear(0);
	sigSumY.clear(0);

	// calc sigX
	for (int32 x=0; x<sizeX; x++) {
		for (int32 y=0; y<sizeY; y++) {
			sigX(x) += tmp (x,y);
		} // for
		sigX(x) /=sizeY;
	} // for

	// calc sigSumX (aggregate values for 7 lines and find max)
	lineMaxX = 0;
	for (int32 x=2; x<sizeX-2; x++) {
		sigSumX(x) = sigX(x-3)+sigX(x-2)+sigX(x-1)+sigX(x+0)+sigX(x+1)+sigX(x+2)+sigX(x+3);
		if (sigSumX(lineMaxX) < sigSumX(x)) lineMaxX = x;
	} // for

	// calc sigY
	for (int32 y=0; y<sizeY; y++) {
		for (int32 x=0; x<sizeX; x++) {
			sigY(y) += tmp (x,y);
		} // for
		sigY(y) /=sizeX;
	} // for

	// calc sigSumY (aggregate values for 7 lines and find max)
	lineMaxY = 0;
	for (int32 y=2; y<sizeY-2; y++) {
		sigSumY(y) = sigY(y-3)+sigY(y-2)+sigY(y-1)+sigY(y+0)+sigY(y+1)+sigY(y+2)+sigY(y+3);
		if (sigSumY(lineMaxY) < sigSumY(y)) lineMaxY = y;
	} // for


	return;
}


ieee64 
LogoDetection::getRatio (Image<uint8> &img, int32 minX, int32 minY, int32 maxX, int32 maxY)
{
	int32 v=0;
	int32 n=0;
	for (int32 y=minY; y<=maxY; y++) {
		for (int32 x=minX; x<=maxX; x++) {
			v +=img(x,y);
			n++;
		} // for
	} // for
	if (n==0) n=1;
	return ((ieee64)v)/n;
} // getRatio


void 
LogoDetection::getRowY (Image<uint8> &img, int32 lineMaxY, Signal<int32> &sigY, Signal<int32> &sigSumY, int32 avg, 
						int32 &minX, int32 &minY, int32 &maxX, int32 &maxY)
{
	// set minY, maxY
	minY = lineMaxY;
	maxY = lineMaxY;
	int32 th = (int32)(0.5 * sigY(lineMaxY));
	while ((minY>1) && ((sigY(minY-1)>th) || (sigY(minY-2)>th))) {
		minY--;
	} // while
	while ((maxY<img.getSize()[DIM_Y]-2) && ((sigY(maxY+1)>th) || (sigY(maxY+2)>th))) {
		maxY++;
	} // while

	// set minXm maxX
	minX = 0;
	maxX = img.getSize()[DIM_X]-1;

	ieee64 v0 = getRatio (img, minX, minY, maxX, maxY);
	ieee64 v1 = getRatio (img, minX+1, minY, maxX, maxY);
	while ( (minX+5<maxX) && (v1>v0) ) {
		minX++;
		v0=v1;
		v1=getRatio (img, minX+1, minY, maxX, maxY);
	} // while

	v0 = getRatio (img, minX, minY, maxX, maxY);
	v1 = getRatio (img, minX, minY, maxX-1, maxY);
	while ( (maxX-5>minX) && (v1>v0) ) {
		maxX--;
		v0=v1;
		v1 = getRatio (img, minX, minY, maxX-1, maxY);
	} // while

	// check, if there is a border
	int32 tmpMinX = minX;
	int32 tmpMaxX = maxX;

	if (minX+15<maxX) {
		minX+=10;
		v0 = getRatio (img, minX, minY, maxX, maxY);
		v1 = getRatio (img, minX+1, minY, maxX, maxY);
		while ( (minX+5<maxX) && (v1>v0) ) {
			minX++;
			v0=v1;
			v1=getRatio (img, minX+1, minY, maxX, maxY);
		} // while

		v0 = getRatio (img, minX, minY, maxX, maxY);
		v1 = getRatio (img, tmpMinX, minY, maxX, maxY);
		// new border should be at least 20% better
		if (v1>v0*1.2) {
			minX = tmpMinX;
		} // if
	} // if

	// check, if there is a border
	if (maxX-15>minX) {
		maxX-=10;

		v0 = getRatio (img, minX, minY, maxX, maxY);
		v1 = getRatio (img, minX, minY, maxX-1, maxY);
		while ( (maxX-5>minX) && (v1>v0) ) {
			maxX--;
			v0=v1;
			v1 = getRatio (img, minX, minY, maxX-1, maxY);
		} // while


		v0 = getRatio (img, minX, minY, maxX, maxY);
		v1 = getRatio (img, minX, minY, tmpMaxX, maxY);
		// new border should be at least 20% better
		if (v1>v0*1.2) {
			maxX = tmpMaxX;
		} // if
	} // if

	// find exact border (minX)
	v0 = 0.5 * getRatio (img, minX, minY, maxX, maxY);
	while (minX>5) {
		v1 = getRatio (img, minX-5, minY, minX-1, maxY);
		if (v1 > v0) {
			minX--;
		} else {
			break;
		} // if
	} // while
	while (minX>2) {
		v1 = getRatio (img, minX-2, minY, minX-1, maxY);
		if (v1 > v0) {
			minX--;
		} else {
			break;
		} // if
	} // while
	v1 = getRatio (img, minX-1, minY, minX-1, maxY);
	if (v1 > v0) minX--;


	// find exact border (maxX)
	v0 = 0.5 * getRatio (img, minX, minY, maxX, maxY);
	while (maxX<img.getSize()[DIM_X]-5-1) {
		v1 = getRatio (img, maxX+1, minY, maxX+5, maxY);
		if (v1 > v0) {
			maxX++;
		} else {
			break;
		} // if
	} // while
	while (maxX<img.getSize()[DIM_X]-2-1) {
		v1 = getRatio (img, maxX+1, minY, maxX+2, maxY);
		if (v1 > v0) {
			maxX++;
		} else {
			break;
		} // if
	} // while
	v1 = getRatio (img, maxX+1, minY, maxX+1, maxY);
	if (v1 > v0) maxX++;


	// find exact border (minY)
	v0 = 0.5 * getRatio (img, minX, minY, maxX, maxY);
	while (minY>5) {
		v1 = getRatio (img, minX, minY-5, maxX, minY-1);
		if (v1 > v0) {
			minY--;
		} else {
			break;
		} // if
	} // while
	while (minX>2) {
		v1 = getRatio (img, minX, minY-2, maxX, minY-1);
		if (v1 > v0) {
			minY--;
		} else {
			break;
		} // if
	} // while
	v1 = getRatio (img, minX, minY-1, maxX, minY-1);
	if (v1 > v0) minY--;


	// find exact border (maxY)
	v0 = 0.5 * getRatio (img, minX, minY, maxX, maxY);
	while (maxY<img.getSize()[DIM_Y]-5-1) {
		v1 = getRatio (img, minX, maxY+1, maxX, maxY+5);
		if (v1 > v0) {
			maxY++;
		} else {
			break;
		} // if
	} // while
	while (maxX<img.getSize()[DIM_Y]-2-1) {
		v1 = getRatio (img, minX, maxY+1, maxX, maxY+2);
		if (v1 > v0) {
			maxY++;
		} else {
			break;
		} // if
	} // while
	v1 = getRatio (img, minX, maxY+1, maxX, maxY+1);
	if (v1 > v0) maxY++;

	return;
}
		


void 
LogoDetection::getColumnX (Image<uint8> &img, int32 lineMaxX, Signal<int32> &sigX, Signal<int32> &sigSumX, int32 avg, 
						   int32 &minX, int32 &minY, int32 &maxX, int32 &maxY)
{
	// set minY, maxY
	minX = lineMaxX;
	maxX = lineMaxX;
	int32 th = (int32)(0.5 * sigX(lineMaxX));
	while ((minX>1) && ((sigX(minX-1)>th) || (sigX(minX-2)>th))) {
		minX--;
	} // while
	while ((maxX<img.getSize()[DIM_X]-2) && ((sigX(maxX+1)>th) || (sigX(maxX+2)>th))) {
		maxX++;
	} // while

	// set minY, maxY
	minY = 0;
	maxY = img.getSize()[DIM_Y]-1;

	ieee64 v0 = getRatio (img, minX, minY, maxX, maxY);
	ieee64 v1 = getRatio (img, minX, minY+1, maxX, maxY);
	while ( (minY+5<maxY) && (v1>v0) ) {
		minY++;
		v0=v1;
		v1=getRatio (img, minX, minY+1, maxX, maxY);
	} // while

	v0 = getRatio (img, minX, minY, maxX, maxY);
	v1 = getRatio (img, minX, minY, maxX, maxY-1);
	while ( (maxY-5>minY) && (v1>v0) ) {
		maxY--;
		v0=v1;
		v1 = getRatio (img, minX, minY, maxX, maxY-1);
	} // while

	// check, if there is a border
	int32 tmpMinY = minY;
	int32 tmpMaxY = maxY;

	if (minY+15<maxY) {
		minY+=10;
		v0 = getRatio (img, minX, minY, maxX, maxY);
		v1 = getRatio (img, minX, minY+1, maxX, maxY);
		while ( (minX+5<maxX) && (v1>v0) ) {
			minY++;
			v0=v1;
			v1=getRatio (img, minX, minY+1, maxX, maxY);
		} // while

		v0 = getRatio (img, minX, minY, maxX, maxY);
		v1 = getRatio (img, minX, tmpMinY, maxX, maxY);
		// new border should be at least 20% better
		if (v1>v0*1.2) {
			minY = tmpMinY;
		} // if
	} // if

	// check, if there is a border
	if (maxY-15>minY) {
		maxY-=10;

		v0 = getRatio (img, minX, minY, maxX, maxY);
		v1 = getRatio (img, minX, minY, maxX, maxY-1);
		while ( (maxX-5>minX) && (v1>v0) ) {
			maxY--;
			v0=v1;
			v1 = getRatio (img, minX, minY, maxX, maxY-1);
		} // while


		v0 = getRatio (img, minX, minY, maxX, maxY);
		v1 = getRatio (img, minX, minY, maxX, tmpMaxY);
		// new border should be at least 20% better
		if (v1>v0*1.2) {
			maxY = tmpMaxY;
		} // if
	} // if



	// find exact border (minX)
	v0 = 0.5 * getRatio (img, minX, minY, maxX, maxY);
	while (minX>5) {
		v1 = getRatio (img, minX-5, minY, minX-1, maxY);
		if (v1 > v0) {
			minX--;
		} else {
			break;
		} // if
	} // while
	while (minX>2) {
		v1 = getRatio (img, minX-2, minY, minX-1, maxY);
		if (v1 > v0) {
			minX--;
		} else {
			break;
		} // if
	} // while
	v1 = getRatio (img, minX-1, minY, minX-1, maxY);
	if (v1 > v0) minX--;


	// find exact border (maxX)
	v0 = 0.5 * getRatio (img, minX, minY, maxX, maxY);
	while (maxX<img.getSize()[DIM_X]-5-1) {
		v1 = getRatio (img, maxX+1, minY, maxX+5, maxY);
		if (v1 > v0) {
			maxX++;
		} else {
			break;
		} // if
	} // while
	while (maxX<img.getSize()[DIM_X]-2-1) {
		v1 = getRatio (img, maxX+1, minY, maxX+2, maxY);
		if (v1 > v0) {
			maxX++;
		} else {
			break;
		} // if
	} // while
	v1 = getRatio (img, maxX+1, minY, maxX+1, maxY);
	if (v1 > v0) maxX++;


	// find exact border (minY)
	v0 = 0.5 * getRatio (img, minX, minY, maxX, maxY);
	while (minY>5) {
		v1 = getRatio (img, minX, minY-5, maxX, minY-1);
		if (v1 > v0) {
			minY--;
		} else {
			break;
		} // if
	} // while
	while (minY>2) {
		v1 = getRatio (img, minX, minY-2, maxX, minY-1);
		if (v1 > v0) {
			minY--;
		} else {
			break;
		} // if
	} // while
	v1 = getRatio (img, minX, minY-1, maxX, minY-1);
	if (v1 > v0) minY--;


	// find exact border (maxY)
	v0 = 0.5 * getRatio (img, minX, minY, maxX, maxY);
	while (maxY<img.getSize()[DIM_Y]-5-1) {
		v1 = getRatio (img, minX, maxY+1, maxX, maxY+5);
		if (v1 > v0) {
			maxY++;
		} else {
			break;
		} // if
	} // while
	while (maxX<img.getSize()[DIM_Y]-2-1) {
		v1 = getRatio (img, minX, maxY+1, maxX, maxY+2);
		if (v1 > v0) {
			maxY++;
		} else {
			break;
		} // if
	} // while
	v1 = getRatio (img, minX, maxY+1, maxX, maxY+1);
	if (v1 > v0) maxY++;

	return;
}
		

void 
LogoDetection::storeFrame(Image<uint8> &img, LogoDetectionData& data)
{
	ImageSize size = img.getSize();
	size[DIM_BAND]=3;
	Image<uint8> tmp (size);
	tmp.clear(0);

	DrawRectangle draw (Color::RED);
	for (int i=0; i<4; i++) {
		int32 minX = data.box[i].minX;
		int32 maxX = data.box[i].maxX;
		int32 minY = data.box[i].minY;
		int32 maxY = data.box[i].maxY;

		if (maxX != -1) {
			for (int32 y=minY; y<=maxY; y++) {
				for (int32 x=minX; x<=maxX; x++) {
					if (img(x,y) != 0) {
						tmp(x,y,0) = 255;
						tmp(x,y,1) = 255;
						tmp(x,y,2) = 255;
					} // if
				} // for
			} // for
			ImagePosition p0 (minX-1, minY-1);
			ImagePosition p1 (maxX+1, maxY+1);
			draw.process(p0,p1, tmp);
		} // if
	} // for

	
	CreateMocaPath mocaPath (pathFilename);
	string path = mocaPath.destinationPath;
	string fn = path + "logo.jpg";
	WriteImage (tmp, fn);

	return;
}


void 
LogoDetection::readValues (LogoDetectionData& data) 
{

	// set variables
	data.version = LOGO_DETECTION_VERSION;

	// get path of filename, do not create new directory
	CreateMocaPath mocaPath (pathFilename, false);
	data.filename = mocaPath.file;

	for (int i=0; i<4; i++) {
		data.box[i].minX = -1;
		data.box[i].maxX = -1;
		data.box[i].minY = -1;
		data.box[i].maxY = -1;
		data.box[i].rel  = 0.0;
	} // for

	// open reader
	VideoCodec codec = CODEC_Y;
	VideoDecoder reader (pathFilename, 3, -1, codec);
	VideoSize vidSize = reader.getSize();
	ImageSize imgSize = reader.getImageSize();
	
	// get average edges
	Image<int32> sumImg (imgSize);
	Image<uint8> img;
	sumImg.clear(0);
	int32 n=0;

	// do not analyze every frame
	int32 stepSize = 3;
	if (vidSize[DIM_Z]<100) stepSize = 2;
	if (vidSize[DIM_Z]<50) stepSize = 1;
	if (vidSize[DIM_Z]>1000) stepSize++;
	if (vidSize[DIM_Z]>3000) stepSize++;
	if (vidSize[DIM_Z]>5000) stepSize++;
	if (vidSize[DIM_Z]>10000) stepSize++;

	// analyse
	for (int32 i=0; i<vidSize[DIM_Z]; i+=stepSize) {

		img = reader.getImage(i);
		n++;
		for (int32 y=2; y<vidSize[DIM_Y]-2; y++) {
			for (int32 x=2; x<vidSize[DIM_X]-2; x++) {
				// summarize absolute differences
				sumImg(x,y) += labs(img(x,y,0)-img(x+1,y,0)) + labs(img(x,y,0)-img(x,y+1,0));
			} // for
		} // for
		cerr << "\rframe: " << i << "  " << ends;
	} // for
	cerr << endl;

	
	// get maximum difference, calc normalize factor f
	int32 maxV = 0;
	int32 avgV = 0;
	for (int32 y=0; y<vidSize[DIM_Y]; y++) {
		for (int32 x=0; x<vidSize[DIM_X]; x++) {
			if (maxV < sumImg(x,y)) maxV = sumImg(x,y);
			avgV += sumImg(x,y);
		} // for
	} // for
	avgV/=(vidSize[DIM_X]*vidSize[DIM_Y]);



	// normalize values to 0..255
	if (maxV==0) maxV=1;
	ieee64 f1 = 255.0 / maxV;

	img.clear(0);
	for (int32 y=0; y<vidSize[DIM_Y]; y++) {
		for (int32 x=0; x<vidSize[DIM_X]; x++) {

			// set img
			int32 v = (int32)(sumImg(x,y)*f1);
			if (v<0) v=0;
			if (v>255) v=255;
			img (x,y) = (uint8)(v);
		} // for
	} // for

#if 0
	// debug
	cerr << "debug" << endl;
	WriteImage (img,"D:/logo_test.png");
#endif


	// set maxAbsValue
	if (n==0) n=1;
	ieee64 f2 = 1.0/(2.0*n); 
	int32 maxAbsValue = (int32)(maxV / (2.0*n));
#if 0
	// debug
	cerr << "debug" << endl;
	cerr << "maximum edge: " << maxAbsValue << endl;
#endif

	data.maxEdgeValue = maxAbsValue;
	int32 thMaxEdgeValue = 65;
	if (n<200) thMaxEdgeValue+=5;
	if (n<100) thMaxEdgeValue+=5;
	if (n<60)  thMaxEdgeValue+=5;
	if (n<30)  thMaxEdgeValue+=10;
	if (data.maxEdgeValue < thMaxEdgeValue) {
		cerr << "LogoDetection: no logo detected!" << endl;
		return;
	} // if



#if 0
	// debugging
	cerr << "debugging..." << endl;
	Image<uint8> tmp=img;
	for (int32 j=0; j<256; j++) {
		tmp.clear();
		for (int32 y=0; y<vidSize[DIM_Y]; y++) {
			for (int32 x=0; x<vidSize[DIM_X]; x++) {
				if (img(x,y) > j) tmp(x,y) = 255;
			} // for
		} // for
		WriteImage(tmp, FileHandling::getNumberedFilename ("V:/logo_", j, ".png"));
	} // for
	return;
	exit(1);
#endif




	Image<uint8> tmpImg = img;

	// summarize lines
	Signal<int32> sigX, sigSumX, sigY, sigSumY;
	int32 lineMaxX, lineMaxY;

	int32 maxLineSum = 0;
	for (int32 boxnumber=0; boxnumber<4; boxnumber++) {
		summarizeLines(tmpImg, sigX, sigSumX, sigY, sigSumY, lineMaxX, lineMaxY);

		int32 minX=-1;
		int32 maxX=-1;
		int32 minY=-1;
		int32 maxY=-1;
		if (maxLineSum<sigSumX(lineMaxX)) maxLineSum = sigSumX(lineMaxX);
		if (maxLineSum<sigSumY(lineMaxY)) maxLineSum = sigSumY(lineMaxY);

		if (sigSumX(lineMaxX)>sigSumY(lineMaxY)) {

			// get box at column lineMaxX
			if (sigSumX(lineMaxX) > maxLineSum/5) {
				getColumnX (tmpImg, lineMaxX, sigX, sigSumX, avgV, minX, minY, maxX, maxY);
			} // if
		} else {

			// get box at row lineMaxY
			if (sigSumY(lineMaxY) > maxLineSum/5) {
				getRowY (tmpImg, lineMaxY, sigY, sigSumY, avgV, minX, minY, maxX, maxY);	
			} // if
		} // else

		if (maxX != -1) {
			data.box[boxnumber].minX=minX;
			data.box[boxnumber].maxX=maxX;
			data.box[boxnumber].minY=minY;
			data.box[boxnumber].maxY=maxY;
			data.box[boxnumber].rel =getRatio (tmpImg, minX, minY, maxX, maxY);
			if (data.box[boxnumber].rel > 100.0) data.box[boxnumber].rel = 100.0;

			// clear box
			for (int32 y=minY; y<=maxY; y++) {
				for (int32 x=minX; x<=maxX; x++) {
					tmpImg (x,y) = 0;
				} // for
			} // for
		} // if

	} // for


	// validate boxes
	const ieee64 diffTh  = 50.0;
	const ieee64 ratioTh =  0.3;
	for (int32 boxnumber=0; boxnumber<4; boxnumber++) {
		if (data.box[boxnumber].minX != -1) {
			int32 minX=data.box[boxnumber].minX;
			int32 maxX=data.box[boxnumber].maxX;
			int32 minY=data.box[boxnumber].minY;
			int32 maxY=data.box[boxnumber].maxY;
			bool deleteBox = false;
			
			
			// check size of box (logos should not be larger than 30 percent of the image size)
			ieee64 n = (maxX-minX)*(maxY-minY);
			if (n==0) n=1.0;
			ieee64 ratio = n / (vidSize[DIM_X] * vidSize[DIM_Y]);
			if (ratio > ratioTh) deleteBox=true;
			
			
			
			// get avg difference in box
			ieee64 diff = 0;
			for (int32 y=minY; y<=maxY; y++) {
				for (int32 x=minX; x<=maxX; x++) {
					diff += img(x,y,0);
				} // for
			} // for
			diff /= n;
			diff = diff * maxAbsValue / 255.0;
			
			// remove, if difference is low
			if (diff < diffTh) deleteBox=true;
			
			if (deleteBox) {
				data.box[boxnumber].minX = -1;
				data.box[boxnumber].maxX = -1;
				data.box[boxnumber].minY = -1;
				data.box[boxnumber].maxY = -1;
			} // if
			
		} // if
	} // for
	

	
	if (store) {
		storeFrame(img, data);
	} // if
	
	return;	
} // readValues


// find the nearest border the sysmbol is attached to
// int border: 0:left, 1:right, 2: top, 3: bottom
void 
LogoDetection::getBorder(LogoDetectionBox &box, int &borderNumber, int32 &distanceToBorder)
{
	VideoDecoder reader (pathFilename);
	ImageSize size = reader.getImageSize();

	borderNumber = 0;
	distanceToBorder = 0;

	int32 d0, d1;
	if (box.maxX != -1) {

		// get box width and height
		ieee64 width  = (box.maxX-box.minX+1) / ((ieee64)size[DIM_X]);
		ieee64 height = (box.maxY-box.minY+1) / ((ieee64)size[DIM_Y]);

		if (width > height) {
			d0 = box.maxY;
			d1 = size[DIM_Y] - 1 - box.minY;

			if (d0<d1) {
				// put to top
				borderNumber = 2;
				distanceToBorder = d0;
			} else {
				// put to bottom
				borderNumber = 3;
				distanceToBorder = d1;
			} // else
		} else {
			d0 = box.maxX;
			d1 = size[DIM_X] - 1 - box.minX;

			if (d0<d1) {
				// put to left
				borderNumber = 0;
				distanceToBorder = d0;
			} else {
				// put to right
				borderNumber = 1;
				distanceToBorder = d1;
			} // else
		} // else


	}

	return;
}

