/*
   This file is part of the MoCA video library.
   Name: MinMaxFeature.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Stephan Kopf

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kopf@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

*/

#ifndef MINMAX_FEATURE_H
#define MINMAX_FEATURE_H

// C++ includes
#include <list>
//#include <stack>

// MoCA includes
#include <Image.h>
#include <ImagePosition.h>
#include <MocaSignal.h>
#include <GaussFilter.h>
#include <FillFilter.h>


class MinMaxFeature
{
public:
    MinMaxFeature ();
    MinMaxFeature (ieee64 percentNoise);
    MinMaxFeature (ieee64 percentNoise, uint8 gaussRadius);
    ~MinMaxFeature ();
	void process(list<ImagePosition> &src, Image<uint8> &img, int gaussSmooth, bool isConcave, bool saveContour);
	void process(list<ImagePosition> &src, Image<uint8> &img, string destFileName);

	// static variables
	const static uint8 WHITE;
	const static uint8 BLACK;
	const static uint8 GRAY;
	const static uint16 MAX_SIGMA_SIZE;


private:
	// functions
	//void fill (Image<uint8> &img, ImagePosition firstIp, uint8 color);
	//void robustFill (Image<uint8> &img, ImagePosition firstIp, uint8 color);
	//void fill2Colors (Image<uint8> &img, ImagePosition firstIp, uint8 color2, uint8 color);
	void fillHoles(Image<uint8> &img);
	void fillConcaveHoles(Image<uint8> &img);
	void mark(int x1,int y1,int x2,int y2,int value, Image<uint8> &img, Signal<bool> &isConcave);
    inline ieee64 abs (ieee64 x);
	uint8 findNewValue(int x, uint8& minValue, uint8& maxValue, Signal<bool> &isConcave);

	// variables
	uint8  gaussRadius;
	ieee64 percentNoise;
};

inline ieee64
MinMaxFeature::abs (ieee64 x)
{
    return (x<0) ? (-x) : x; 
} // abs

#include <MinMaxFeature.cpt>

#endif







