/*
    ObjectMaskFeature.cpp     get an object mask for each frame in a shot
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// MoCA includes
#include <ObjectMaskShot.h>




ObjectMaskShot::ObjectMaskShot ()
{
	// global parameters
	cropLeft   = 4;
	cropRight  = 4;
	cropTop    = 4;
	cropBottom = 4;
}

#if 0
TODO!!!

void 
ObjectMask::process(const Video<uint8> &src, Video<uint8> &mask) 
{
	VideoSize size = src.getSize();
	for (int32 i=0; i<size[DIM_Z]; i++) {
		Image<UINT(>
	Video<uint8> &src


	// get camera parameters
	CameraModelData cameraParameters;
	{
		CameraModel reader(pathFilename);
		reader.process(cameraParameters);
	}

	// initialize shots
	ieee64 width  = (srcVidSize[DIM_X]-cropLeft-cropRight) / ((ieee64) srcVidSize[DIM_X]);
	ieee64 height = (srcVidSize[DIM_Y]-cropTop-cropBottom) / ((ieee64) srcVidSize[DIM_Y]);
	if ((width<0.5) || (height<0.5)) {
		// do not calculate frames (no objects found)
		cerr << "ObjectMask::readValues: no objects found!" << endl;
		return;
	} // if

	// initilialize frame list
	Signal<bool> frameSelected;
	initializeFrameList(cameraParameters, frameSelected);

	// select valid frames
	analyseMpegVideo (videoObjects, cameraParameters, frameSelected);

	// check, if results are valid
	validateResults(videoObjects);


	return;
} // process

// -----------------------------------------------------------------------


void 
ObjectMask::validateResults (ObjectMaskData &videoObjects) 
{

	list<ObjectMaskItem>::iterator it;
	for (it=videoObjects.frameData.begin(); it!=videoObjects.frameData.end(); it++) {
		ObjectMaskItem item = (*it);
		
		item.borderMaxX  = 0;
		item.borderMaxY  = 0;
		item.borderMinX  = 10000;
		item.borderMinY  = 10000;
		item.centerX     = 0;
		item.centerY     = 0;
		item.length      = 0;
		item.reliability = 0;

		ieee64 lengthSum = 0.0;
		for (int32 i=0; i<ContourParametrization::DEFAULT_CONTOUR_SIZE; i++) {
			int32 px = item.contour(i,0);
			int32 py = item.contour(i,1);

			lengthSum += sqrt((ieee64)(px-item.contour(i+1,0))*(px-item.contour(i+1,0)) + 
				                      (py-item.contour(i+1,1))*(py-item.contour(i+1,1)));

			if (item.borderMinX > px) item.borderMinX = px;
			if (item.borderMaxX < px) item.borderMaxX = px;
			if (item.borderMinY > py) item.borderMinY = py;
			if (item.borderMaxY < py) item.borderMaxY = py;
			item.centerX += px;
			item.centerY += py;
		} // for
		item.length = (int32) (lengthSum+0.5);
		item.centerX /= ContourParametrization::DEFAULT_CONTOUR_SIZE;
		item.centerY /= ContourParametrization::DEFAULT_CONTOUR_SIZE;

		(*it) = item;
	} // for


	// analyse each frame
	//  - get absolute width and height (if very small or very large --> not reliable)
	//  - aspect ratio is strange (--> not reliable)
	const ieee64 thMinWidth  = 0.08*srcVidSize[DIM_X]+1;
	const ieee64 thMaxWidth  = 0.5 *srcVidSize[DIM_X]+1;
	const ieee64 thMinHeight = 0.08*srcVidSize[DIM_Y]+1;
	const ieee64 thMaxHeight = 0.6 *srcVidSize[DIM_Y]+1;
	const ieee64 thRatio     = 0.2;
	const ieee64 thMaxLength = 1.5 *(srcVidSize[DIM_X]+srcVidSize[DIM_Y]);
	const ieee64 thMinLength = 0.05 *thMaxLength;
	for (it=videoObjects.frameData.begin(); it!=videoObjects.frameData.end(); it++) {
		ObjectMaskItem item = (*it);

		ieee64 rel = 1000.0;
		ieee64 width  = item.borderMaxX - item.borderMinX +1;
		if (width < thMinWidth) {
			ieee64 p = 1.0 - (thMinWidth-width) / thMinWidth; 
			if (p<0.1) p=0.1;
			if (p>1.0) p=1.0;
			rel *= p;
		} // if
		if (width > thMaxWidth) {
			ieee64 p = 1.0 - (width - thMaxWidth) / (srcVidSize[DIM_X]-thMaxWidth); 
			if (p<0.1) p=0.1;
			if (p>1.0) p=1.0;
			rel *= p;
		} // if

		ieee64 height = item.borderMaxY - item.borderMinY +1;
		if (height < thMinHeight) {
			ieee64 p = 1.0 - (thMinHeight-height) / thMinHeight; 
			if (p<0.1) p=0.1;
			if (p>1.0) p=1.0;
			rel *= p;
		} // if
		if (height > thMaxHeight) {
			ieee64 p = 1.0 - (height - thMaxHeight) / (srcVidSize[DIM_Y]-thMaxHeight); 
			if (p<0.1) p=0.1;
			if (p>1.0) p=1.0;
			rel *= p;
		} // if

		ieee64 ratio = width / height;
		if (ratio>1.0) ratio = 1.0/ratio;
		if (ratio<thRatio) {
			ieee64 p = 1.0 - (thRatio-ratio) / (thRatio); 
			if (p<0.1) p=0.1;
			if (p>1.0) p=1.0;
			rel *= p;
		} // if

		if (item.length > thMaxLength) {
			ieee64 p = 1.0 - (item.length - thMaxLength) / (2*thMaxLength); 
			if (p<0.1) p=0.1;
			if (p>1.0) p=1.0;
			rel *= p;
		} // if

		if (item.length < thMinLength) {
			ieee64 p = 1.0 - (thMinLength - item.length) / thMinLength; 
			if (p<0.1) p=0.1;
			if (p>1.0) p=1.0;
			rel *= p;
		} // if

		item.reliability = (int32)(rel+0.5);
		(*it) = item;
	} // for


	/*
	vector<TrackingFeatureData> data;
	for (it=videoObjects.frameData.begin(); it!=videoObjects.frameData.end(); it++) {
		ObjectMaskItem item = (*it);
		TrackingFeatureData d(item.centerX, item.centerY);
		data.push_back(d);
	}
	TrackingFeature track;
	track.process(data);
	*/



	// analyse all frames of one shot
	//  - track center (linear motion)
	//  - track width and height
	//  - set reliability depending on all other frames in this shot
	//  -- existing validity
	/*
	SignalSize analyseDataSize (100,4);
	Signal<int32> analyseData (analyseDataSize);
	for (it=videoObjects.frameData.begin(); it!=videoObjects.frameData.end(); it++) {
		ObjectMaskItem item = (*it);

		// copy data to analyseData
		int n=0;
		while ((it!=videoObjects.frameData.end()) && ((*it).shot == item.shot)) {
			analyseData(n,0) = (*it).borderMinX;
			analyseData(n,1) = (*it).borderMaxX;
			analyseData(n,2) = (*it).borderMinY;
			analyseData(n,3) = (*it).borderMaxY;
			analyseData(n,4) = (*it).centerX;
			analyseData(n,5) = (*it).centerY;
			analyseData(n,6) = (*it).length;
			analyseData(n,7) = (*it).reliability;
			analyseData(n,8) = 0; // distance to center of objects to previous and next frame
			it++;
		} // while
		it--;


		// analyse data
		int32 x0 = analyseData(0,4);
		int32 y0 = analyseData(0,5);
		int32 x1 = analyseData(1,4);
		int32 y1 = analyseData(1,5);
		analyseData(0,8)   = 2.0 * sqrt( ((x0-x1)*(x0-x1)) + ((y0-y1)*(y0-y1)) );

		x0 = analyseData(n-2,4);
		y0 = analyseData(n-2,5);
		x1 = analyseData(n-1,4);
		y1 = analyseData(n-1,5);
		analyseData(n-1,8) = 2.0 * sqrt( ((x0-x1)*(x0-x1)) + ((y0-y1)*(y0-y1)) );

		for (int32 i=1; i<n-1; i++) {
			x0 = analyseData(i, 4);
			y0 = analyseData(i, 5);
			x1 = analyseData(i+1,4);
			y1 = analyseData(i+1,5);
			analyseData(i,8) = sqrt( ((x0-x1)*(x0-x1)) + ((y0-y1)*(y0-y1)) );
		} // for



	} // for
*/

	return;
}


void 
ObjectMask::readValues (ObjectMaskData &videoObjects) 
{

	// get camera parameters
	CameraModelData cameraParameters;
	{
		CameraModel reader(pathFilename);
		reader.process(cameraParameters);
	}

	// initialize shots
	videoObjects.version = OBJECT_MASK_VERSION;
	videoObjects.filename = pathFilename;

	videoObjects.cropLeft   = cropLeft;
	videoObjects.cropRight  = cropRight;
	videoObjects.cropTop    = cropTop;
	videoObjects.cropBottom = cropBottom;

	videoObjects.numberOfItems = 0;
	videoObjects.frameData.clear();

	ieee64 width  = (srcVidSize[DIM_X]-cropLeft-cropRight) / ((ieee64) srcVidSize[DIM_X]);
	ieee64 height = (srcVidSize[DIM_Y]-cropTop-cropBottom) / ((ieee64) srcVidSize[DIM_Y]);
	if ((width<0.5) || (height<0.5)) {
		// do not calculate frames (no objects found)
		cerr << "ObjectMask::readValues: no objects found!" << endl;
		return;
	} // if

	// initilialize frame list
	Signal<bool> frameSelected;
	initializeFrameList(cameraParameters, frameSelected);

	// select valid frames
	analyseMpegVideo (videoObjects, cameraParameters, frameSelected);

	// check, if results are valid
	validateResults(videoObjects);

	return;
} // readValues


void 
ObjectMask::initializeFrameList(CameraModelData &cameraParameters, Signal<bool> &frameSelected) 
{
	// initialize shot list (select all frames)
	frameSelected.resize (srcVidSize[DIM_Z]);
	for (int32 i=0; i<srcVidSize[DIM_Z]; i++) {
		frameSelected(i) = true;
	} // for
	frameSelected(srcVidSize[DIM_Z]-1) = false;
	frameSelected(srcVidSize[DIM_Z]-2) = false;


	// find shot boundaries
	//cerr << "find shot boundaries..." << endl;
	CutDetectionData cuts;
	{
		CutDetection reader (pathFilename);
		reader.process (cuts);
	}



	// unselect frames near shot boundaries (false)
	for (int32 i=0; i<cuts.numberOfFrames; i++) {
		if ((cuts.cuts(i) != SHOT_CHANGE_NO_CUT) && (cuts.cuts(i) != SHOT_CHANGE_UNKNOWN)) {
			frameSelected(i-2) = false;
			frameSelected(i-1) = false;
			frameSelected(i)   = false;
			frameSelected(i+1) = false;
			frameSelected(i+2) = false;
		} // if
	} // for

	// unselect frames with illegal camera parameters (false)
	int32 n = cameraParameters.data.size();
	for (int32 i=0; i<n; i++) {

		if (cameraParameters.data[i].valid!=0) {

			int32 frameNo = cameraParameters.data[i].frame;
			frameSelected(frameNo-2) = false;
			frameSelected(frameNo-1) = false;
			frameSelected(frameNo)   = false;
			frameSelected(frameNo+1) = false;
			frameSelected(frameNo+2) = false;
		} // if
	} // for

	return;
} // initializeShots



#if 0
// debug 
void storeShot (string &fnYuv, string &fnImg, string &fnData, Video<uint8> &vid)
{
	WriteYVideo (vid, fnYuv);
	Image<uint8> img = vid.getImage((vid.getSize()[DIM_Z])/2);
	WriteImage (img, fnImg);
	
	ofstream ofs (fnData.c_str(), std::ios::out | std::ios::binary);
	ofs << vid.getSize()[DIM_X] << endl;
	ofs << vid.getSize()[DIM_Y] << endl;
	ofs << vid.getSize()[DIM_Z] << endl;
	ofs << vid.getSize()[DIM_BAND] << endl;
	ofs.close();
	
	return;
}

void readShot (string &fnYuv, string &fnData, Video<uint8> &vid)
{
	VideoSize size;
	ifstream ifs (fnData.c_str(), std::ios::in | std::ios::binary);
	ifs >> size[DIM_X];
	ifs >> size[DIM_Y];
	ifs >> size[DIM_Z];
	ifs >> size[DIM_BAND];
	ifs.close();
	
	ReadYVideo (fnYuv, size, vid);
	return;
}
#endif


void 
ObjectMask::analyseMpegVideo(ObjectMaskData &videoObjects, CameraModelData &cameraParameters, Signal<bool> &frameSelected) 
{

	// open MPEG reader
	int32 lastFrameToDecode = -1;
	int cacheSize = 3;
	VideoCodec cMode=CODEC_Y;
	VideoDecoder mpegReader (pathFilename, cacheSize, lastFrameToDecode, cMode);
	Image<uint8> img;

	int32 shotNumber=-1;
	for (int32 i=0; i<srcVidSize[DIM_Z]; i++) {

		// analyse next shot
		shotNumber++;

		// find first frame
		while ((i<srcVidSize[DIM_Z]) && (!frameSelected(i))) i++;
		int32 firstFrame=i;

		// find last frame
		while ((i<srcVidSize[DIM_Z]) && (frameSelected(i))) i++;
		int32 lastFrame=i;
		int32 numberOfFrames = lastFrame-firstFrame+1;

		// the shot should contain a minimum number of frames
		if (numberOfFrames < thMinimumShotLength) {
			continue;
		} // if

#if 0
		// debug!
		int dShotNumber=49;
		if (shotNumber != dShotNumber) {
			continue;
		}
#endif

		// calculate size of background image
		int32 bgSizeX = 0;
		int32 bgSizeY = 0;
		int32 bgShiftX = 0;
		int32 bgShiftY = 0;
		{
			int32 minX=0, maxX=0, minY=0, maxY=0;
			int32 width  = 100000;
			int32 height = 100000;
			int32 numberOfValidFrames = 0;
			while ((width>=thMaxBackgroundWidth) || (height>=thMaxBackgroundHeight)) {
				
				calcBackgroundSize(firstFrame, lastFrame, cameraParameters, 
					minX, maxX, minY, maxY, numberOfValidFrames);
				width  = maxX - minX+1;
				height = maxY - minY+1;
				
#if 0
				// debug
				cerr << "background: " 
					<< firstFrame << "   " << lastFrame << "  " 
					<< minX << "  " << maxX << "  " << minY << "  " << maxY << "  "
					<< "width: " << (maxX-minX) << "   height: " << (maxY-minY) << " "
					<< "numberOfValidFrames: " << numberOfValidFrames
					<< endl;
#endif
				
				if ((width>=thMaxBackgroundWidth) || (height>=thMaxBackgroundHeight)) {
					numberOfFrames = lastFrame-firstFrame+1;
					int32 reduceFrames = numberOfFrames - numberOfValidFrames;
					reduceFrames = reduceFrames/5 + 1;
					firstFrame +=reduceFrames;
					lastFrame -=reduceFrames;
				} // if
			} // while
			
			
			// set background size and position
			/*bgSizeX = maxX-minX+1+16;
			bgSizeY = maxY-minY+1+16;
			bgSizeX = ((bgSizeX+1)/2)*2;
			bgSizeY = ((bgSizeY+1)/2)*2;
			bgShiftX = bgSizeX-analyseVidSize[DIM_X]-8-1;
			bgShiftY = bgSizeY-analyseVidSize[DIM_Y]-8-1;
			*/
			bgSizeX = maxX-minX+12;
			bgSizeY = maxY-minY+12;
			bgShiftX = -minX+6;
			bgShiftY = -minY+6;
		}

		// the shot should still contain a minimum number of frames
		numberOfFrames = lastFrame-firstFrame+1;
		if (numberOfFrames < thMinimumShotLength) {
			continue;
		} // if


		// too many frames in shot?
		if (numberOfFrames > thMaxFramesPerShot) {

			// remove frames at the begining and end of the shot
			int32 nFrames = (numberOfFrames - thMaxFramesPerShot) / 5;
			firstFrame+=nFrames;
			lastFrame -=nFrames;
			numberOfFrames = lastFrame-firstFrame+1;
		} // if
		

		// still too many frames in shot?
		ieee64 skipFactor = 0.0;
		if (numberOfFrames > thMaxFramesPerShot) {

			// skip frames
			skipFactor = numberOfFrames - thMaxFramesPerShot;
			skipFactor /= (thMaxFramesPerShot-1);
			if (skipFactor<0.0) skipFactor=0.0;
			if (skipFactor>5.0) skipFactor=5.0;
		} // if
		numberOfFrames = lastFrame-firstFrame+1;
		if (numberOfFrames > thMaxFramesPerShot) {
			numberOfFrames = thMaxFramesPerShot;
		} // if


		// define frames to select
		SignalSize size(numberOfFrames);
		Signal<int32> frameNumberList (size);

		ieee64 skipSum=0.0;
		for (int32 i=0; i<numberOfFrames; i++) {
			int32 frameNumber = (int32) (skipSum+i+firstFrame+0.5);
			frameNumberList(i) = frameNumber;
			skipSum+=skipFactor;
		} // for


//if (firstFrame < 71800) {
//	continue;
//}
//cerr << "TODO" << endl << endl;



		// read yuv video to vid
		Video<uint8> vid;
		bool error = false;
		error = readYuvVideo(frameNumberList, vid, mpegReader);
		if (error) {
			continue;
		} // if


#if 0
		// debug
		string fnData = "V:/tmpdata.idx";
		string fnImg  = "V:/tmpdata.jpg";
		string fnVid  = "V:/tmpdata.y";
		cerr << "TODO: store shot" << endl;
		storeShot(fnVid, fnImg, fnData, vid);
		exit(1);
		//continue;
#endif		

#if 0
		// debug
		cerr << "TODO: readShot" << endl;
		string fnData = "V:/tmpdata.idx";
		string fnVid  = "V:/tmpdata.y";
		readShot(fnVid, fnData, vid);
#endif		


		// calculate background
		Image<uint8> bgImg;
		Video<uint8> maskVid;
		if (!error) {
			error = calculateObjects (vid, bgSizeX, bgSizeY, bgShiftX, bgShiftY, shotNumber, bgImg, maskVid);
		} // if
		if (error) {
			continue;
		} // if


#if 0
		// debug
		cerr << "debug" << endl;
		WriteYVideo (maskVid, debugPath+"maskVid.yuv");
		WriteImage (bgImg, debugPath+"bgImg.pgm");
		cout << "maskVid: " << maskVid.getSize()[DIM_X] << "  " << maskVid.getSize()[DIM_Y] << endl;
#endif


		// find contour of objects
		getContour (frameNumberList, maskVid, shotNumber, videoObjects);
	} // for

	return;
}

void
ObjectMask::storeFrames (ObjectMaskData &videoObjects)
{
	// open reader
	int32 lastFrameToDecode = -1;
	int cacheSize = 3;
	VideoCodec cMode=CODEC_RGB;
	VideoDecoder reader (pathFilename, cacheSize, lastFrameToDecode, cMode);

	ImageSize imgSize = reader.getImageSize();
	ImageSize maskSize = imgSize;
	maskSize[DIM_BAND] = 1;
	Image<uint8> img (imgSize);
	Image<uint8> mask(maskSize);
	Image<uint8> destImg(maskSize);
	
	DrawLine line (Color::BLACK);
	Image<uint8> objectImage;
	bool objectFound = false;

	list<ObjectMaskItem>::iterator it;
	for (it=videoObjects.frameData.begin(); it!=videoObjects.frameData.end(); it++) {
		ObjectMaskItem item = (*it);
		img = reader.getImage(item.frame);
		cerr << "\rstore frame: " << item.frame << "    " << ends;

		// set filename 
		CreateMocaPath mocaPath (pathFilename, false);
		string path = mocaPath.path;
		string fn   = mocaPath.file;
		fn = fn.substr(0,fn.size()-4);

		string fnObject;
		string fnFrame;
		string fnMask;
		string fnObjMask;
		{
		  stringstream streamNumber;
		  streamNumber << setw(4) << setfill('0') << item.shot << "_" 
			       << setw(6) << setfill('0') << item.frame << ".png";
		  fnObject  = path +"object_" +  fn + "_" + streamNumber.str() + ".png";
		  fnFrame   = path +"frame_" +  fn + "_" + streamNumber.str() + ".png";
		  fnMask    = path +"mask_" +  fn + "_" + streamNumber.str() + ".png";
		  fnObjMask = path +"objectMask_" +  fn + "_" + streamNumber.str() + ".png";
		}

		//WriteImage (img, fnFrame);


		// get object mask
		mask.clear(255);
		for (int32 p=0; p<ContourParametrization::DEFAULT_CONTOUR_SIZE; p++) {

			ImagePosition p1(item.contour(p,0), item.contour(p,1));
			ImagePosition p2(item.contour(p+1,0), item.contour(p+1,1));  // use periodic border

			if ((p1[DIM_X]<0) || (p1[DIM_X]>=imgSize[DIM_X]-1) || (p1[DIM_Y]<0) || (p1[DIM_Y]>=imgSize[DIM_Y]-1) ||
				(p2[DIM_X]<0) || (p2[DIM_X]>=imgSize[DIM_X]-1) || (p2[DIM_Y]<0) || (p2[DIM_Y]>=imgSize[DIM_Y]-1) ) {
				cerr << "ObjectMask::storeFrames: illigal object position (" << p1[DIM_X] << "," << p1[DIM_Y] << "),(" << p2[DIM_X] << "," << p2[DIM_Y] << ")" << endl;
			} // if
			line.process(p1,p2,mask);
		} // for
		//WriteImage (mask, fnMask);

		// fill mask
		RegionFeature<uint8> rf;
		rf.process(mask, objectImage, objectFound);
		//WriteImage (objectImage, fnObjMask);


		// convert image (dark background)		
		int v;
		for (int32 y=0; y<maskSize[DIM_Y]; y++) {
			for (int32 x=0; x<maskSize[DIM_X]; x++) {
				v = img(x,y);
				if (objectImage(x,y) == 255) {
					v = v/2;
				} else {
					v = 129 + v/2;
				} // else
				if (v<0) v=0;
				if (v>255) v=255;

				destImg (x,y) = (uint8)v;
			} // for
		} // for

		// write images
		//WriteImage (img, fnObject);
		if (objectFound) {
			//WriteImage (destImg, fnObject);
		} // if
	} // for

	return;
}

void
ObjectMask::getContour (const Signal<int32> &frameNumberList, const Video<uint8> &maskVid, 
						const int32 shotNumber, ObjectMaskData &videoObjects)
{
	VideoSize size = maskVid.getSize();
	for (int32 z=0; z<size[DIM_Z]; z++) {
		int32 frame = frameNumberList(z);
		ObjectMaskItem item (frame, shotNumber);
		bool valid = true;


        // find and parametrize contour
        vector<ImageEdgePosition> iepList;
		uint8 backgroundColor = 0;
        ContourEdgeList contourEdge (backgroundColor);
        contourEdge.process (maskVid.getImage(z), iepList);
		if (iepList.size() == 0) continue;
		
        vector<ImagePosition> imgPosList;
		ContourParametrization cp(ContourParametrization::DEFAULT_CONTOUR_SIZE);
		cp.process(iepList, imgPosList);
		if (imgPosList.size() != ContourParametrization::DEFAULT_CONTOUR_SIZE) continue;


		// copy values to item
		int32 minX=100000;
		int32 minY=100000;
		int32 maxX=-1;
		int32 maxY=-1;
        vector<ImagePosition>::iterator it;
		int32 number=0;
		for (it = imgPosList.begin(); it != imgPosList.end(); it++) {
			int32 px = (*it)[DIM_X]+cropLeft;
			int32 py = (*it)[DIM_Y]+cropTop;
			if (minX>px) minX=px;
			if (minY>py) minY=py;
			if (maxX<px) maxX=px;
			if (maxY<py) maxY=py;
			item.contour(number,0) = px;
			item.contour(number,1) = py;
			number++;
		} // for


#if 0
		// debug
		// set image to background color
		ImageSize imgSize (size[DIM_X], size[DIM_Y], 1);
		Image<uint8> tmp (imgSize);
		tmp.clear(255);
        DrawDot dot;
		for (int32 number=0; number<ContourParametrization::DEFAULT_CONTOUR_SIZE; number++) {
			ImagePosition p(item.contour(number,0)-cropLeft, item.contour(number,1)-cropTop);
            dot.process(p, Color::BLACK, tmp);
        } // for

		cerr << "debug" << endl;
		string debugPath = "V:/object_recognition/results/";
		WriteImage (tmp, FileHandling::getNumberedFilename (debugPath + "objects_", item.frame, ".pgm"));
#endif

		int32 width = maxX - minX + 1;
		int32 height  = maxY - minY + 1;
		if (width > 0.75 * size[DIM_X]) {
			valid = false;
		} // if
		if (height > 0.75 * size[DIM_Y]) {
			valid = false;
		} // if

		if (valid) {
			videoObjects.numberOfItems++;
			videoObjects.frameData.push_back(item);
		} // if
	} // for

	
	return;
} // 

void
ObjectMask::getObjectSize (Image<uint8> &img, int32 &minX, int32 &maxX, int32 &minY, int32 &maxY)
{
	ImageSize size = img.getSize();
	minX = 0;
	minY = 0;
	maxX = size[DIM_X]-1;
	maxY = size[DIM_Y]-1;

	bool objFound=false;
	while ((minY<size[DIM_Y]-1) && (!objFound)) {
		for (int32 x=0; x<size[DIM_X]; x++) {
			if (img(x,minY,0)!=0) {
				objFound=true;
				break;
			} // if
		} // for
		minY++;
	} // while

	objFound=false;
	while ((minX<size[DIM_X]-1) && (!objFound)) {
		for (int32 y=0; y<size[DIM_Y]; y++) {
			if (img(minX,y,0)!=0) {
				objFound=true;
				break;
			} // if
		} // for
		minX++;
	} // while

	objFound=false;
	while ((maxY>0) && (!objFound)) {
		for (int32 x=0; x<size[DIM_X]; x++) {
			if (img(x,maxY,0)!=0) {
				objFound=true;
				break;
			} // if
		} // for
		maxY--;
	} // while

	objFound=false;
	while ((maxX>0) && (!objFound)) {
		for (int32 y=0; y<size[DIM_Y]; y++) {
			if (img(maxX,y,0)!=0) {
				objFound=true;
				break;
			} // if
		} // for
		maxX--;
	} // while

	return;
} // getObjectSize 



bool 
ObjectMask::calculateObjects (const Video<uint8> &vid, 
							  const int32 bgSizeX, const int32 bgSizeY, const int32 bgShiftX, const int32 bgShiftY,
							  const int32 shotNumber, Image<uint8> &bgImg, Video<uint8> &maskVid)
{
//	string debugPath = "V:/object_recognition/results/";
	maskVid.resize(vid.getSize());

	bool error = false;
	list<TransformationData> currTransformation;
	try {

		bool objects = true;
		vector<TransformationData> vidTrans;
		CameraBackgroundFeature cbf (vid.getSize(), bgSizeX, bgSizeY, bgShiftX, bgShiftY, objects, preciseCalculation);
		cbf.process (vid, maskVid, bgImg, vidTrans);

		// store data
		cerr << "ObjectMask: calculateObjects: store data" << endl;
		string fnPrefix = FileHandling::getNumberedFilename(destPath,shotNumber,"");
		//cbf.storeData (maskVid, bgImg, fnPrefix);


		// store additional data
		cerr << "ObjectMask: calculateObjects: store additional data" << endl;
		fnPrefix = FileHandling::getNumberedFilename(destPath,shotNumber,"_");
		for (int32 i=0; i<vid.getSize()[DIM_Z]; i++) {
			string fnPre = FileHandling::getNumberedFilename(fnPrefix,i,"");
			cbf.addObject (vid.getImage(i), maskVid.getImage(i), bgImg, vidTrans[i], fnPre);
		} // for
		exit(1);
		

#ifdef 0
		{
			cerr << "debug" << endl;
			string path = FileHandling::getNumberedFilename("C:/test/object_recognition_bgImg_",shotNumber,".png");
			WriteImage (bgImg, path);
		}
#endif


#if 0
		// write background
		cerr << "debug" << endl;
		WriteYVideo (vid, debugPath+"vid.yuv");
		WriteYVideo (maskVid, debugPath+"maskVid.y");
		WriteImage (bgImg, debugPath+"bgImg.pgm");
		cout << endl << "vid: " << vid.getSize()[DIM_X] << "  " << vid.getSize()[DIM_Y] << endl;
		cout << "maskVid: " << maskVid.getSize()[DIM_X] << "  " << maskVid.getSize()[DIM_Y] << endl;
#endif


		// get valid objects
		VideoSize objSize = vid.getSize();
		int32 numberOfFrames = objSize[DIM_Z];


#if 0
		int32 px = 106;
		int32 py = 64;
#endif
		// get largest object
		for (int32 z=0; z<numberOfFrames; z++) {

#ifdef 0
			{
				string path1 = FileHandling::getNumberedFilename("C:/test/object_recognition_vid_",shotNumber,"_");
				string path2 = FileHandling::getNumberedFilename("",z,".png");
				path1 = path1 + path2;
				ImageSize size = vid.getImage(z).getSize();
				size[DIM_BAND]=3;
				Image<uint8> tmp (size);
				Image<uint8> tmp1 = vid.getImage(z);
				Image<uint8> tmp2 = maskVid.getImage(z);

#if 0
				// get region in area: debug!
				Image<uint32> regions;
				RegionFeature<uint8> rf (1, N8);
				rf.process (tmp2, regions);
				int32 mr = rf.getLargestRegion();
				int32 r=-1;

				cerr << "debug" << endl;
				WriteImage (regions, "C:/test3.png");
				bool b = true;
				while (b) {
					if (tmp2 (px,py) == 255) {
						b = false;
					}
					if (b) {
						cout << endl;
					}
				}
				if (tmp2 (px,py) == 255) {
					r = regions(px,py);
				}

				ImageSize tSize = tmp2.getSize();
				Image<uint8> t = tmp2;
				t.clear(0);
				for (int32 y=0; y<tSize[DIM_Y]; y++) {
					for (int32 x=0; x<tSize[DIM_X]; x++) {
						if (regions(x,y) == r)
							t(x,y) = 255;
					} // for
				} // for
				cerr << "debug" << endl;
				WriteImage (t, "C:/test1.png");

				Image<uint8> largestObjectInImage;
				bool objectFound;
				rf.process (t, largestObjectInImage, objectFound);
				WriteImage (largestObjectInImage, "C:/test.png");
#endif

				for (int32 y=0; y<size[DIM_Y]; y++) {
					for (int32 x=0; x<size[DIM_X]; x++) {
						tmp(x,y,0) = tmp1(x,y);
						tmp(x,y,1) = tmp1(x,y);
						tmp(x,y,2) = tmp1(x,y);

#if 0
						if (largestObjectInImage(x,y) == 0)
#endif
						if (tmp2(x,y) == 255) 
						{
							int32 v = tmp1(x,y);
							tmp(x,y,0) = (v+255+255)/3;
							tmp(x,y,1) = (v)/2;
							tmp(x,y,2) = (v)/2;
						} // if
					} // for
				} // for
				cerr << "debug" << endl;
				WriteImage (tmp, path1);
			}
#endif


			try {
				bool objectFound = false;
				Image<uint8> objectImage;

				RegionFeature<uint8> rf;
				rf.process(maskVid.getImage(z), objectImage, objectFound);
				if (!objectFound) {
					objectImage.clear(255);
				} // if

				// store object
				maskVid.setImage(z, objectImage);

#if 0
				// debug
				cerr << "debug" << endl;
				cout << "object: " << FileHandling::getNumberedFilename (debugPath + "object_one_", z, ".pgm") << endl;
				WriteImage (objectImage, FileHandling::getNumberedFilename (debugPath + "object_one_", z, ".pgm"));
#endif

			} catch(...) {
				error = true;
				cerr << "ObjectMask::calculateObjects: exception in RegionFeature!" << endl;
			} // catch

		} // for

	} catch(...) {
		cerr << "ObjectMask::calculateObjects: unknown exception!" << endl;
	} // catch


	return error;
} // calculateObjects




void 
ObjectMask::calcBackgroundSize(int32 firstFrame, int32 lastFrame, CameraModelData &cameraParameters, 
							   int32 &minX1, int32 &maxX1, int32 &minY1, int32 &maxY1, int32 &numberOfValidFrames) 
{
	numberOfValidFrames = 0;


	// set size of background
	ieee64 xMin = 0;
	ieee64 xMax = analyseVidSize[DIM_X]-1;
	ieee64 yMin = 0;
	ieee64 yMax = analyseVidSize[DIM_Y]-1;

	ieee64 vx=0, vy=0;
	
	// get parameters for each frame
	for (int32 i=firstFrame; i<lastFrame; i++) {
		
		// find camera parameters (the frames are sorted)
		if (i>=(int32)cameraParameters.data.size()) {
			cerr << "ObjectMask:: could not find camera parameters of frame " << i << endl;
			exit(1);
		} // if
		if (cameraParameters.data[i].frame != i) {
			cerr << "ObjectMask:: could not find camera parameters of frame " << i << endl;
		} // if
		
		// test 9 positions for each frame
		ieee64 px0 = xMin;
		ieee64 px1 = xMax;
		ieee64 py0 = yMin;
		ieee64 py1 = yMax;
		int32 xStep = (int32)(xMax - xMin +1)/2;
		int32 yStep = (int32)(yMax - yMin +1)/2;
		for (int32 y=0; y<3; y++) {
			for (int32 x=0; x<3; x++) {
				
				// get motion vectors vx, vy
				ieee64 px = xMin + x*xStep;
				if (px>xMax) px = xMax;
				ieee64 py = yMin + y*yStep;
				if (py>yMax) py = yMax;
				cameraParameters.data[i].getInverseVectors (px, py, vx,vy);
				
				// update frame-background size
				if (px0 > vx) px0 = vx;
				if (px1 < vx) px1 = vx;
				if (py0 > vy) py0 = vy;
				if (py1 < vy) py1 = vy;
			} // for
		} // for

		// update position
		xMin = px0;
		xMax = px1;
		yMin = py0;
		yMax = py1;

		// check size of background
		int32 width  = (int32)(xMax - xMin +1);
		int32 height = (int32)(yMax - yMin +1);
		if ((width < thMaxBackgroundWidth) && (height < thMaxBackgroundHeight)) {
			numberOfValidFrames = i-firstFrame+1;
		} // if
	} // for

	minX1 = (int32)xMin;
	maxX1 = (int32)(xMax+1);
	minY1 = (int32)yMin;
	maxY1 = (int32)(yMax+1);

	return;
}


// read the shot to memory
bool 
ObjectMask::readYuvVideo(Signal<int32> &frameNumberList, Video<uint8> &vid, VideoDecoder &mpegReader) 
{
	bool error=false;
	try {

		// clip video, 	reserve pixel value 255 for "undefined"
		ClipFilter<uint8> clip(0,254);

		Image<uint8> img1;
		Image<uint8> img2;
		ImageSize imgSize = mpegReader.getImageSize();
		CropFilter<uint8> crop (imgSize, cropLeft, cropRight, cropTop, cropBottom);

		int32 numberOfFramesInShot = frameNumberList.getSize()[DIM_X];
		VideoSize vidSize = analyseVidSize;
		vidSize[DIM_Z] = numberOfFramesInShot;
		vidSize[DIM_BAND] = 1;
		vid.resize (vidSize);

		for (int i=0; i<numberOfFramesInShot; i++) {
			
			int32 frame = frameNumberList(i);
			
			// read next image
			img1 = mpegReader.getImage(frame);
			cerr << "\rread frame: " << frame << ends;
			
			// crop image
			crop.process(img1,img2);

			// clip image
			clip.process(img2);
			
			//set frame
			vid.setImage(i,img2);
		} // for
	} catch (...) {
		error = true;
		cerr << "ObjectMask::could not read MPEG shot  " << endl << endl;
	} // catch
	//cout << endl;

	return error;
} // readYuvVideo() 



#endif
