/*
    SceneDetection.cpp     detect scenes in videos
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// MoCA includes
#include <SceneDetection.h>


std::ostream& operator<< (ostream &os, const SceneDetectionData &data) 
{
	os << endl;
	os << "version                           : " << data.version << endl;
	os << "filename                          : " << data.filename << endl;
	os << "number of possible scenes (shots) : " << data.numberSceneItems<< endl;
	os << "number of scenes                  : " << data.numberOfScenesIDs<< endl;
	if (data.usePredefinedClusters) {
		os << "use predefined clusters           : yes" << endl;
	} else {
		os << "use predefined clusters           : yes" << endl;
	} // else
	os << endl;

	
	if (data.numberSceneItems > 0) {


		list<SceneDetectionItem>::const_iterator it;

		os << "   id   first- reference-   last-frame  not relevant" << endl;
		for (it=data.scenes.begin(); it!=data.scenes.end(); it++) {
			os << setw(5) << (*it).sceneID 
			   << setw(8) << (*it).firstFrame
			   << setw(11) << (*it).referenceFrame
			   << setw(8) << (*it).lastFrame;
			if ((*it).irrelevantFrame) {
				os << "           yes";
			}
			os << endl;
		} // for
		os << endl;

		/*
		os << "   id  data" << endl;
		for (it=data.scenes.begin(); it!=data.scenes.end(); it++) {
			os << setw(5) << (*it).sceneID << "  ";
			for (int32 i=0; i<(*it).hist.getSize()[DIM_X]; i++) {
				os << (*it).hist(i) << " ";
			} // for
			os << endl;
		} // for
		*/
	} // if

	return os;
} // operator <<

OTagStream& operator<< (OTagStream& os, const SceneDetectionData &data) 
{
    TagFormat::writeStartTag (os, "SCENE_DESCRIPTION");
	os << endl;
	
    TagFormat::writeNodeValue(os, "version", data.version, 1);
    TagFormat::writeNodeValue(os, "filename", data.filename, 1);
    TagFormat::writeNodeValue(os, "numberSceneItems", data.numberSceneItems, 1);
    TagFormat::writeNodeValue(os, "numberOfScenesIDs", data.numberOfScenesIDs, 1);
    TagFormat::writeNodeValue(os, "usePredefinedClusters", data.usePredefinedClusters, 1);
	os << endl;

	// write scenes
	if (data.numberSceneItems > 0) {

		list<SceneDetectionItem>::const_iterator it;
		for (it=data.scenes.begin(); it!=data.scenes.end(); it++) {

			TagFormat::writeStartTag (os, "SCENE", 1);
			os << " ";

			TagFormat::writeStartTag (os, "sceneID", -1);
			os << setfill(' ') << setw(6) << (*it).sceneID; 
			TagFormat::writeEndTag (os, "sceneID",-1);
			os << " ";

			TagFormat::writeStartTag (os, "first", -1);
			os << setfill(' ') << setw(6) << (*it).firstFrame; 
			TagFormat::writeEndTag (os, "first", -1);
			os << " ";

			TagFormat::writeStartTag (os, "ref", -1);
			os << setfill(' ') << setw(6) << (*it).referenceFrame; 
			TagFormat::writeEndTag (os, "ref", -1);
			os << " ";

			TagFormat::writeStartTag (os, "last", -1);
			os << setfill(' ') << setw(6) << (*it).lastFrame; 
			TagFormat::writeEndTag (os, "last", -1);
			os << " ";

			TagFormat::writeStartTag (os, "irrelevant", -1);
			if ((*it).irrelevantFrame) {
				os << "yes"; 
			} else {
				os << " no"; 
			} // else
			TagFormat::writeEndTag (os, "irrelevant", -1);
			os << " ";

			TagFormat::writeEndTag (os, "SCENE", 1);
		} // for
	} // if

	os << endl;
    TagFormat::writeEndTag (os, "SCENE_DESCRIPTION");
	os << endl;
	
    return os;
} // operator<<


ITagStream& operator>> (ITagStream& is, SceneDetectionData& data)
{
	
    char c;
    bool failFlag = true;
	
	if (is >> c) {
		failFlag = false;
		is.putback(c);

		TagFormat::readStartTag (is, "SCENE_DESCRIPTION");
		
		TagFormat::readNodeValue(is, "version", data.version);
		TagFormat::readNodeValue(is, "filename", data.filename);
		TagFormat::readNodeValue(is, "numberSceneItems", data.numberSceneItems);
		TagFormat::readNodeValue(is, "numberOfScenesIDs", data.numberOfScenesIDs);
		TagFormat::readNodeValue(is, "usePredefinedClusters", data.usePredefinedClusters);

		if (is.rdstate() & ios::failbit) {
			throw MocaException ("SceneDetection: Illegal index file format!");
		} // if

		// initialize scenes
		data.scenes.clear();

		// read cuts
		if (is >> c) {
			is.putback(c);

			// read scene data
			if (data.numberSceneItems>0) {

				for (int32 i=0; i<data.numberSceneItems; i++) {
	
					TagFormat::readStartTag(is, "SCENE");

					int32 p1, p2, p3, p4;
					bool p5;
					TagFormat::readNodeValue(is, "sceneID",    p1);
					TagFormat::readNodeValue(is, "first", p2);
					TagFormat::readNodeValue(is, "ref", p3);
					TagFormat::readNodeValue(is, "last",  p4);

					string tmp;
					TagFormat::readNodeValue(is, "irrelevant",  tmp);
					p5=false;
					if (tmp=="yes") p5=true;

					data.scenes.push_back(SceneDetectionItem(p1,p2,p3,p4,p5));

					TagFormat::readEndTag(is, "SCENE");
				} // for

			} // if
			if (is.rdstate() & ios::failbit) {
				throw MocaException ("SceneDetection: Illegal index file format!");
			} // if
		} // if
		
		TagFormat::readEndTag (is, "SCENE_DESCRIPTION");
	} // if
	
	if (is.rdstate() & ios::failbit) {
		failFlag = true;
	} // if
    if (failFlag) {
		throw MocaException ("SceneDetection: Illegal index file format!");
	} else {
        is.clear();
    } // else
  
    return is;
} // operator>> 



SceneDetection::SceneDetection(const string &pathFilename, int numberOfClusters, bool usePredefinedClusters) 
{
	this->pathFilename = pathFilename;
	this->numberOfClusters      = numberOfClusters;
	this->usePredefinedClusters = usePredefinedClusters;
	numberOfPixel = 0;
	videoSize     = 0;
	histSize      = 0;

	// initialize 
	VideoDecoder reader(this->pathFilename);
	VideoSize size = reader.getSize();
	videoSize=size[DIM_Z];
	numberOfPixel=(size[DIM_X]*size[DIM_Y]);

	{
		VideoStatisticData data;
		VideoStatistic vidStat (this->pathFilename);
		vidStat.process(data);
		grayscale = (data.color==TYPE_GRAYSCALE);
	}

	//grayscale = reader.isGrayscale();
} 


void 
SceneDetection::process(SceneDetectionData& data) 
{
	
	// read data
	ReadMocaData<SceneDetectionData> reader (pathFilename, SCENE_DETECTION_FILENAME);
	bool dataValid = reader.process (SCENE_DETECTION_VERSION, data); 
	
	if (!dataValid) {
		
		// create new index
		cout << "SceneDetection: Creating index file ..." << endl;
		readValues(data);
		
		// save index file
		WriteMocaData<SceneDetectionData> writer (pathFilename, SCENE_DETECTION_FILENAME);
		writer.process(data);
	} // if
	return;
} // process



int32 
SceneDetection::histDiff (Signal<uint32> &hist1, Signal<uint32> &hist2)
{
	int32 dist=0;

	// get histogram difference
	for (int32 a=0; a<histSize[DIM_BAND]; a++) {
		for (int32 i=0; i<histSize[DIM_X]; i++) {
			dist += labs(hist1(i,a)-hist2(i,a));
		} // for
	} // for

	return dist;
} // histDiff


#if 0
int32 
SceneDetection::getHistDist (int32 firstBorder, int32 lastBorder, Signal<int32> &hist)
{
	//cout << "[" << firstBorder << ".." << lastBorder << "]"<< endl;
	int32 dist=0;

	if (lastBorder>firstBorder) {

		// initilaize data
		SignalSize size (histSize);
		Signal<int32> avgHist (size);
		for (int32 j=0; j<histSize; j++) {
			avgHist(j)=0;
		} // for
		
		// get average histogram
		for (int32 i=firstBorder; i<=lastBorder; i++) {
			//int32 ttmp=0;
			for (int32 j=0; j<histSize; j++) {
				//int32 tmp = hist(i,j);
				//ttmp+=tmp;
				avgHist(j)+=hist(i,j);
			} // for
		} // for
		for (int32 j=0; j<histSize; j++) {
			avgHist(j)/=(lastBorder-firstBorder+1);
			//cerr << " " << avgHist(j) << endl;
		} // for

		// get histogram difference
		for (int32 i=firstBorder; i<=lastBorder; i++) {
			for (int32 j=0; j<histSize; j++) {
				dist += labs(avgHist(j)-hist(i,j));
			} // for
		} // for
		dist/=(lastBorder-firstBorder+1);
	} // if

	return dist;
} // getHistDist 
#endif



#if 0
void saveScenes (SceneDetectionData& data, string &directory)
{
	cerr << "save video scenes" << endl;
	
	// create video abstracts!
	for (int32 i=0; i<data.numberOfScenes; i++) {
		cerr << endl;
		stringstream oss;
		oss << directory << setfill('0') << setw(4) << i << ".mpg";
		string destFilename = oss.str();
		cerr << "store abstract: " << destFilename << endl;
		
		int32 lastFrameToDecode=0;
		list<SceneDetectionItem>::iterator it; 
		for (it=data.scenes.begin(); it!=data.scenes.end(); it++) {
			if ((*it).sceneID==i) {
				if (lastFrameToDecode < (*it).lastFrame) {
					lastFrameToDecode = (*it).lastFrame;
				} // if
			} // if
		} // for
		
		if (lastFrameToDecode > 0) {
			
			// open reader
			VideoDecoder reader (data.filename);
			reader.setLastFrameToDecode (lastFrameToDecode+3);
			double framerate = reader.getFrameRate();
			bool createGrayscale=false;
			if (reader.isGrayscale()) createGrayscale=true;
			bool cropTo16=true;
			ImageSize imgSize = reader.getImageSize();
			
			// open writer
			OMpegVideo writer (destFilename, imgSize, framerate, createGrayscale, cropTo16);
			
			
			// process scenes (read and save images)
			Image<uint8> img;
			bool firstFrameDublicated=true;
			
			for (it=data.scenes.begin(); it!=data.scenes.end(); it++) {
				if ((*it).sceneID==i) {
					for (int32 frame=(*it).firstFrame; frame<=(*it).lastFrame; frame++) {
						cerr << "\rframe: " << frame << "  ";
						img = reader.getImage(frame);

						if (firstFrameDublicated) {

							// dublicate first frames
							firstFrameDublicated=false;
							for (int k=0; k<5; k++) 
								writer.write (img);
						} else {
							writer.write (img);
						} // else
					} // for
					
				} // if
			} // for

			// dublicate last frames
			for (int k=0; k<5; k++) 
				writer.write (img);
			
			
			// close process
			writer.close();
			reader.close();
		} // if
		
	} // for

	return;
} // saveScenes
#endif


#if 0
void 
SceneDetection::initializeScenes (SceneDetectionData& data, CutDetectionData &cuts, Signal<int32> &hist) {

	cout << "start scene detection" << endl << endl;
	data.filename = filename;
	data.version  = SCENE_DETECTION_VERSION; 
	data.numberOfScenes=0;
	data.scenes.clear();


	// (a) calculate areas: the frames of one area are very similar
	//     (1) read shots
	
	cout << "start module: cut detection" << endl;
	//CutDetectionData cuts;
	{
		CutDetection reader (filename);
		reader.process (cuts);
	}

	//     (2) read histograms
	cout << "start module: histogram analysis" << endl;
	//Signal<int32> hist;
	{
		int msbits = 4;
		bool grayscale=true;
		bool smoothing=true;
		uint8 areas=1;
		HistogramFeature hf (msbits, grayscale, smoothing, areas);
		
		// read histogram values
removed
//VideoCodec codec=CODEC_DVDVIEW_Y;
		int cacheSize=3;
		int32 lastFrameToDecode=-1;
		VideoDecoder reader (filename, codec, cacheSize, lastFrameToDecode);

		Signal<uint32> histOneFrame;
		hf.process(reader.getImage(0), histOneFrame);
		SignalSize histSize (reader.getSize()[DIM_Z], histOneFrame.getSize()[DIM_X]);
		hist.resize(histSize);
		
		for (int32 frame=0; frame<hist.getSize()[DIM_X]; frame++) {
			hf.process(reader.getImage(frame), histOneFrame);
			for (int32 v=0; v<hist.getSize()[DIM_BAND]; v++) {
				hist(frame,v) = histOneFrame (v);
			} // for
		} // for
	}
	histSize=hist.getSize()[DIM_BAND];

	
	// initialize variables
	SignalSize vidSize (videoSize);
	Signal<uint8> frameAnalysed(vidSize);
	for (int32 i=0; i<videoSize; i++) {
		frameAnalysed(i)=0;
	} // for
	SignalSize size (histSize);
	Signal<int32> histogramValues (size);
	
	// set one scene for each shot
	//const int32 threshold=(int32)(0.10*numberOfPixel);
	
	// find first frame
	int32 i=0;
	while ((i<videoSize) && 
		   (cuts.cuts(i)!=SHOT_CHANGE_NO_CUT) && 
		   (cuts.cuts(i)!=SHOT_CHANGE_UNKNOWN)) i++;
	int32 lastHardCut=i;

	int32 sceneID=0;
	while (i<videoSize) {
		if ((cuts.cuts(i)!=SHOT_CHANGE_NO_CUT) && 
			(cuts.cuts(i)!=SHOT_CHANGE_UNKNOWN)){
			
			// initialize borders
			int32 center=(i+lastHardCut+1)/2;
			int32 left=lastHardCut;
			int32 right=i-1;
			
			// add scene
			for (int32 j=0; j<histSize; j++) {
				histogramValues(j) = hist(center,j);
			} // for
			SceneDetectionItem item(sceneID++, left, right, center, histogramValues);
			data.scenes.push_back(item);
			data.numberOfScenes++;
			
			while ((i<videoSize) && 
				   (cuts.cuts(i)!=SHOT_CHANGE_NO_CUT) && 
				   (cuts.cuts(i)!=SHOT_CHANGE_UNKNOWN)) i++;
			if (cuts.cuts(i-1)==SHOT_CHANGE_HARD_CUT) i--;
			lastHardCut=i;

		} // if
		i++;
	} // while
		
		
#if 0
	  //     (3) set borders for areas:
	  //         one border on an hard cut
	  //         two borders on fades/dissolves
	  //         one border on first/last frame
	  //         additional borders for longer shots (every 15 frames one additional border)
	  //cout << "(3)" << endl << endl << endl;
    const int32 borderSkip=25;
	int32 n = 3;
	int32 prevBorder=0;

	for (int32 i=0; i<cuts.numberOfFrames; i++) {
		while ((i<cuts.numberOfFrames) &&(cuts.cuts(i) == SHOT_CHANGE_UNKNOWN)) i++;
		while (i-prevBorder>borderSkip) {
			prevBorder+=borderSkip;
			n++;
		} // while
		prevBorder=i;
		n++;

		if ((i<cuts.numberOfFrames) && (cuts.cuts(i) != SHOT_CHANGE_HARD_CUT)) {
			while ((i<cuts.numberOfFrames) && (cuts.cuts(i) != SHOT_CHANGE_UNKNOWN)) i++;
			prevBorder=i;
			n++;
		} // if
	} // for

	// initialize border
	Signal<int32> border;
	SignalSize size(n);
	border.resize(size);	
	for (int32 i=0; i<size[DIM_X]; i++) {
		border(i)=-1;
	} // for

  
	n=0;
	prevBorder=0;
	border(n++)=prevBorder;
	for (int32 i=0; i<cuts.numberOfFrames; i++) {
		while ((i<cuts.numberOfFrames) &&(cuts.cuts(i) == SHOT_CHANGE_UNKNOWN)) i++;
		while (i-prevBorder>borderSkip) {
			prevBorder+=borderSkip;
			border(n++)=prevBorder;
		} // while
		prevBorder=i;
		border(n++)=prevBorder;

		if ((i<cuts.numberOfFrames) && (cuts.cuts(i) != SHOT_CHANGE_HARD_CUT)) {
			while ((i<cuts.numberOfFrames) && (cuts.cuts(i) != SHOT_CHANGE_UNKNOWN)) i++;
			prevBorder=i;
			border(n++)=prevBorder;
		} // if
	} // for
	//border(n-1)=cuts.numberOfFrames-1;

	
	//for (int32 i=0; i<border.getSize()[DIM_X]; i++) {
	//	cout << "border: " << border(i) << endl;
	//} // for
	


	//     (4) move borders:
	//         move border between 2 areas to the position, where both areas are most similar
	//
	// difference between 2 frames i and j:
	//   diff = SUM |i(x)-j(x)| for each histogram value x 
	// difference within one area:
	//   (1) if length(area) == 1: diff=0 else:
	//   (2)   avgHist = average histogram
	//   (3)   diff = MAX(avgHist-i) for each histogram i in area

	//cout << "(4)" << endl << endl << endl;
	Signal<uint8> borderChanged(size);
	for (int32 i=0; i<size[DIM_X]; i++)
		borderChanged(i)=0;
	borderChanged(0)=5;
	borderChanged(size[DIM_X]-1)=5;


	
	//cout << "FIRST BORDERS: "<< endl;
	//cout << "+++++++++++++ "<< endl;
	//for (int i=0; i<size[DIM_X]; i++)
	//	cout << i << "  " << border(i) << endl;
		
		
	int32 lastBorder=size[DIM_X];
	while ((lastBorder>1) && (border(lastBorder-1)==-1)) lastBorder--;
	borderChanged(lastBorder-1)=5;

	cerr << "move borders" << endl;
	for (int32 u=0; u<20; u++) {

		int32 totalMove=0;
		for (int32 i=0; i<lastBorder-2; i++) {
			if (borderChanged(i+1) < 3) {
				
				borderChanged(i+1)++;
				
				int32 minDist = 10000000;
				int32 minBorder = border(i+1);
				if ((border(i) <=border(i+1)) && (border(i+1) <=border(i+2))) {
					for (int32 j=border(i)+1; j<border(i+2)-2; j++) {
					    int32 dist = getHistDist (border(i), j, hist) + getHistDist (j+1, border(i+2)-1, hist);
					    //cout << "(" << border(i) << ".." << j << ")   (" <<  j+1 << ".." << border(i+2)-1 << ")   " << dist << endl;
					    if (dist<minDist) {
						minDist=dist;
						minBorder=j+1;
					    } // if
					} // for
					int32 deltaBorder = labs(border(i+1)-minBorder);
					totalMove+= deltaBorder;
					if (deltaBorder!=0) {
						borderChanged(i+1)=0;
					} // if
					border(i+1) =minBorder;
				} // if
			} // if
		} // for

		
		
		cerr << "\rborders changed: " << totalMove << "     " << ends;
		//cerr << endl;

		//cout << "BORDERS: "<< endl;
		//cout << "+++++++ "<< endl;
		//for (int i=0; i<lastBorder; i++)
		//	cout << border(i) << endl;
			
			

		// remove identical borders
		bool borderChanged=true;
		while (borderChanged) {
			borderChanged=false;
			for (int32 i=0; i<lastBorder-1; i++) {
				if ((border(i)==border(i+1)) && (border(i)!=-1)) {
					for (int32 j=i+1; j<lastBorder-1; j++)
						border(j)=border(j+1);
					border(lastBorder-1)=-1;
					borderChanged=true;
					totalMove+=10;
				} // if
			} // while
			while ((lastBorder>1) && (border(lastBorder-1)==-1)) lastBorder--;
		} // while


		
		//cout << "BORDERS: "<< endl;
		//cout << "+++++++ "<< endl;
		//for (int i=0; i<lastBorder; i++)
		//	cout << i << "   " << border(i) << endl;
		//cout << "total " << totalMove << endl;
		//cout << endl;
		
		
		if (totalMove==0) break;
	} // for
	cerr << endl;

	
	//cout << "FINAL BORDERS: "<< endl;
	//cout << "+++++++++++++ "<< endl;
	//for (int i=0; i<lastBorder; i++)
	//	cout << border(i) << endl;
	//cout << endl;
	



	//     (5) get average histogram for each area
	//     (6) store for each area: 
	//         areaID, 
	//         firstframe, lastframe, 
	//         area type: 0:UNKNOWN, 1:shot(!), 2:shotchange(1), 3:selected, 4:dark area, 5: white area
	//                    only (!) is available after first step
	//         average histogram

	//cout << "(4,5)" << endl << endl << endl;
	//cout << data << endl;
	//cout << "(5,6)" << endl << endl << endl;

	int32 histElements = hist.getSize()[DIM_BAND];
	SignalSize histElementsSize (histElements);
	Signal<int32> tmp (histElementsSize);
	int32 sceneID=0;
	for (int32 i=0; i<size[DIM_X]-1; i++) {
		if (border(i) < border(i+1)) {
			int32 position = (border(i) + border(i+1))/2;
			for (int32 j=0; j<histElements; j++) {
				tmp(j) = hist(position,j);
				//cout << " " << tmp(j);
			} // for
			//cout << endl;
			SceneDetectionItem item(sceneID++, border(i), border(i+1)-1, position, tmp);
			data.scenes.push_back(item);
			data.numberOfScenes++;
		} // if
	} // for
#endif


		return;
} // initializeScenes
#endif




#if 0
void 
SceneDetection::mergeScenes (SceneDetectionData& data, Signal<int32> &hist, int32 &histThreshold) 
{

	// (b) merge similar areas (based on histogram differences):
	//     for area i=0..N-1
	//       for area j=i+1..N
	//         if i is similar to j:
	//            set j(ID)=i(ID)


	int32 minThreshold=100000000;
	list<SceneDetectionItem>::iterator it1;
	list<SceneDetectionItem>::iterator it2;
	for (it1=data.scenes.begin(); it1!=data.scenes.end(); it1++) {
		it2=it1;
		it2++;
		while (it2!=data.scenes.end()) {
			
			if ((*it1).sceneID!=(*it2).sceneID) {
				
				SceneDetectionItem item1=(*it1);
				SceneDetectionItem item2=(*it2);
				
				// calculate distance between 2 areas (2 center frames)
				int32 dist = histDiff ((*it1).hist, (*it2).hist);
				if (minThreshold>dist) minThreshold=dist;
				
				// merge similar areas
				if (dist < histThreshold) {
					
					cout << "merge [" << item1.firstFrame << ".." << item1.lastFrame << "] " 
						<< "[" << item2.firstFrame << ".." << item2.lastFrame << "] " 
						<< dist << endl;
					
					(*it2).sceneID = (*it1).sceneID;
					item2=(*it2);
				} // if
			} // if
			it2++;
		} // while
	} // for
	histThreshold=minThreshold;


	//cout << "(output1)" << endl << endl << endl;
	//cout << data << endl;
	//cout << "(output2)" << endl << endl << endl;

#if 0
	// merge neighbour scenes with same ID
	bool sceneMerged=true;
	while (sceneMerged) {
		sceneMerged=false;
		for (it1=data.scenes.begin(); it1!=data.scenes.end(); it1++) {
			it2=it1;
			it2++;
			if (it2!=data.scenes.end()) {
				if (((*it1).sceneID==(*it2).sceneID) && 
					((*it1).lastFrame==(*it2).firstFrame-1)) {

					//cout << data << endl;
					SceneDetectionItem item1=(*it1);
					SceneDetectionItem item2=(*it2);

					// merge scenes
					(*it1).lastFrame=(*it2).lastFrame;
					(*it1).histPosition=((*it1).firstFrame+(*it1).lastFrame + 1)/2;
					for (int32 j=0; j<histSize; j++) {
						(*it1).hist(j) = hist((*it1).histPosition,j);
					} // for
					data.scenes.erase (it2);
					sceneMerged=true;
					item1=(*it1);

					//cout << data << endl;
					//break;
				} // if
			} // if
		} // for
	} // while


	// set new id's
	Signal<int32> tmpMappting (SignalSize(data.numberOfScenes,2));
	for (int32 i=0; i<data.numberOfScenes; i++) {
		tmpMappting(i,0) = -1;
		tmpMappting(i,1) = -1;
	} // for
	for (it1=data.scenes.begin(); it1!=data.scenes.end(); it1++) {
		tmpMappting((*it1).sceneID,0)=1;
	} // for
	int32 nextID=0;
	for (int32 i=0; i<data.numberOfScenes; i++) {
		if (tmpMappting(i,0) != -1) {
			tmpMappting(i,1) = nextID++;
		} // if
	} // for
	for (it1=data.scenes.begin(); it1!=data.scenes.end(); it1++) {
		(*it1).sceneID = tmpMappting((*it1).sceneID,1);
	} // for
	
	data.numberOfScenes=data.scenes.size();
	data.scenes.sort();
#endif


	return;
} // mergeScenes
#endif




// read histograms 
void 
SceneDetection::initializeHistograms (SceneDetectionData& data) 
{
	VideoCodec codec = CODEC_RGB;
	int msbits = SCENE_DETECTION_HIST_MSBITS_COLOR;
	if (grayscale) {
		grayscale=true;
		codec=CODEC_Y;
		msbits = SCENE_DETECTION_HIST_MSBITS_GRAY;
	} // if

	// inisialize histogram calculation
	bool smoothing=true;
	int areas=SCENE_DETECTION_HIST_AREAS;
	Signal<uint32> hist;
	HistogramFeature hf (msbits, grayscale, smoothing, areas);

	// inistialize MPEG decoder
	int cacheSize=3;
	int32 lastFrameToDecode=-1;
	VideoDecoder reader (pathFilename, cacheSize, lastFrameToDecode, codec);

	// for each scene item in data: read histogram
	list<SceneDetectionItem>::iterator it;
	Image<uint8> img;
	for (it=data.scenes.begin(); it!=data.scenes.end(); it++) {
		SceneDetectionItem item = (*it);
		int32 frame = (*it).referenceFrame;
		cerr << "\rframe: " << frame << "     " << ends;
		img = reader.getImage(frame);
		hf.process(img, hist);
		(*it).hist = hist;

		// print values
#if 0
		cout << "\r\n" << frame << "\r\n";
		for (int32 a=0; a<hist.getSize()[DIM_BAND]; a++) {
			for (int32 i=0; i<hist.getSize()[DIM_X]; i++) {
				cout << hist(i,a) << ",";
			} // for
		} // for
		cout << "\r\n" << "\r\n";
#endif
	} // for

	cerr << "\r                           \r" << ends;
	histSize=hist.getSize();

	return;
} // initializeHistograms



void 
SceneDetection::clusterScenes (SceneDetectionData& data) 
{
	// use k-means algorithms to cluster scenes
	list<SceneDetectionItem> clusters;
	clusters.clear();
	if (usePredefinedClusters) {
		initializePredefinedClusters(clusters);
	} // if
	int32 clusterID = 0;
	//int32 numberDefaultClusters = clusters.size();

#if 0
	{
		int32 n=0;
		cout << endl << endl;
		list<SceneDetectionItem>::iterator itClusterTmp;
		
		for (itClusterTmp=clusters.begin(); itClusterTmp!=clusters.end(); itClusterTmp++) {
			SceneDetectionItem item = (*itClusterTmp);
			cout << "cluster: " << n++ << "  ";
			for (int32 i=0; i<20; i++) {
				cout << setw(4) << item.hist(i,0) << " ";
			} // for
			cout << endl;
		} // for
		cout << endl << endl;
	}
#endif


	// set first cluster (average cluster)
	SceneDetectionItem item(clusterID, 0, 0, 0, false);

	// get histogram values for first cluster (average of all histogram values)
	Signal<uint32> clusterData (histSize);
	list<SceneDetectionItem>::iterator itCluster;
	list<SceneDetectionItem>::iterator itData;
	for (int32 a=0; a<histSize[DIM_BAND]; a++) {
		for (int32 i=0; i<histSize[DIM_X]; i++) {
			clusterData(i,a) = 0;

			for (itData=data.scenes.begin(); itData!=data.scenes.end(); itData++) {
				clusterData(i,a) += (*itData).hist(i,a);
			} // for
			clusterData(i,a) /= data.numberSceneItems;
		} // for
	} // for
	item.hist = clusterData;
	clusters.push_back(item);

#if 0
	{
		int32 n=0;
		cout << endl << endl;
		list<SceneDetectionItem>::iterator itClusterTmp;
		
		for (itClusterTmp=clusters.begin(); itClusterTmp!=clusters.end(); itClusterTmp++) {
			SceneDetectionItem item = (*itClusterTmp);
			cout << "cluster: " << n++ << "  ";
			for (int32 i=0; i<20; i++) {
				cout << setw(4) << item.hist(i,0) << " ";
			} // for
			cout << endl;
		} // for
		cout << endl << endl;
	}
#endif



	//int32 step=0;
	while (clusterID+1<numberOfClusters) {
	
#if 0
	{
		int32 n=0;
		cout << endl << endl;
		list<SceneDetectionItem>::iterator itClusterTmp;
		
		for (itClusterTmp=clusters.begin(); itClusterTmp!=clusters.end(); itClusterTmp++) {
			SceneDetectionItem item = (*itClusterTmp);
			cout << "cluster: " << n++ << "  ";
			for (int32 i=0; i<20; i++) {
				cout << setw(4) << item.hist(i,0) << " ";
			} // for
			cout << endl;
		} // for
		cout << endl << endl;
	}
#endif
		

		// (1) calculate distance to nearest cluster
		for (itData=data.scenes.begin(); itData!=data.scenes.end(); itData++) {
			
			// for each shot: set distance very high
			(*itData).clusterDistance = 10000000;
			for (itCluster=clusters.begin(); itCluster!=clusters.end(); itCluster++) {
				
				// for each shot: find minimal distance
				int32 diff = histDiff ((*itData).hist, (*itCluster).hist);
				if ((*itData).clusterDistance > diff) {
					(*itData).clusterDistance = diff;
					(*itData).sceneID = (*itCluster).sceneID;
				} // if
			} // for
		} // for
		
		
		// (2) remove shot with greatest distance, and create new cluster at this position
		list<SceneDetectionItem>::iterator itMaxDistance=data.scenes.begin();
		for (itData=data.scenes.begin(); itData!=data.scenes.end(); itData++) {
			if ((*itMaxDistance).clusterDistance < (*itData).clusterDistance) {
				itMaxDistance = itData;
			} // if
		} // for
		clusterID++;
		item.sceneID = clusterID;
		item.hist = (*itMaxDistance).hist;
		clusters.push_back(item);
		(*itMaxDistance).sceneID = clusterID;   // new cluster

		
		// (3) update cluster centers
		for (itCluster=clusters.begin(); itCluster!=clusters.end(); itCluster++) {
			SceneDetectionItem ttmp = (*itCluster);
			if (!((*itCluster).irrelevantFrame)) {
				for (int32 a=0; a<histSize[DIM_BAND]; a++) {
					for (int32 i=0; i<histSize[DIM_X]; i++) {
						clusterData(i,a) = 0;
					} // for
				} // for
				
				int32 n=0;
				for (itData=data.scenes.begin(); itData!=data.scenes.end(); itData++) {
					if ((*itData).sceneID == (*itCluster).sceneID) {
						n++;
						for (int32 a=0; a<histSize[DIM_BAND]; a++) {
							for (int32 i=0; i<histSize[DIM_X]; i++) {
								clusterData(i,a) += (*itData).hist(i,a);
							} // for
						} // for
					} // if
				} // for
				if (n>1) {
					for (int32 a=0; a<histSize[DIM_BAND]; a++) {
						for (int32 i=0; i<histSize[DIM_X]; i++) {
							clusterData(i,a) /= n;
						} // for
					} // for
				} // if
				
				(*itCluster).hist = clusterData;
			} // for
		} // if
	} // while
	data.numberOfScenesIDs = clusterID+1;

	//update: irrelevantFrame
	for (itData=data.scenes.begin(); itData!=data.scenes.end(); itData++) {
		if ((*itData).sceneID >= 0) {
			(*itData).irrelevantFrame = false;
		} else {
			(*itData).irrelevantFrame = true;
		} // else
	} // for

	return;
} // clusterScenes




void 
SceneDetection::readValues (SceneDetectionData& data) 
{
	// initialize values
	data.version = SCENE_DETECTION_VERSION;
	data.numberSceneItems = 0;
	data.numberOfScenesIDs = 0;
	data.usePredefinedClusters = usePredefinedClusters;
	data.scenes.clear();

	// get path of filename, do not create new directory
	CreateMocaPath mocaPath (pathFilename, false);
	data.filename = mocaPath.file;

	// read shots
	CutDetectionData cuts;
	{
		CutDetection reader (pathFilename);
		reader.process (cuts);

		Signal<int32> shotFrameNumber;
		reader.getShotFrameNumbers(cuts, shotFrameNumber);
		for (int32 i=0; i<shotFrameNumber.getSize()[DIM_X]; i++) {
			if (shotFrameNumber(i,0) != 0) {
				int32 referenceFrame = i;
				int32 firstFrame = shotFrameNumber(i,1);
				int32 lastFrame = shotFrameNumber(i,2);

				SceneDetectionItem item (data.numberOfScenesIDs++, firstFrame, referenceFrame, lastFrame, false);
				data.scenes.push_back(item);
			} // if
		} // if
	} 
	data.numberSceneItems = data.scenes.size();

#if 0
	// create one SceneDetectionItem for each shot
	int32 firstFrame=0;
	int32 lastFrame=0;
	for (int32 i=0; i<cuts.numberOfFrames; i++) {
		while ((i<cuts.numberOfFrames) && (cuts.cuts(i) != SHOT_CHANGE_UNKNOWN)) {
			i++;
		} // while
		firstFrame=i;
		if ((firstFrame>0) && (cuts.cuts(firstFrame-1) == SHOT_CHANGE_HARD_CUT)) firstFrame--;
		while ((i<cuts.numberOfFrames) && (cuts.cuts(i) == SHOT_CHANGE_UNKNOWN)) {
			i++;
		} // while
		lastFrame=i-1;
		if (firstFrame<lastFrame) {
			int32 referenceFrame = (lastFrame+firstFrame)/2;
			SceneDetectionItem item (data.numberOfScenesIDs++, firstFrame, referenceFrame, lastFrame, false);
			data.scenes.push_back(item);
		} // if
	} // for
	data.numberSceneItems = data.scenes.size();
#endif


	// read histograms
	initializeHistograms (data);


	// cluster scene items, if: data.numberOfScenesIDs > numberOfClusters
	if (data.numberOfScenesIDs > numberOfClusters) {
		clusterScenes(data);
	} // if



	// (a) calculate areas: the frames of one area are very similar
	//     (1) read shots
	//     (2) read histograms
	//     (3) set borders for areas:
	//         one border on an hard cut
	//         two borders on fades/dissolves
	//         one border on first/last frame
	//         additional borders for longer shots (every 10 frames one additional shot)
	//     (4) move borders:
	//         move border between 2 areas to the position, so that both areas are most similar
	//     (5) get average histogram for each area
	//     (6) store for each area: 
	//         areaID, 
	//         firstframe, lastframe, 
	//         area type: 0:UNKNOWN, 1:shot(!), 2:shotchange(1), 3:selected, 4:dark area, 5: white area
	//                    only (!) is available after first step
	//         average histogram
//	Signal<int32> hist;
//	CutDetectionData cuts;
//	initializeScenes (data, cuts, hist);
	//cout << data << endl << endl;
	//string directory="D:/videos/scene/test_scene_";
	//saveScenes(data, directory);




	// (b) merge similar areas (based on histogram differences):
	//     for area i=0..N-1
	//       for area j=i+1..N
	//         if i is similar to j:
	//            set j(ID)=i(ID)

	// difference between 2 frames i and j:
	//   diff = SUM |i(x)-j(x)| for each histogram value x 

	// difference within one area:
	//   (1) if length(area) == 1: diff=0 else:
	//   (2)   avgHist = average histogram
	//   (3)   diff = MAX(avgHist-i) for each histogram i in area

	// difference between two areas:
	//   store average histogram for each area and compare 2 histograms

	
	
#if 0
	cerr << "merge scenes" << endl;
	int32 histThreshold=(int32)(0.02*numberOfPixel);
	mergeScenes (data, hist, histThreshold);
	cout << data << endl << endl;


	// count number of neighbour scenes with same id (2 different shots have the same id)
	int32 errorCounter=0;
	list<SceneDetectionItem>::iterator it1;
	list<SceneDetectionItem>::iterator it2;
	for (it1=data.scenes.begin(); it1!=data.scenes.end(); it1++) {
		it2=it1;
		it2++;
		if (it2!=data.scenes.end()) {
			SceneDetectionItem item1=(*it1);
			SceneDetectionItem item2=(*it2);
			if ((*it1).sceneID == (*it2).sceneID) errorCounter++;
		} // if
	} // for
	


	// store valid data
	SceneDetectionData validData=data;

	// optimize (find best threshold)
	int32 histStepSize = (int32) (0.4*numberOfPixel);
	int32 minHistStepSize = (int32) (0.005*numberOfPixel);

	bool valid=true;
	while (valid) {
		histThreshold+=histStepSize;

		// analyse scenes
		int32 tmpThreshold = histThreshold;
//		mergeScenes (data, hist, tmpThreshold);
		cout << data << endl << endl;

		// data is not valid, if one scene combines 2 different shots
		int32 tmpErrorCounter=0;
		for (it1=data.scenes.begin(); it1!=data.scenes.end(); it1++) {
			it2=it1;
			it2++;
			if (it2!=data.scenes.end()) {
				SceneDetectionItem item1=(*it1);
				SceneDetectionItem item2=(*it2);
				if ((*it1).sceneID == (*it2).sceneID) tmpErrorCounter++;
			} // if
		} // for
		if (tmpErrorCounter==errorCounter) {
			validData=data;
		} else {
			histThreshold-=histStepSize;
			data=validData;
			histStepSize/=3;
			if (histStepSize<minHistStepSize) {
				valid=false;
			} // if
		} // else
	} // while

	// store data
	cout << data << endl << endl;
	string directory="D:/videos/scene/test_scene_";
	//saveScenes(data, directory);



	cout << "BORDERS - 2" << endl;
	cerr << "BORDERS - 2" << endl;
	cout << data << endl << endl;

	cout << "BORDERS - 3" << endl;


#endif
	return;	
} // readValues






void 
SceneDetection::initializePredefinedClusters (list<SceneDetectionItem> &clusters) 
{
	// number of pixels (for scaling factor), histogram values

	int ID=-1;
	if (grayscale) {
		Signal<uint32> hist;
		SignalSize size(SCENE_DETECTION_HIST_BINS_GRAY, SCENE_DETECTION_HIST_AREAS);
		hist.resize(size);

		for (int32 i=0; i<NUMBER_GRAY_CLUSTER; i++) {
			SceneDetectionItem item(ID--, 0, 0, 0, true);
			int32 n=0;
			ieee64 pixelInHist = 0;
			for (int32 a=0; a<size[DIM_BAND]; a++) {
				for (int32 x=0; x<size[DIM_X]; x++) {
					pixelInHist += GRAY_DEFAULT_HIST[i][n++];
				} // for
			} // for
			if (pixelInHist < 1.0) pixelInHist = 1.0;
			ieee64 pixelFactor = numberOfPixel / pixelInHist;

			n=0;
			for (int32 a=0; a<size[DIM_BAND]; a++) {
				for (int32 x=0; x<size[DIM_X]; x++) {
//uint32 tmp=(uint32) (pixelFactor*GRAY_DEFAULT_HIST[i][n]);
					hist(x,a) = (uint32) (pixelFactor*GRAY_DEFAULT_HIST[i][n++]);
				} // for
			} // for

			item.hist=hist;
			clusters.push_back(item);
		} // for
	} // if


	if (!grayscale) {
		Signal<uint32> hist;
		SignalSize size(SCENE_DETECTION_HIST_BINS_COLOR, SCENE_DETECTION_HIST_AREAS);
		hist.resize(size);

		for (int32 i=0; i<NUMBER_COLOR_CLUSTER; i++) {
			SceneDetectionItem item(ID--, 0, 0, 0, true);
			int32 n=0;
			ieee64 pixelInHist = 0;
			for (int32 a=0; a<size[DIM_BAND]; a++) {
				for (int32 x=0; x<size[DIM_X]; x++) {
					pixelInHist += COLOR_DEFAULT_HIST[i][n++];
				} // for
			} // for
			if (pixelInHist < 1.0) pixelInHist = 1.0;
			ieee64 pixelFactor = numberOfPixel / pixelInHist;

			n=0;
			for (int32 a=0; a<size[DIM_BAND]; a++) {
				for (int32 x=0; x<size[DIM_X]; x++) {
					hist(x,a) = (uint32) (pixelFactor*COLOR_DEFAULT_HIST[i][n++]);
				} // for
			} // for

			item.hist=hist;
			clusters.push_back(item);
		} // for
	} // if

	return;
} // initializePredefinedClusters 



