/*
   This file is part of the MoCA video library.
   Name: VideoStatistic.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

 
  
   \author    Stephan Kopf
   
	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de
	
	 (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt
	 
*/


#ifndef VIDEO_STATISTIC_H
#define VIDEO_STATISTIC_H

// MoCA includes
#include <MocaTypes.h>
#include <Image.h>
#include <VideoDecoder.h>
#include <ITagStream.h>
#include <ImageStatistic.h>
#include <MocaSignal.h>

// C++ includes
#include <iostream>
using namespace std;


const static string VIDEO_STATISTIC_VERSION  ="Version-0.61";
const static string VIDEO_STATISTIC_FILENAME ="statistics.idx";

/*
	Constructor: initializes variables
	
	process(): calculate statistically values from mpeg-file, save "statistic"-file
	read()   : read values from "statistic"-file
  */

struct VideoStatisticData
{
	VideoStatisticData() {
		version         = VIDEO_STATISTIC_VERSION;
		filename        = "";
		frames          = 0;
		numberOfColorFrames    = 0;
		numberOfNearGrayFrames = 0;
		numberOfGrayFrames     = 0;
		color = TYPE_UNKNOWN;
	} 

	string version;
	string filename;
	int32 frames;
	vector<ImageStatisticData> imgData;
	ImageStatisticData avgVideoData;
	ImageStatisticData minVideoData;
	ImageStatisticData maxVideoData;

	int32 numberOfColorFrames;
	int32 numberOfNearGrayFrames;
	int32 numberOfGrayFrames;
	GrayscaleType color;

};

std::ostream& operator<< (ostream&,    const VideoStatisticData&);
OTagStream&   operator<< (OTagStream&, const VideoStatisticData&);
ITagStream&   operator>> (ITagStream&, VideoStatisticData&);



class VideoStatistic
{
public:

	VideoStatistic(const string &pathFilename)
	{
		this->pathFilename = pathFilename;
	}
	~VideoStatistic() {}

	// analyse full video
	void process (VideoStatisticData &data);

	// do not calculate values, but use already calculated data of frames,
	// and store them in data
	void process (list<ImageStatisticData> &imgStatData, VideoStatisticData &data);

private:
	void readValues (VideoStatisticData &data);
	void readValues (list<ImageStatisticData> &imgStatData, VideoStatisticData &data);

	string pathFilename;

private:

};


#endif
