/*
    FillFilter.cpp     fill region in an image
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



#include<FillFilter.h>

void
FillFilter::
process (Image<uint8> &img, ImagePosition firstIp, uint8 newColor) 
{
	uint8 imageColor1 = img(firstIp);
	switch (neighbour) {
	case N4: 
		if (secondColor) {
			fill4 (img, firstIp, imageColor1, imageColor2, newColor);
		} else {
			fill4 (img, firstIp, imageColor1, imageColor1, newColor);
		}
		break;
	case N8: 
		if (secondColor) {
			fill8 (img, firstIp, imageColor1, imageColor2, newColor);
		} else {
			fill8 (img, firstIp, imageColor1, imageColor1, newColor);
		}
		break;
	case Nd: 
		if (secondColor) {
			fillD (img, firstIp, imageColor1, imageColor2, newColor);
		} else {
			fillD (img, firstIp, imageColor1, imageColor1, newColor);
		}
		break;
	case Nm: 
		if (secondColor) {
			fill8 (img, firstIp, imageColor1, imageColor2, newColor);
		} else {
			fill8 (img, firstIp, imageColor1, imageColor1, newColor);
		}
		break;
	default:
		cerr << "fill-filter (default) not implemented" << endl;
		exit(1);
		break;
	} // switch

	return;
} // process


void
FillFilter::
fill8 (Image<uint8> &img, ImagePosition firstIp, uint8 imageColor1, uint8 imageColor2, uint8 newColor)
{
	stack<ImagePosition> ipl;
	ipl.push (firstIp);
	ImagePosition p;
	while (!ipl.empty()) {
		p = ipl.top();
		ipl.pop();
		img(p) = newColor;

		p[DIM_X]++;
		if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
			ipl.push (p);
		} // if
		p[DIM_Y]++;
		if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
			ipl.push (p);
		} // if
		p[DIM_X]--;
		if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
			ipl.push (p);
		} // if
		p[DIM_X]--;
		if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
			ipl.push (p);
		} // if
		p[DIM_Y]--;
		if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
			ipl.push (p);
		} // if
		p[DIM_Y]--;
		if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
			ipl.push (p);
		} // if
		p[DIM_X]++;
		if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
			ipl.push (p);
		} // if
		p[DIM_X]++;
		if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
			ipl.push (p);
		} // if
		p[DIM_X]--;
		p[DIM_Y]++;

		// enlargeHorizontal
		int i;
		if (enlargeHorizontal>0) {
			p[DIM_X]++;
			for (i=0; i<enlargeHorizontal; i++) {
				p[DIM_X]++;
				if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
					ipl.push (p);
				} // if
			} // for
			p[DIM_X]-=(enlargeHorizontal+1);

			p[DIM_X]--;
			for (i=0; i<enlargeHorizontal; i++) {
				p[DIM_X]--;
				if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
					ipl.push (p);
				} // if
			} // for
			p[DIM_X]+=(enlargeHorizontal+1);
		} // if

		// enlargeVertical
		if (enlargeVertical>0) {
			p[DIM_Y]++;
			for (i=0; i<enlargeVertical; i++) {
				p[DIM_Y]++;
				if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
					ipl.push (p);
				} // if
			} // for
			p[DIM_Y]-=(enlargeVertical+1);

			p[DIM_Y]--;
			for (i=0; i<enlargeVertical; i++) {
				p[DIM_Y]--;
				if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
					ipl.push (p);
				} // if
			} // for
			p[DIM_Y]+=(enlargeVertical+1);
		} // if
	} // while

	return;
} // fill8

void
FillFilter::
fill4 (Image<uint8> &img, ImagePosition firstIp, uint8 imageColor1, uint8 imageColor2, uint8 newColor)
{
	if (imageColor1 == newColor) { 
		cerr << "error in FillFeature::fill4 (imageColor1 equals newColor)" << endl;
		exit(1);
	} // if
	if (imageColor2 == newColor) { 
		cerr << "error in FillFeature::fill4 (imageColor2 equals newColor)" << endl;
		exit(1);
	} // if

	/*
	cout << "---------" << endl << (int)imageColor1 << endl;
	cout << (int)imageColor2 << endl;
	cout << (int)newColor << endl;
	cout << endl;
*/

	//long int ccc = 0;
	stack<ImagePosition> ipl;
	ipl.push (firstIp);
	ImagePosition p;
	while (!ipl.empty()) {
		p = ipl.top();
		ipl.pop();
		img(p) = newColor;


		p[DIM_X]++;
		if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
			ipl.push (p);
		} // if
		p[DIM_X]-=2;
		if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
			ipl.push (p);
		} // if
		p[DIM_X]++;
		p[DIM_Y]++;
		if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
			ipl.push (p);
		} // if
		p[DIM_Y]-=2;
		if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
			ipl.push (p);
		} // if
		p[DIM_Y]++;

		// enlargeHorizontal
		int i;
		if (enlargeHorizontal>0) {
			p[DIM_X]++;
			for (i=0; i<enlargeHorizontal; i++) {
				p[DIM_X]++;
				if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
					ipl.push (p);
				} // if
			} // for
			p[DIM_X]-=(enlargeHorizontal+1);

			p[DIM_X]--;
			for (i=0; i<enlargeHorizontal; i++) {
				p[DIM_X]--;
				if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
					ipl.push (p);
				} // if
			} // for
			p[DIM_X]+=(enlargeHorizontal+1);
		} // if

		// enlargeVertical
		if (enlargeVertical>0) {
			p[DIM_Y]++;
			for (i=0; i<enlargeVertical; i++) {
				p[DIM_Y]++;
				if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
					ipl.push (p);
				} // if
			} // for
			p[DIM_Y]-=(enlargeVertical+1);

			p[DIM_Y]--;
			for (i=0; i<enlargeVertical; i++) {
				p[DIM_Y]--;
				if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
					ipl.push (p);
				} // if
			} // for
			p[DIM_Y]+=(enlargeVertical+1);
		} // if
	} // while
	return;
} // fill4


void
FillFilter::
fillD (Image<uint8> &img, ImagePosition firstIp, uint8 imageColor1, uint8 imageColor2, uint8 newColor)
{
	stack<ImagePosition> ipl;
	ipl.push (firstIp);
	ImagePosition p;
	while (!ipl.empty()) {
		p = ipl.top();
		ipl.pop();
		img(p) = newColor;

		p[DIM_X]++;
		p[DIM_Y]++;
		if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
			ipl.push (p);
		} // if
		p[DIM_X]-=2;
		if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
			ipl.push (p);
		} // if
		p[DIM_Y]-=2;
		if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
			ipl.push (p);
		} // if
		p[DIM_X]+=2;
		if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
			ipl.push (p);
		} // if
		p[DIM_X]--;
		p[DIM_Y]++;

		// enlargeHorizontal
		int i;
		if (enlargeHorizontal>0) {
			p[DIM_X]++;
			for (i=0; i<enlargeHorizontal; i++) {
				p[DIM_X]++;
				if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
					ipl.push (p);
				} // if
			} // for
			p[DIM_X]-=(enlargeHorizontal+1);

			p[DIM_X]--;
			for (i=0; i<enlargeHorizontal; i++) {
				p[DIM_X]--;
				if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
					ipl.push (p);
				} // if
			} // for
			p[DIM_X]+=(enlargeHorizontal+1);
		} // if

		// enlargeVertical
		if (enlargeVertical>0) {
			p[DIM_Y]++;
			for (i=0; i<enlargeVertical; i++) {
				p[DIM_Y]++;
				if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
					ipl.push (p);
				} // if
			} // for
			p[DIM_Y]-=(enlargeVertical+1);

			p[DIM_Y]--;
			for (i=0; i<enlargeVertical; i++) {
				p[DIM_Y]--;
				if ( (img(p) == imageColor1) || (img(p) == imageColor2) ){
					ipl.push (p);
				} // if
			} // for
			p[DIM_Y]+=(enlargeVertical+1);
		} // if
	} // while

	return;
} // fillD

