/*
   This file is part of the MoCA video library.
   Name: FlipFilter.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

   FlipFilter.h  -- implementation of the flip filter

   Author:    Stephan Kopf
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: kopf@pi4.informatik.uni-mannheim.de

*/



#ifndef FLIP_FILTER_H
#define FLIP_FILTER_H

#include <ImageFilterBase.h>
#include <Image.h>

/*
  Flip an image horizontally or vertically about its centerline.
*/

enum {FLIP_UNKNOWN=0, FLIP_HORIZONTAL=1, FLIP_VERTICAL=2};

template<class T>
class FlipFilter: public ImageFilterBase<T>
{
public:
    FlipFilter (int flip)
    {
        this->flip = flip;
        if ((flip != FLIP_HORIZONTAL) && (flip != FLIP_VERTICAL)) {
            this->flip = FLIP_UNKNOWN; 
        } // if
        if (this->flip == FLIP_UNKNOWN) {
            cout << "FlipFilter (int)  unknown flip-filter mode" << endl;
            exit(1);
        } // if
    } //  constructor

    //! Apply flip filter to src, write out into dest
    void process (const Image<T>& src, Image<T>& dest);

    //! Apply flip filter to src, write into same image
    void process (Image<T>& srcDest)
	{
		Image<T> tmp=srcDest;
		process (tmp, srcDest);
		return;
	}

private:
    int flip;
}; // FlipFilter

#include <FlipFilter.cpt>

#endif

