/*
   This file is part of the MoCA video library.
   Name: TransposeFilter.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


/* -*- C++ -*-

   TransposeFilter.h  -- implementation of the flip filter

   Author:    Stephan Kopf
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: kopf@pi4.informatik.uni-mannheim.de

*/



#ifndef TRANSPOSE_FILTER_H
#define TRANSPOSE_FILTER_H

#include <ImageFilterBase.h>
#include <Image.h>

/*
  Transpose an image such that row i, column j in the source 
  image becomes row j, column i in the destination image.
*/

template<class T>
class TransposeFilter : public ImageFilterBase<T>
{
public:
    TransposeFilter (){}

    //! Apply transpose filter to src, write out into dest
    void process (const Image<T>& src, Image<T>& dest);

    //! Apply transpose filter to src, write into same image
    void process (Image<T>& srcDest){
		Image<T> tmp=srcDest;
		process(tmp,srcDest);
		return;
	}

private:
}; // TransposeFilter

#include <TransposeFilter.cpt>

#endif

