/*
   This file is part of the MoCA video library.
   Name: DrawIcon.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

   DrawIcon.h  -- draws different icons, new icons can be added (e.g. characters)

   Author:    Stephan Kopf
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: kopf@pi4.informatik.uni-mannheim.de

   $Id: 
*/



#ifndef DRAW_ICON_H
#define DRAW_ICON_H

#include <ImagePosition.h>
#include <Image.h>
#include <Color.h>
#include <IImageStream.h>


enum IconType {ICON_UNKNOWN = 0, ICON_CAR=1, ICON_PEOPLE=2, ICON_BOAT=3};

class DrawIcon
{
public:
    DrawIcon (const Color &col = Color::WHITE) 
	{
	    c = col;
	}


    //! draw icon to image
    void process (const ImagePosition &p, IconType icon, Image<uint8>& srcDest);
    void process (const ImagePosition &p, IconType icon, const Image<uint8>& src, Image<uint8>& dest)
    {
	dest = src;
	process (p,icon,dest);
	return;
    }

    // read an image (filename, e.g. png-file) and print the pixels of the icon 
    // the color of the icon is black, the backkground is white
    // the program terminates with this function
    // use cut&paste to add the lines as new icon in DrawIcon.cpp
    void createIcon (const string &filename);


private:
	Color c;

}; // DrawIcon


#endif

