/*
   This file is part of the MoCA video library.
   Name: DrawRectangle.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: {kuehne,kopf}@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

   DrawRectangle.h  -- implementation of a rectangle routine

   Author:    Stephan Kopf
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: {kuehne,kopf}@pi4.informatik.uni-mannheim.de

*/

#ifndef DRAW_RECTANGLE_H
#define DRAW_RECTANGLE_H


// MoCA includes
#include <ImagePosition.h>
#include <Image.h>
#include <Color.h>
#include <DrawLine.h>

class DrawRectangle 
{
public:
    DrawRectangle(const Color &c = Color::WHITE, int32 width = 1) 
	{
		this->c    =c;
		this->width=width;
	};
    ~DrawRectangle () {};

    //! draw rectangle
	//! ImagePosition p1 : upper left corner
	//! ImagePosition p2 : lower right corner
    void process (ImagePosition p1, ImagePosition p2, Image<uint8>& srcDest);
    void process (ImagePosition p1, ImagePosition p2, const Image<uint8>& src, Image<uint8>& dest);

private:
	void draw (int32 x0,int32 y0,int32 x1,int32 y1, Image<uint8>& srcDest);

	Color c;
	int32 width;
}; // DrawRectangle

#endif

