/*
   This file is part of the MoCA video library.
   Name: IImageStream.h
   Copyright (C) 2004 Oliver Schuster

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: schuster@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file


   \author    Oliver Schuster

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: schuster@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: IImageStream.h,v 1.1 2004/02/17 12:00:48 kopf Exp $

*/

#ifndef I_IMAGE_STREAM_H
#define I_IMAGE_STREAM_H

// C++ includes
#include <string>
#include <fstream>

// MoCA includes
#include <MocaTypes.h>
#include <MocaExceptions.h>
#include <Image.h>
#include <IImageFormatStream.h>

// MoCA includes, format specific stream types
#include <IPgmFormatStream.h>
#include <IPpmFormatStream.h>
#include <IPngFormatStream.h>
#include <IJpegFormatStream.h>

/*!

  Input stream for image files

 */
template<class T>
class IImageStream
{
public:
    //! the default constructor
    IImageStream() : is(NULL), ifs(NULL) {};

    //! construct stream based on url provided
    IImageStream(const string &);

    //! open an image stream
    void open(const string &) throw (MocaException);

    //! the destructor
    ~IImageStream();

    //! close an image stream
    void close();

    //! read image from stream
    void read(Image<T> &) throw (MocaException);

    //! read image from stream
    IImageStream &operator>>(Image<T> &);

private:
    //! raw stream
    istream *is;

    //! format specific stream based on is
    IImageFormatStream<T> *ifs;

    //! build "ifs" based on "is"
    void getFormatStream();

}; // class IImageStream


class ReadImage
{
public:
	ReadImage (const string &filename, Image<uint8> &img)
	{
		process (filename, img);
	}

	ReadImage (const string &filename, Image<int32> &img)
	{
		Image<uint8> tmp;
		process (filename, tmp);
		ImageSize size = tmp.getSize();
		img.resize(size);

		// copy image
		for (int32 b=0;b<size[DIM_BAND]; b++) {
			for (int32 y=0;y<size[DIM_Y]; y++) {
				for (int32 x=0;x<size[DIM_X]; x++) {
					img(x,y,b) = tmp(x,y,b);	
				} // for
			} // for
		} // for
	}

private:
	void process (const string &filename, Image<uint8> &img)
	{
		try {
			// open output filestream and write image
			IImageStream<uint8> iis(filename);
			iis.read(img);
		} catch (MocaException e) {
			cout << "ReadImage: " << e.what() << endl;
			exit (-1);
		} // try
	} // process

};


#include <IImageStream.cpt>

#endif
