/*
   This file is part of the MoCA video library.
   Name: IWaveAudio.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

 IWaveAudio.h -- Interface for WAV audio files
 
  Author: Stephan Kopf
  Lehrstuhl fuer Praktische Informatik IV
  Universitaet Mannheim, Germany
  email: kopf@pi4.informatik.uni-mannheim.de
  

*/

#ifndef I_WAVE_AUDIO_H
#define I_WAVE_AUDIO_H

// C includes
#include <stdio.h>

// C++ includes
#include <fstream>
#include <iostream>
using namespace std;

// MoCA includes
#include <MocaTypes.h>
#include <WaveHeader.h>
#include <MocaSignal.h>
#include <OpenFile.h>




class IWaveAudio 
{
public:

	// constuctor setting filename of audio wav file 
	IWaveAudio (string &pathFilename);

	// destructor
	~IWaveAudio () {
		close ();
	}

	// get file information
	WaveHeader& getHeader () {
		return header;
	} // getFormat

	// get size of raw pcm data
	SignalSize getSize () {
		return SignalSize (header.getDataLength() / header.getBlockAlign(), header.getChannels());
	} // getSize

	int32 getOffset() {
		return (bytesRead / header.getBlockAlign());
	} // getOffset

	// store MoCA signal
	// if number of samples == -1: read all samples
	// if offsetSamples == -1: continue reading from last offset 
	// continues reading from last offset, call reset() to start from the beginning of the stream
	void read (Signal<int16> &sig, int32 samples = -1, int32 offsetSamples = -1);

	// gets file handler to raw data (fh does not contain header information, only raw pcm data)
	// if offsetSamples == -1: start reading with first pcm sample, else start reading with offset
	FILE* getFileHandle (int32 offsetSamples = -1);

	// resets input stream
	void reset () {
		close ();
		open ();
	} // reset

private:
	void open ();
	void close ();

	string pathFilename;
	WaveHeader header;
	FILE *fh;
	int32 bytesRead;
};


#endif
