/*
   This file is part of the MoCA video library.
   Name: MpegStatus.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

 
  
   \author    Stephan Kopf
   
	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de
	
	 (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt
	 
*/


#ifndef MPEG_STATUS_H
#define MPEG_STATUS_H

// C++ includes
#include <iostream>
using namespace std;

// MoCA includes
#include <VideoDecoder.h>
#include <CreateMocaPath.h>

const static string MPEG_STATUS_VERSION  ="Version-1.0";
const static string MPEG_STATUS_FILENAME ="status.idx";


struct MpegData
{
	MpegData ()
	{
		version = "";
		filename = "";
		size[DIM_X] = 0;
		size[DIM_Y] = 0;
		size[DIM_Z] = 0;
		size[DIM_BAND] = 0;
	}
    MpegData& operator=(const MpegData &other) {
		if (this != &other) {
			version  = other.version;
			size     = other.size;
			filename = other.filename;
		} // if
        return *this;
    } // operator=

	string version;
	VideoSize size;
	string filename;
};

std::ostream& operator<< (ostream&,    const MpegData&);
OTagStream&   operator<< (OTagStream&, const MpegData&);
ITagStream&   operator>> (ITagStream&, MpegData&);



/*
	Constructor: initializes variables (filename of mpeg file)
	
	process(): calculate values from mpeg-file, save status file
*/
class MpegStatus
{

public:
	MpegStatus(const string &pathFilename)
	{
		this->pathFilename = pathFilename;
	} // MpegStatus
	~MpegStatus() {}

	void process (MpegData&);

private:
	void readValues(MpegData &data);

	string pathFilename;
};


#endif
