/*
   This file is part of the MoCA video library.
   Name: OVideoFormatStream.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Stephan Kopf

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kopf@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: OVideoFormatStream.h,v 1.1 2004/02/17 12:00:48 kopf Exp $

*/

#ifndef O_VIDEO_FORMAT_STREAM_H
#define O_VIDEO_FORMAT_STREAM_H

// Moca includes
#include <Image.h>
#include <FormatParameter.h>

/*!
  Abstract base class for output video format streams
*/
template<typename T>
class OVideoFormatStream
{
public:
    //! the default constructor
    OVideoFormatStream(ostream &os);

    //! the destructor
    virtual ~OVideoFormatStream();

    //! to open a stream
    void open(ostream &);

    //! to open a stream
    virtual void open(ostream &, const FormatParameter *) = 0;

    //! close the image stream, not is!
    void close();

    //! read an image
    virtual void write(const Image<T> &) = 0;

    //! read an image (calls read)
    OVideoFormatStream &operator<<(const Image<T> &);

protected:
    //! the underlying output stream
    ostream *os;

}; // OVideoFormatStream

#include <OVideoFormatStream.cpt>

#endif
