/*
   This file is part of the MoCA video library.
   Name: OYuvVideo.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

  OYuvVideo.h -- write YUV videos 
 
  Author: Stephan Kopf
  Lehrstuhl fuer Praktische Informatik IV
  Universitaet Mannheim, Germany
  email: kopf@pi4.informatik.uni-mannheim.de
  
*/

#ifndef O_YUV_VIDEO_H
#define O_YUV_VIDEO_H


// C includes
#include<stdio.h>

// C++ includes
#include <sstream>
#include <string>
#include <iostream>
using namespace std;

// MoCA includes
#include <MocaTypes.h>
#include <Image.h>
#include <Video.h>
#include <ConvertColorSpace.h>
#include <OpenFile.h>


class OYuvVideo 
{
	
public:
	
	// constuctor setting filename
	OYuvVideo (const string &pathFilename, VideoCodec srcCodec=CODEC_RGB, VideoCodec destCodec=CODEC_YUV444);
	
	// destructor
	~OYuvVideo ()
	{
		close();
	}
	
	// write frame
	void write(const Image<uint8> &src);

	// write frame
	void write(const Video<uint8> &src);
	
private:

	void close()
	{
		if (fh_out!=NULL) {
			fclose(fh_out);
		} // if
		fh_out = NULL;
		return;
	}
	void open()
	{
		close();

		fh_out=NULL;
		OpenFile::process(&fh_out, pathFilename, F_WRITE);
		return;
	}

	
	FILE *fh_out;
	string pathFilename;
	VideoCodec srcCodec;
	VideoCodec destCodec;
};



// TODO: create class similar to WriteImage (the file extension should be checked automatically!)
class WriteYVideo
{
public:
	WriteYVideo (const Video<uint8> &vid, const string &filename)
	{
		try {
			// open output filestream and write image
			VideoCodec srcCodec=CODEC_Y;
			VideoCodec destCodec=CODEC_Y;
			OYuvVideo writer (filename, srcCodec, destCodec);
			writer.write(vid);
		} catch (MocaException e) {
			cout << "WriteYVideo: " << e.what() << endl;
			exit (-1);
		} // try
	}
};



#endif
