/*
   This file is part of the MoCA video library.
   Name: PpmVideoDecoder.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

  PpmVideoDecoder.h -- decodes PPM video sequences
                     
 
  Author: Stephan Kopf
  Lehrstuhl fuer Praktische Informatik IV
  Universitaet Mannheim, Germany
  email: kopf@pi4.informatik.uni-mannheim.de
  

*/

#ifndef PPM_VIDEO_DECODER_H
#define PPM_VIDEO_DECODER_H


// MoCA includes
#include <VideoDecoderBase.h>


// this class provides the basic routines to decode a video
class PpmVideoDecoder : VideoDecoderBase
{
public:

	// constuctor defines filename 
	PpmVideoDecoder (const string &pathFile);

	// destructor
	~PpmVideoDecoder ();

	// get image data
	void getImage (int32 frame, Image<uint8> &img);
	void getVideo (Video<uint8> &vid);
	void getVideo (ImageVector<uint8> &vid);

	// get size
	VideoSize getSize()
	{
		return VideoDecoderBase::getSize();
	}
	ImageSize getImageSize()
	{
		return VideoDecoderBase::getImageSize();
	}

private:

	bool getHeader (bool &isBinary, int &width, int &height);
	bool getImageData (Image<uint8> &img);
	void init ();


	// variables to decode the images
	vector<std::ios::pos_type> filePos;

	bool binary;
	int32 bufSize;
    char* buf;

};


#endif
