/* -*- C++ -*-

   ProcessReader.h  -- open a process, read all data and store it in memory 

   Author:    Stephan Kopf
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: kopf@pi4.informatik.uni-mannheim.de


    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <ProcessReader.h>


void 
ProcessReader::init()
{
#ifdef COMPILER_MSVC
	//  Switch Win32 standard input to binary mode, so that piping the
	//  raw image data works. 
	int result = _setmode( _fileno(stdin), _O_BINARY );
	if (result == -1) {
		cerr << "Cannot set standard input to binary mode.\n";
		exit(1);
	}
#endif


	// read data to temporary buffer (size is unknown)
	vector<ProcessReaderBuffer> tmpBuf;
	{
		// open process
		FILE* fh_in = NULL;
		OpenProcess::open(&fh_in, cmd, P_READ, true);

		while (!feof(fh_in)) {
			
			// read data
			ProcessReaderBuffer b;
			b.size = fread (b.buf, 1, 1024, fh_in);
			
			// store data
			tmpBuf.push_back(b);
		} // while
		OpenProcess::close(&fh_in);
	}

	// get total size
	size = 0;
	for (int32 i=0; i<tmpBuf.size(); i++) {
		size += tmpBuf[i].size;
	} // for


	// allocate memory
	close();
	buf = new char [size];
	char *bufPtr = buf;
	for (int32 i=0; i<tmpBuf.size(); i++) {
		int32 cnt = tmpBuf[i].size;
		char *tmpPtr = tmpBuf[i].buf;
		for (int32 j=0; j<cnt; j++) {
			*bufPtr = *tmpPtr;
			bufPtr++;
			tmpPtr++;
		} // for
	} // for

	return;
} // show

void 
ProcessReader::convertToText (vector<string> &textLines)
{
	textLines.clear();
	bool error = false;
	int count = 0;

	string s = "";
	for (int32 i=0; i<size; i++) {
		if (buf[i]==10 || buf[i]==13) {
			if (s.length()>0) textLines.push_back(s);
			s = "";
		} else {
			s += buf[i];
		} // else
	} // for

	return;
}

