/*
   This file is part of the MoCA video library.
   Name: ReadMocaData.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Stephan Kopf

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kopf@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt


*/

#ifndef READ_MOCA_DATA_H
#define READ_MOCA_DATA_H

// C++ includes
#include <string>
#include <fstream>
#include <iostream>
using namespace std;

// MoCA includes
#include <MocaTypes.h>
#include <MocaExceptions.h>
#include <ITagStream.h>
#include <CreateMocaPath.h>


// Constructor sets path and filename of video (or image) and the filename of the index file (indexFilename).
// Example:
//		ReadMocaData ("/opt/Test.mpg", "status.idx");
//
// The function bool process(string &version) 
//		(1) call CreateMocaPath and calculates the path of the index file (pathIndexFile).
//		(2) creates the directory pathIndexFile (if it does not exist).
//		(3) does the index file exist? (indexFilename). 
//		(4) opens the index file (indexFilename). 
//		(5) reads data from index file.
//		(6) compares data.version with the parameter version.
//      (7) compares data.filename with filename (constructor)
//		(8) return true, if all operations were sucessful and version fits.	


template<class T>
class ReadMocaData
{
public:

	ReadMocaData (const string &pathFilename, const string &indexFilename, bool createNewMocaPath=false) 
	{
		this->pathFilename = pathFilename;
		this->indexFilename = indexFilename;
		destinationPath = "";
		this->createNewMocaPath=createNewMocaPath;

		// (1) call CreateMocaPath and calculates the path of the index file (pathIndexFile).
		// (2) creates the directory pathIndexFile (if it does not exist).
		CreateMocaPath mocaPath (pathFilename,false);
		destinationPath = mocaPath.destinationPath;
		filename = mocaPath.file;
	} // ReadMocaData


	bool process (const string &version, T &data);
	
	void setName (string &indexFile, string &pathFile) {
		pathFilename = pathFile;
		indexFilename = indexFile;

		CreateMocaPath mocaPath (pathFilename, false);
		destinationPath = mocaPath.destinationPath;
		filename = mocaPath.file;

		// replace '\\' with '/'
		string::size_type p;	
		for (p=0; p<indexFilename.size(); p++) {
			if (indexFilename[p] == '\\') {
				indexFilename[p] = '/';
			} // if
		} // for

		p = indexFilename.find_last_of ("/");
		if (p == string::npos) {
			destinationPath ="./";
		} else {
			destinationPath.assign(indexFilename, 0, p+1);
			indexFilename.assign(indexFilename, p+1, string::npos);
		} // else
		return;
	}

	// path, where data is stored
	string destinationPath;

private:
	// mpg filename with path
	string pathFilename;

	// mpg filename without path
	string filename;

	// name of the index file (without path)
	string indexFilename;

	// create new MOCA directory (default=no)
	bool createNewMocaPath;

};

#include <ReadMocaData.cpt>

#endif




