/*
    VideoDecoderBase.cpp     abstract class to decode videos
    Copyright (C) 2005 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <VideoDecoderBase.h>


	// constuctor defines filename 
VideoDecoderBase::VideoDecoderBase ()
{
	pathFilename = "";
	is = NULL;

	imgSize[DIM_X]=0;
	imgSize[DIM_Y]=0;
	imgSize[DIM_Z]=1;
	imgSize[DIM_BAND]=3;

	size[DIM_X]=0;
	size[DIM_Y]=0;
	size[DIM_Z]=0;
	size[DIM_BAND]=3;
}

VideoDecoderBase::VideoDecoderBase (const string &pathFile)
{
	pathFilename = pathFile;
	is = NULL;

	imgSize[DIM_X]=0;
	imgSize[DIM_Y]=0;
	imgSize[DIM_Z]=1;
	imgSize[DIM_BAND]=3;

	size[DIM_X]=0;
	size[DIM_Y]=0;
	size[DIM_Z]=0;
	size[DIM_BAND]=3;
}


// destructor
VideoDecoderBase::~VideoDecoderBase ()
{
	close();
}



// close file handler
void
VideoDecoderBase::close ()
{
	if (is!=NULL) {
		delete is;
	} // if
	is=NULL;
	return;
}


// open file handler
void
VideoDecoderBase::open ()
{
	close();
    is = new ifstream (pathFilename.c_str(), std::ios::in | std::ios::binary);
    if (is==NULL || !(*is).is_open() || !(*is).good()) {
		cerr << "Error in VideoDecoderBase: cannot open file " << pathFilename << endl << endl;
		exit(1);
    } // if
	
	return;
}



/*!
  read next character 
 */
char
VideoDecoderBase::getChar(istream *is) throw (MocaException)
{
	char data=0;
	is->read(&data, 1);
	if (is->gcount() != 1) {
		throw MocaException("Not enough pixels in image.");
	} // if
	return data;
}



/*!
  skip whitespace
 */
void
VideoDecoderBase::rmWhiteSpace(istream *is) throw (MocaException)
{
	const char blank= 0x20;
	const char tab  = 0x09;
	const char CR   = 0x0d;
	const char LF   = 0x0a;

	char data=is->peek();
	bool isWs=false;
	if ((data == blank) || (data == tab) || (data == CR) || (data == LF)) isWs = true;

	while (isWs) {

		// read ws
		is->read(&data, 1);
		if (is->gcount() != 1) {
			throw MocaException("Not enough pixels in image.");
		} // if

		// peek next character
		data=is->peek();
		isWs=false;
		if ((data == blank) || (data == tab) || (data == CR) || (data == LF)) isWs = true;
	} // while

	return;
}

/*!
  read integer value 
 */
int32
VideoDecoderBase::getInt(istream *is) throw (MocaException)
{
	rmWhiteSpace (is);
	int v=0;

	char data=is->peek();
	bool isNumber=false;
	if ((data >= 0x30) && (data <= 0x39)) isNumber = true;

	while (isNumber) {

		// read number
		is->read(&data, 1);
		if (is->gcount() != 1) {
			throw MocaException("Not enough pixels in image.");
		} // if

		int tmp = data - 0x30;
		v = 10*v + tmp;

		// peek next character
		data=is->peek();
		isNumber=false;
		if ((data >= 0x30) && (data <= 0x39)) isNumber = true;
	} // while
	return v;
}


