/*
   This file is part of the MoCA video library.
   Name: VideoDecoderBase.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

  VideoDecoderBase.h -- Abstract class to decode videos
                        derived classes: Avi, Yuv, Ppm, Pgm
 
  Author: Stephan Kopf
  Lehrstuhl fuer Praktische Informatik IV
  Universitaet Mannheim, Germany
  email: kopf@pi4.informatik.uni-mannheim.de
  

*/

#ifndef VIDEO_DECODER_BASE_H
#define VIDEO_DECODER_BASE_H

// C++ includes
#include <string>
using namespace std;

// MoCA includes
#include <Image.h>
#include <Video.h>


// this class provides the basic routines to decode a video
class VideoDecoderBase
{
protected:

	// constuctor defines filename 
	VideoDecoderBase ();
	VideoDecoderBase (const string &pathFile);

	// destructor
	~VideoDecoderBase ();
	

	// abstract classes
	virtual void getImage (int32 frame, Image<uint8> &img) = 0;
	virtual void getVideo (Video<uint8> &vid) = 0;
	virtual void getVideo (ImageVector<uint8> &vid) = 0;


	VideoSize getSize()
	{
		return size;
	} // getSize
	
	ImageSize getImageSize()
	{
		return imgSize;
	} // getImageSize


protected:
	virtual void init () = 0;

	void open ();
	void close ();
	char getChar(istream *is) throw (MocaException);
	void rmWhiteSpace(istream *is) throw (MocaException);
	int32 getInt(istream *is) throw (MocaException);


protected:

	// file name and index
	string pathFilename;

	ImageSize imgSize;
	VideoSize size;

	// file handler
	ifstream *is;
};


#endif
