/*
   This file is part of the MoCA video library.
   Name: WriteMocaData.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Stephan Kopf

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kopf@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt


*/

#ifndef WRITE_MOCA_DATA_H
#define WRITE_MOCA_DATA_H

// C++ includes
#include <string>
#include <fstream>
#include <iostream>
using namespace std;

// MoCA includes
#include <MocaTypes.h>
#include <OTagStream.h>


// Constructor sets filename of video (or image) and the filename of the index file (indexFilename).
// Example:
//		WriteMocaData ("/opt/Test.mpg", "status.idx");
//
// The function void process(T &data) 
//		(1) call CreateMocaPath and calculates the path of the index file (pathIndexFile).
//		(2) creates the directory pathIndexFile (if it does not exist).
//		(3) stores the data in the indexFilename.

template<class T>
class WriteMocaData
{
public:

	WriteMocaData (const string &pathFn, const string &indexFn) 
	{
		indexFilename = indexFn;		

		// remove leading or ending '"'
		string tmp=pathFn;
		while (tmp[0] == '"') tmp = tmp.substr(1);
		while (tmp[tmp.size()-1] == '"') tmp = tmp.substr(0,tmp.size()-1);
		pathFilename = tmp;


		// (1) call CreateMocaPath and set the path of the index file (pathIndexFile).
		// (2) creates the directory pathIndexFile (if it does not exist).
		CreateMocaPath mocaPath (pathFilename, false);
		indexPath = mocaPath.destinationPath;

	} // WriteMocaData


	void setName (string &indexFile, string &pathFile) 
	{
		pathFilename = pathFile;
		indexFilename = indexFile;		

		CreateMocaPath mocaPath (indexFilename, false);
		indexPath = mocaPath.path;
		indexFilename = mocaPath.file;
	}

	void process (T &data);

private:
	string pathFilename;
	string indexFilename;
	string indexPath;

};

#include <WriteMocaData.cpt>

#endif




