/*
   This file is part of the MoCA video library.
   Name: XmlFormat.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Stephan Kopf

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kopf@pi4.informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

*/

#ifndef XML_FORMAT_H
#define XML_FORMAT_H


// C++ includes
#include <list>
#include <string>
#include <fstream>
#include <iomanip>
#include <iostream>
using namespace std;

// MoCA includes
#include <FileHandling.h>
#include <MocaTypes.h>
#include <MocaExceptions.h>

struct XmlItem
{
	XmlItem () {
		tag  = "";
		data = "";
		status = 0;
	}

    XmlItem& operator=(const XmlItem &other) {
		if (this != &other) {
			tag      = other.tag;
			data     = other.data;
			status   = other.status;
		} // if
        return *this;
    } // operator=

	string tag;
	string data;
	int8 status;  // -1: unknown, 0 : data, 1: startTag, 2 : endTag
};

std::ostream& operator<< (ostream &, const XmlItem&);
std::ostream& operator<< (ostream &, const list<XmlItem>&);


class XmlFormat
{
public:

    //! open xml file
	XmlFormat(const string &file, bool compareUpperCase=true);

    bool isXmlFormat()
	{
		return xmlValidFormat;
	}
    void process (list<XmlItem> &xmlData);
    void findData (string &tagName, list<XmlItem> &xmlData, XmlItem &data);

private:
	bool openFile(ifstream &ifs);
	void createList (ifstream &ifs, list<XmlItem> &xmlData);
	void aggregateList (list<XmlItem> &xmlData);
	void analyseFile();


	list<XmlItem> xmlData;
	bool xmlValidFormat;
	bool compareUpperCase;
	string filename;
};


#endif




