/*
    ImagePosition.cpp     handle position in images
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <ImagePosition.h>


std::ostream& operator<< (ostream& os, const ImagePosition &ip) 
{
	os << ip[DIM_X] << "\t" << ip[DIM_Y] << "\t" << ip[DIM_BAND] << endl;
	return os;
} // operator<<

OTagStream& operator<< (OTagStream& os, const ImagePosition& ip)
{
    TagFormat::writeNodeValue(os, "DIM_X", ip[DIM_X]);  
    TagFormat::writeNodeValue(os, "DIM_Y", ip[DIM_Y]);  
    TagFormat::writeNodeValue(os, "DIM_BAND", ip[DIM_BAND]);  
	os << endl;
    return os;
} // operator<<

ITagStream& operator>> (ITagStream& is, ImagePosition& ip)
{
	TagFormat::readNodeValue(is, "DIM_X", ip[DIM_X]);  
	TagFormat::readNodeValue(is, "DIM_Y", ip[DIM_Y]);  
	TagFormat::readNodeValue(is, "DIM_BAND", ip[DIM_BAND]);  
    return is;
} // operator>>



std::ostream& operator<< (ostream& os, const list<ImagePosition> &ip) 
{
	list<ImagePosition>::const_iterator it;
	os << "size: " << ip.size() << endl;
	for (it=ip.begin(); it != ip.end(); it++) {
		os << (*it)[DIM_X] << "\t" << (*it)[DIM_Y] << "\t" << (*it)[DIM_BAND] << endl;
	} // for
	return os;
} // operator<<

OTagStream& operator<< (OTagStream& os, const list<ImagePosition>& ip)
{
	list<ImagePosition>::const_iterator it;
	int32 size = (int32) ip.size();
    TagFormat::writeNodeValue(os, "elements", size);  
	os << endl;
	for (it=ip.begin(); it != ip.end(); it++) {
	    TagFormat::writeNodeValue(os, "DIM_X", (*it)[DIM_X]);  
		TagFormat::writeNodeValue(os, "DIM_Y", (*it)[DIM_Y]);  
		TagFormat::writeNodeValue(os, "DIM_BAND", (*it)[DIM_BAND]);  
		os << endl;
	} // for
    return os;
} // operator<<

ITagStream& operator>> (ITagStream& is, list<ImagePosition>& ip)
{
	ip.clear();
	ImagePosition p;

    char c;
    bool failFlag = false;

	int32 size;
    if (is >> c) {
        is.putback(c);
		TagFormat::readNodeValue(is, "elements", size);  		
	} // if

    while (is >> c) {
        is.putback(c);

		TagFormat::readNodeValue(is, "DIM_X", p[DIM_X]);  
		TagFormat::readNodeValue(is, "DIM_Y", p[DIM_Y]);  
		TagFormat::readNodeValue(is, "DIM_BAND", p[DIM_BAND]);  
	    if (is.rdstate() & ios::failbit) {
            failFlag = true;
		} else {
            ip.push_back (p);
		} // else
	} // while
    if (!failFlag) {
        is.clear(is.rdstate() & ~ios::failbit);
    } // if
    return is;
} // operator>>

