/*
   This file is part of the MoCA video library.
   Name: SubVideo.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef SUB_VIDEO_H
#define SUB_VIDEO_H

// MoCA includes
#include <Video.h>

template<class T> class SubVideo : public Video<T>
{
public:
  SubVideo (const Video<T>&, const VideoPosition&, const VideoSize&);

protected:
  VideoPosition position;
};

template<class T>
SubVideo<T>::
SubVideo (const Video<T>& parent, const VideoPosition& position,
          const VideoSize& size)
{
  this->size        = size;
  this->capacity    = size;
  this->position    = position;
  this->tileSize    = parent.tileSize;
  this->borderMode  = parent.borderMode;
  this->data        = NULL;

  sampleIndexX = &(parent.sampleIndexX[position[DIM_X]]);
  sampleIndexY = sampleIndex[DIM_Y] = &(parent.sampleIndex[DIM_Y][position[DIM_Y]]);
  sampleIndexZ = sampleIndex[DIM_Z] = &(parent.sampleIndex[DIM_Z][position[DIM_Z]]);
  sampleIndexBand
    = sampleIndex[DIM_BAND] = &(parent.sampleIndex[DIM_BAND][position[DIM_BAND]]);

  sampleOffsetX = sampleOffset[DIM_X] = &(parent.sampleOffset[DIM_X][position[DIM_X]]);
  sampleOffsetY = sampleOffset[DIM_Y] = &(parent.sampleOffset[DIM_Y][position[DIM_Y]]);
  sampleOffsetZ = sampleOffset[DIM_Z] = &(parent.sampleOffset[DIM_Z][position[DIM_Z]]);
  sampleOffsetBand =
    sampleOffset[DIM_BAND] = &(parent.sampleOffset[DIM_BAND][position[DIM_BAND]]);


  SampleBlockSize tilePosition;
  tilePosition[DIM_X]    = position[DIM_X]/tileSize[DIM_X];
  tilePosition[DIM_Y]    = position[DIM_Y]/tileSize[DIM_Y];
  tilePosition[DIM_Z]    = position[DIM_Z]/tileSize[DIM_Z];
  tilePosition[DIM_BAND] = position[DIM_BAND]/tileSize[DIM_BAND];

  tileIndexX = &(parent.tileIndexX[tilePosition[DIM_X]]);
  tileIndexY = tileIndex[DIM_Y]     = &(parent.tileIndex[DIM_Y][tilePosition[DIM_Y]]);
  tileIndexZ = tileIndex[DIM_Z]     = &(parent.tileIndex[DIM_Z][tilePosition[DIM_Z]]);
  tileIndexBand
    = tileIndex[DIM_BAND]  = &(parent.tileIndex[DIM_BAND][tilePosition[DIM_BAND]]);

  tileOffsetX = tileOffset[DIM_X] = parent.tileOffset[DIM_X];
  tileOffsetY = tileOffset[DIM_Y] = parent.tileOffset[DIM_Y];
  tileOffsetZ = tileOffset[DIM_Z] = parent.tileOffset[DIM_Z];
  tileOffsetBand = tileOffset[DIM_BAND] = parent.tileOffset[DIM_BAND];
}

#endif
