/*
   This file is part of the MoCA video library.
   Name: Matrix.h
   Copyright (C) 2004 Gerald Kuehne

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kuehne@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

  simple class for a matrix

   \author    Gerald Kuehne

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kuehne@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: Matrix.h,v 1.1 2004/02/17 12:01:05 kopf Exp $

*/

#ifndef MATRIX_H
#define MATRIX_H

// MoCA includes
#include <MocaTypes.h>
#include <MocaError.h>
#include <MocaExceptions.h>
#include <MatrixSize.h>

/*!
  \brief Simple matrix class

  This is a simple matrix class for storing arbitrary data elements.
  The reason why yet another matrix class is needed is, that we need
  flexible indexing (starting from 0 or starting from 1) to easily
  integrate code from "Numerical recipes". The different index starts
  are realized by manipulating the data pointers.
*/

template<class T>
class Matrix
{
public:
    //! The constructor
    Matrix(const MatrixSize&, indexType = 0);
    //! The copy constructor
    Matrix(const Matrix&);
    //! The destructor
    ~Matrix();

    //! Assignment
    Matrix& operator=(const Matrix&);
    //! Element access
    T* operator[](indexType);
    //! Element access for const matrix
    const T* operator[](indexType) const;

    //! Get the size of the matrix
    MatrixSize getSize() const;
    //! Get the index start (0 or 1)
    indexType getIndexStart() const;
    //! Set the index start (0 or 1)
    void setIndexStart(indexType);

private:
    //! Initialise the matrix
    void initialise(const MatrixSize&, indexType);
    //! Remove the matrix storage
    void finalise();

    //! The size of the matrix
    MatrixSize size;
    //! The index start (0 or 1)
    indexType indexStart;
    //! The data pointer
    T** data;
};

#include <Matrix.cpt>

#endif

