/*
   This file is part of the MoCA video library.
   Name: MatrixSize.h
   Copyright (C) 2004 Gerald Kuehne

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kuehne@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

   class for storing matrix dimensions

   \author    Gerald Kuehne

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kuehne@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: MatrixSize.h,v 1.1 2004/02/17 12:01:05 kopf Exp $

*/

#ifndef MATRIX_SIZE_H
#define MATRIX_SIZE_H

#include <MocaTypes.h>

enum MatrixDimensions { DIM_ROW = 0, DIM_COL = 1 };

class MatrixSize
{
public:
    MatrixSize();
    MatrixSize(sizeType x, sizeType y);

    sizeType& operator[](sizeType i);
    const sizeType& operator[](sizeType i) const;

    int operator== (const MatrixSize&) const;
    int operator!= (const MatrixSize&) const;

private:
    sizeType size[2];
};

inline
MatrixSize::
MatrixSize ()
{
  size[DIM_ROW]    = 0;
  size[DIM_COL]    = 0;
}

inline
MatrixSize::
MatrixSize (sizeType x, sizeType y)
{
  size[DIM_ROW]    = x;
  size[DIM_COL]    = y;
}

inline
sizeType&
MatrixSize::
operator[] (sizeType i)
{
    return size[i];
}

inline
const sizeType&
MatrixSize::
operator[] (sizeType i) const
{
  return size[i];
}

inline
int
MatrixSize::operator== (const MatrixSize& other) const
{
  return(size[DIM_ROW] == other.size[DIM_ROW] &&
         size[DIM_COL] == other.size[DIM_COL]);
}

inline
int
MatrixSize::operator!= (const MatrixSize& other) const
{
  return !(*this == other);
}
#endif


