/*
   This file is part of the MoCA video library.
   Name: Vector.h
   Copyright (C) 2004 Gerald Kuehne

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kuehne@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

  simple class for vectors

   \author    Gerald Kuehne

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kuehne@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: Vector.h,v 1.1 2004/02/17 12:01:05 kopf Exp $

*/

#ifndef VECTOR_H
#define VECTOR_H

// MoCA includes
#include <MocaTypes.h>
#include <MocaError.h>
#include <MocaExceptions.h>

/*!
  \brief Simple vector class

  This is a simple vector class for storing arbitrary data elements.
  The reason why yet another vector class besides STL's valarray is
  needed is, that we need flexible indexing (starting from 0 or
  starting from 1) to easily integrate code from "Numerical
  recipes". The different index starts are realized by manipulating
  the data pointer.
*/

template<class T>
class Vector
{
public:
    //! The constructor
    Vector(sizeType, indexType = 0);
    //! The copy constructor
    Vector(const Vector&);
    //! The destructor
    ~Vector();

    //! Assignment
    Vector& operator=(const Vector&);
    //! Element access
    T& operator[](indexType);

    //! Get the size of the vector
    sizeType getSize() const;
    //! Get the index start (0 or 1)
    indexType getIndexStart() const;
    //! Set the index start (0 or 1)
    void setIndexStart(indexType);

private:
    //! Initialise the vector
    void initialise(sizeType, indexType);
    //! Remove the vector storage
    void finalise();

    //! The size of the vector
    sizeType size;
    //! The index start (0 or 1)
    indexType indexStart;
    //! The data pointer
    T* data;
};

#include <Vector.cpt>

#endif

