/*
   This file is part of the MoCA video library.
   Name: MocaTypes.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef TYPES_H
#define TYPES_H

// check if MSVC or GNU compiler is used
#undef COMPILER_MSVC
#if defined(_WIN32) 
#	if defined(_MSC_VER) 
#		define COMPILER_MSVC
#	endif
#endif


#undef COMPILER_GCC
#if defined(__GNUC__) 
#define COMPILER_GNU
#endif

 
// include config file (fixed file for MSVC or generated by configure)
#ifdef COMPILER_MSVC
#   include <MocaConfigWin32.h>
#else
#   ifdef COMPILER_GNU
#      include <MocaConfig.h>
#   else
#      error "Unknown compiler, please add identification of compiler in MocaTypes.h."
#   endif
#endif


// check header files
#ifdef HAVE_LIMITS_H
#include <limits.h>
#else
#error "No limits.h available."
#endif


// check libraries
#ifndef HAVE_LIBGSL
#error "No libgsl available."
#endif
#ifndef HAVE_LIBGSLCBLAS
#error "No libgsl available."
#endif


// determine appropriate unsigned data types

#if SIZEOF_UNSIGNED_CHAR == 1
typedef unsigned char uint8;    /*  8bit */
#define MOCA_UINT8_MAX UCHAR_MAX
#else
#error "Could not determine uint8 data type."
#endif

#if SIZEOF_UNSIGNED_SHORT_INT == 2
typedef unsigned short int  uint16; /* 16bit */
#define MOCA_UINT16_MAX USHRT_MAX
#elif SIZEOF_UNSIGNED_INT == 2
typedef unsigned int  uint16;   /* 16bit */
#define MOCA_UINT16_MAX UINT_MAX
#else
#error "Could not determine uint16 data type."
#endif

#if SIZEOF_UNSIGNED_LONG_INT == 4
typedef unsigned long int   uint32;  /* 32bit */
#define MOCA_UINT32_MAX ULONG_MAX
#elif SIZEOF_UNSIGNED_INT == 4
typedef unsigned int  uint32;   /* 32bit */
#define MOCA_UINT32_MAX UINT_MAX
#else
#error "Could not determine uint32 data type."
#endif




#if SIZEOF_UNSIGNED_LONG_LONG_INT == 8
#if defined(_MSC_VER) 

	typedef unsigned __int64 uint64;  /* 64 bit */

	#define MOCA_UINT64_MAX  0xffffffffffffffffui64

//	#define UINT64_CONST (c) c ## ui64  /* constant unsigned 64 bit (ui64) */

#else

	typedef unsigned long long uint64;  /* 64 bit */

	#define MOCA_UINT64_MAX  0xffffffffffffffffULL

//	#define UINT64_CONST (c) c ## ULL	    /* constant unsigned 64 bit (ui64) */

#endif 

//#define MOCA_UINT64_MAX  UINT64_CONST(0xffffffffffffffff)
#else
#error "Could not determine uint64 data type."
#endif





// determine appropriate signed data types

#if SIZEOF_CHAR == 1
typedef char int8;              /*  8bit */
#define MOCA_INT8_MAX CHAR_MAX
#define MOCA_INT8_MIN CHAR_MIN
#else
#error "Could not determine int8 data type."
#endif

#if SIZEOF_SHORT_INT == 2
typedef short int int16;        /* 16bit */
#define MOCA_INT16_MAX SHRT_MAX
#define MOCA_INT16_MIN SHRT_MIN
#elif SIZEOF_INT == 2
typedef int int16;              /* 16bit */
#define MOCA_INT16_MAX INT_MAX
#define MOCA_INT16_MIN INT_MIN
#else
#error "Could not determine int16 data type."
#endif

#if SIZEOF_LONG_INT == 4
typedef long int int32;         /* 32bit */
#define MOCA_INT32_MAX LONG_MAX
#define MOCA_INT32_MIN LONG_MIN
#elif SIZEOF_INT == 4
typedef int int32;              /* 32bit */
#define MOCA_INT32_MAX INT_MAX
#define MOCA_INT32_MIN INT_MIN
#else
#error "Could not determine int32 data type."
#endif

#if SIZEOF_LONG_LONG_INT == 8

#if defined(_MSC_VER) 

	typedef signed __int64 int64;   /* 64bit */

	#define MOCA_INT64_MAX   9223372036854775807i64

	#define MOCA_INT64_MIN  -9223372036854775807i64

//	#define INT64_CONST (c)	c##i64	/* constant signed 64 bit (i64) */

#else

	typedef signed long long int64; /* 64bit */

	#define MOCA_INT64_MAX   9223372036854775807LL

	#define MOCA_INT64_MIN  -9223372036854775807LL

//	#define INT64_CONST (c)	c##LL	/* constant signed 64 bit (i64) */

#endif 

//#define MOCA_INT64_MAX  INT64_CONST(9223372036854775807)
//#define MOCA_INT64_MIN  INT64_CONST(-9223372036854775807)
#else
#error "Could not determine int64 data type."
#endif




// determine appropriate data types for floats

//! \TODO hack used for MOCA_IEEE32_MAX / _MIN, took values from solaris limits.h

#if SIZEOF_FLOAT == 4
typedef float ieee32;
#define MOCA_IEEE32_MAX  3.40282347e+38F
#define MOCA_IEEE32_MIN -3.40282347e+38F
#else
#error "Could not determine ieee32 data type."
#endif


//! \TODO hack used for MOCA_IEEE64_MAX / _MIN, took values from solaris limits.h
#if SIZEOF_DOUBLE == 8
typedef double ieee64;
#define MOCA_IEEE64_MAX  1.7976931348623157E+308 
#define MOCA_IEEE64_MIN -1.7976931348623157E+308 
#else
#error "Could not determine ieee64 data type."
#endif

typedef int32               indexType;
typedef int32               offsetType;
typedef int32               enumType;
typedef int32               ptrdiffType;
typedef int32               sizeType;

#endif







